/***************************************************************************
                          satcal.cpp  -  description
                             -------------------
    copyright            : (C) 2013 by Holtkamp
 Open Source software. See header of main.cpp for details.
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <QtGui>
#include <QtNetwork>

#include "satcal.h"
#include "satccom.h"
#include "astrolib.h"

#include "stquickdlg.h"

//#include <QLocale>

//using namespace Qt;

int fntsz = 10;  // display font size
int picsize = 0;  // 0 normal, 2 50%, 3 25%
QString APPDIR;   // absolute path of the application directory
QString CFDIR;   // current picture directory 
QString TLEname;  // name of the current TLE file;

SatcalApp::SatcalApp()
{
  APPDIR = QDir::currentPath();
  CFDIR = QDir::homePath();

  QLocale::setDefault(QLocale::English);

  setWindowTitle(tr("SatCal - 11.8"));
  setWindowIcon(QPixmap(":/satcal32.png"));

  http = new QHttp (this);

  initActions();
  initMenuBar();
  initToolBar();
  initStatusBar();

  resize(837, 560);

  initView();
  setFocus();

  slotViewStatusBar(false);
  slotViewLeft(view->getLeft());

  TLEname = view->getTLEname();
  if (TLEname.isEmpty()) QDir::setCurrent(QDir::homePath());

  fntsz = view->getFsz();
  if (fntsz == 8) resize(744,560);
  
  timer = new QTimer(this);
  connect( timer, SIGNAL(timeout()), SLOT(slotTmCheck()) );
  timon = 0;
  wwwon = 0;
  wwwsave = false;
}

SatcalApp::~SatcalApp()
{
}

/** initializes all QActions of the application */
void SatcalApp::initActions(){

  QPixmap openIcon, saveIcon, selectIcon, nextIcon, dateIcon, locationIcon;
  QPixmap aosIcon, iridiumIcon, findIcon, editIcon, rtimIcon, deltaIcon, passIcon;
  QPixmap visbIcon, switchIcon, stepIcon, mapIcon, refreshIcon;

  openIcon = QPixmap(":/fileopen.png");
  saveIcon = QPixmap(":/filesave.png");
  selectIcon = QPixmap(":/satpsels.png");
  nextIcon = QPixmap(":/satpnx4p.png");
  dateIcon = QPixmap(":/satpdate.png");
  locationIcon = QPixmap(":/satploc.png");
  aosIcon = QPixmap(":/satppaos.png");
  iridiumIcon = QPixmap(":/satpirid.png");
  findIcon = QPixmap(":/satpfind.png");
  editIcon = QPixmap(":/satpxorb.png");
  rtimIcon = QPixmap(":/satprtim.png");
  deltaIcon = QPixmap(":/satpdltm.png");
  passIcon = QPixmap(":/satppass.png");
  visbIcon = QPixmap(":/satpvisb.png");
  switchIcon = QPixmap(":/satpswss.png");
  stepIcon = QPixmap(":/satptstp.png");
  mapIcon = QPixmap(":/satpsmap.png");
  refreshIcon = QPixmap(":/satprefr.png");

  fileOpen = new QAction(QIcon(openIcon), tr("&Open TLE File..."), this);  
  fileOpen->setShortcut(tr("Ctrl+F"));
  fileOpen->setStatusTip(tr("Opens a Two Line Element file"));
  fileOpen->setWhatsThis(tr("Open TLE File\n\nOpens an existing Two Line Element file. "
        "A TLE file contains orbit elements in a particular widely used format."));
  connect(fileOpen, SIGNAL(triggered()), this, SLOT(slotFileOpen()));

  fileUpdate = new QAction(tr("Onl&ine TLE Update"), this);
  fileUpdate->setShortcut(tr("Ctrl+I"));
  fileUpdate->setStatusTip(tr("Download of TLE files"));
  fileUpdate->setWhatsThis(tr("Online TLE Update\n\nUpdates the currently selected TLE file from a specified "
        "http address. You could also download other files via this option into the current directory. "
        "You must be online when giving this command."));
  connect(fileUpdate, SIGNAL(triggered()), this, SLOT(slotFileUpdate()));

  fileSave = new QAction(QIcon(saveIcon), tr("&Save"), this);
  fileSave->setStatusTip(tr("Saves the result of the AOS calculation"));
  fileSave->setWhatsThis(tr("Save File.\n\nSaves the result of the most recent visiblity calculation on file."
     "This can be AOS times or Sun transits or Iridium flares etc. whatever is just displayed on the screen."
     "Consecutive saves to the same file will be appended."));
  connect(fileSave, SIGNAL(triggered()), this, SLOT(slotFileSave()));

  fileSaveAs = new QAction(tr("Save &as..."), this);
  fileSaveAs->setStatusTip(tr("Saves the result of the AOS calculations under a new filename"));
  fileSaveAs->setWhatsThis(tr("Save As\n\nCreates a new file and saves the most recent visibility "
      "calcultion under this new filename."));
  connect(fileSaveAs, SIGNAL(triggered()), this, SLOT(slotFileSaveAs()));

  fileTLESave = new QAction(tr("Save &TLE"), this);
  fileTLESave->setStatusTip(tr("Saves the currently used orbit elements."));
  fileTLESave->setWhatsThis(tr("Save Orbit Elements.\n\nSaves the orbit elements of the currently selected "
      "satellites in the form of TLE (Two Line Elements) to the currently selected orbit element save file."
      " Former entries in this file will be overwritten."));
  connect(fileTLESave, SIGNAL(triggered()), this, SLOT(slotFileTLESave()));

  fileTLESaveAs = new QAction(tr("Save T&LE as..."), this);
  fileTLESaveAs->setStatusTip(tr("Saves the currently used orbit elements under a new filename"));
  fileTLESaveAs->setWhatsThis(tr("Save As\n\nCreates a new file and saves the orbit elements of the "
      "currently selected satellites as TLE (Two Line Elements) on this file."));
  connect(fileTLESaveAs, SIGNAL(triggered()), this, SLOT(slotFileTLESaveAs()));

  filePicture = new QAction(tr("&Picture..."), this);
//  openAct->setShortcut(tr("Ctrl+O"));
  filePicture->setWhatsThis(tr("Load Picture\n\nOpens an existing image file"));
  connect(filePicture, SIGNAL(triggered()), this, SLOT(slotFilePicture()));

  fileQuit = new QAction(tr("E&xit"), this);
  fileQuit->setShortcut(tr("Ctrl+Q"));
  fileQuit->setStatusTip(tr("Quits SatCal"));
  fileQuit->setWhatsThis(tr("Exit\n\nQuits SatCal and saves various settings."));
  connect(fileQuit, SIGNAL(triggered()), this, SLOT(slotFileQuit()));

  inputSelectSat = new QAction(QIcon(selectIcon), tr("&Select Satellite"), this);
  inputSelectSat->setShortcut(tr("Ctrl+S"));
  inputSelectSat->setStatusTip(tr("Select Satellite from TLE file."));
  inputSelectSat->setWhatsThis(tr("Select Satellite\n\nSelects one or more satellites from the currently active "
      "TLE file. Up to 10 satellites can be selected. If there are already 10 satellites in the list the last "
      "one will be overwritten by any new entries."));
  connect(inputSelectSat, SIGNAL(triggered()), this, SLOT(slotInputSelectSat()));

  inputRemoveSat = new QAction(tr("&Remove Satellite"), this);
  inputRemoveSat->setShortcut(tr("Ctrl+Z"));
  inputRemoveSat->setStatusTip(tr("Removes satellites from the list of selected satellites."));
  inputRemoveSat->setWhatsThis(tr("Remove Satellite\n\nRemoves satellites from the list of currently selected "
      "satelites."));
  connect(inputRemoveSat, SIGNAL(triggered()), this, SLOT(slotInputRemoveSat()));

  inputNextSat = new QAction(QIcon(nextIcon), tr("&Next Satellites"), this);
  inputNextSat->setShortcut(tr("Ctrl+P"));
  inputNextSat->setStatusTip(tr("Selects the next satellites which will pass the observation site."));
  inputNextSat->setWhatsThis(tr("Next Satellites\n\nSelects the next satellites which satisfy the set visibility "
      "conditions for the set observation site. Up to 10 satellites will be selected. Satellites already selected "
      "before will be overwritten."));
  connect(inputNextSat, SIGNAL(triggered()), this, SLOT(slotInputNextSat()));

  inputDate = new QAction(QIcon(dateIcon), tr("&Date"), this);
  inputDate->setShortcut(tr("Ctrl+D"));
  inputDate->setStatusTip(tr("Input date and timezone"));
  inputDate->setWhatsThis(tr("Date\n\nSelects the date and the timezone for starting the calculations."));
  connect(inputDate, SIGNAL(triggered()), this, SLOT(slotInputDate()));

  inputLocation = new QAction(QIcon(locationIcon), tr("&Location"), this);
  inputLocation->setShortcut(tr("Ctrl+C"));
  inputLocation->setStatusTip(tr("Set the observation site."));
  inputLocation->setWhatsThis(tr("Location\n\nSets the location of the observation site."
        " The coordinates can be in decimal degrees or in DDD.MMss format (Degrees.MinutesSeconds)"));
  connect(inputLocation, SIGNAL(triggered()), this, SLOT(slotInputLocation()));

  inputIers = new QAction(tr("&IERS Parameters"), this);
  inputIers->setStatusTip(tr("Set IERS Parameters"));
  inputIers->setWhatsThis(tr("IERS\n\nSet the x, y Polar Motion parameters and UT1-UTC and UTC-TAI "
      "from the International Earth Rotation Service (Bulletins B and C)."));
  connect(inputIers, SIGNAL(triggered()), this, SLOT(slotInputIers()));

  calcAOS = new QAction(QIcon(aosIcon), tr("&AOS"), this);
  calcAOS->setShortcut(tr("Ctrl+A"));
  calcAOS->setStatusTip(tr("Calculate visibility times for passes."));
  calcAOS->setWhatsThis(tr("AOS (Visibility Times)\n\nCalculates the times of visibility of the selected "
      "satellite with respect to the set visibility conditions and within the set time interval. Listed are "
      "the beginning, maximum and end of the visibility and the respective satellite positions."
      "\n\nTo get details of the passes, click the respective line."));
  connect(calcAOS, SIGNAL(triggered()), this, SLOT(slotCalcAOS()));

  calcOrbitTrack = new QAction(tr("Write &Orbit Track"), this);
  calcOrbitTrack->setShortcut(tr("Ctrl+O"));
  calcOrbitTrack->setStatusTip(tr("Write the orbit track on file."));
  calcOrbitTrack->setWhatsThis(tr("Write Orbit Track\n\nWrites the geographic coordinates of the "
      "groundtrack, flare track or transit/occultation track on the save file (usually satsave.txt)."
      "This file can then be used by other programs to plot the track."));
  connect(calcOrbitTrack, SIGNAL(triggered()), this, SLOT(slotCalcOrbitTrack()));

  calcAllSats = new QAction(tr("All Satellite &Passes"), this);
  calcAllSats->setStatusTip(tr("Calculate visibility times for passes of all satellites."));
  calcAllSats->setWhatsThis(tr("All Satellite Passes\n\nCalculates the times of visibility for all satellites "
      "in the currently selected TLE file with respect to the set visibility conditions and within the set "
      "time interval. Listed are the beginning, maximum and end of the visibility and the respective satellite "
      "positions. The output is written on file."));
  connect(calcAllSats, SIGNAL(triggered()), this, SLOT(slotCalcAllSats()));

  calcIridium = new QAction(QIcon(iridiumIcon), tr("&Iridium Flares"), this);
  calcIridium->setStatusTip(tr("Calculate Iridium Flares."));
  calcIridium->setWhatsThis(tr("Iridium Flares\n\nCalculates the times, positions and magnitudes of Iridium Flares. "
      "The TLE's of the Iridium satellites should be on the TLE file IRIDIUM.TXT. "
      "The limiting magnitude for the flare will be asked at the beginning."));
  connect(calcIridium, SIGNAL(triggered()), this, SLOT(slotCalcIridium()));

  calcMatch = new QAction(QIcon(findIcon), tr("&Find Matching Sats"), this);
  calcMatch->setStatusTip(tr("Find satellites which match a particular position and time."));
  calcMatch->setWhatsThis(tr("Find Matching Sats\n\nDisplays a list of all satellites from the currently "
      "selected TLE file which match a stated position and time interval."));
  connect(calcMatch, SIGNAL(triggered()), this, SLOT(slotCalcMatch()));

  manvEdit = new QAction(QIcon(editIcon), tr("&Edit Orbit Elements"), this);
  manvEdit->setShortcut(tr("Ctrl+E"));
  manvEdit->setStatusTip(tr("Edit orbit elements of selected satellite."));
  manvEdit->setWhatsThis(tr("Edit Orbit Elements\n\nDisplays the orbit elements of the currently selected "
      "prime satellite which can then be edited."));
  connect(manvEdit, SIGNAL(triggered()), this, SLOT(slotManvEdit()));

  manvDeltaV = new QAction(tr("Delta-&V"), this);
  manvDeltaV->setStatusTip(tr("Perform a delta-v maneuver or continuous thrust of selected satellite."));
  manvDeltaV->setWhatsThis(tr("Delta-V\n\nPerform an impulsive delta-v maneuver of given velocity change (in km/sec) "
      "and into a specified direction (yaw and pitch angles in degrees) of the selected prime satellite. "
      "If the orbit is propagated numerically, a continuous thrust option can be set."));
  connect(manvDeltaV, SIGNAL(triggered()), this, SLOT(slotManvDeltaV()));

  manvLaunch = new QAction(tr("&Launch"), this);
  manvLaunch->setShortcut(tr("Ctrl+L"));
  manvLaunch->setStatusTip(tr("Calculate a launch orbit."));
  manvLaunch->setWhatsThis(tr("Launch\n\nCalculates the orbit of a launch for which the launch site and time is given. "
      "Also given are the inclination and intended apogee/perigee (or semimajor axis and eccentricity). "
      "A list of launch sites will be provided. The launch can be in a northerly or southerly direction."));
  connect(manvLaunch, SIGNAL(triggered()), this, SLOT(slotManvLaunch()));

  manvRendz = new QAction(tr("&Rendezvous"), this);
  manvRendz->setStatusTip(tr("Rendezvous Launch Orbit."));
  manvRendz->setWhatsThis(tr("Rendezvous\n\nCalculate the orbit of a launch to rendezvous with the "
      "currently selected satellite. Only the day of launch and apogee/perigee need to be specified. "
      "The appropriate time and everything else will be calculated."));
  connect(manvRendz, SIGNAL(triggered()), this, SLOT(slotManvRendz()));

  manvOrbDet = new QAction(tr("&Orbit Determination"), this);
  manvOrbDet->setStatusTip(tr("Determine a circular orbit during a pass."));
  manvOrbDet->setWhatsThis(tr("Orbit Determination\n\nDetermine the orbit elements of a satellite for which two "
      "observations (azimuth/elevation or RA/Dec and the respective times) of the same pass and from the "
      "same site are available. A circular orbit is assumed."));
  connect(manvOrbDet, SIGNAL(triggered()), this, SLOT(slotManvOrbDet()));

  plotRT = new QAction(QIcon(rtimIcon), tr("&Real Time"), this);
  plotRT->setShortcut(tr("Ctrl+R"));
  plotRT->setStatusTip(tr("Plot Real Time positions of selected satellites."));
  plotRT->setWhatsThis(tr("Real Time\n\nDisplays the real time positions of the currently selected satellites "
      "either on a world map or on a sky map."));
  connect(plotRT, SIGNAL(triggered()), this, SLOT(slotPlotRT()));

  plotDelta = new QAction(QIcon(deltaIcon), tr("&Delta Time"), this);
  plotDelta->setShortcut(tr("Ctrl+G"));
  plotDelta->setStatusTip(tr("Plot satellite trajectory over a specified time interval."));
  plotDelta->setWhatsThis(tr("Delta Time\n\nDisplays the trajectories of the selected satellites "
      "either on a world map or on a sky map over a specified time interval in specified time steps."));
  connect(plotDelta, SIGNAL(triggered()), this, SLOT(slotPlotDelta()));

  plotCurrent = new QAction(tr("&Current Step"), this);
  plotCurrent->setShortcut(tr("Ctrl+U"));
  plotCurrent->setStatusTip(tr("Current positions of selected satellites."));
  plotCurrent->setWhatsThis(tr("Current Step\n\nDisplays the positions of the selected satellites at the currently "
      "set time either on a world map or on a sky map."));
  connect(plotCurrent, SIGNAL(triggered()), this, SLOT(slotPlotCurrent()));

  plotNext = new QAction(QIcon(passIcon), tr("&Next Pass"), this);
  plotNext->setStatusTip(tr("Jumps to the (maximum) position of the next pass."));
  plotNext->setWhatsThis(tr("Next Pass\n\nDisplays the (maximum) position of the selected prime satellite "
      "either on a world map or on a sky map during the next pass which is compatible with the currently set "
      "visibility conditions. The search only extends over the time interval specified in the date dialog." ));
  connect(plotNext, SIGNAL(triggered()), this, SLOT(slotPlotNext()));

  plotLast = new QAction(tr("&Last Pass"), this);
  plotLast->setStatusTip(tr("Jumps to the (maximum) position of the last pass."));
  plotLast->setWhatsThis(tr("Last Pass\n\nDisplays the (maximum) position of the selected prime satellite "
      "either on a world map or on a sky map during the last pass which is compatible with the currently set "
      "visibility conditions. The search only extends over the time interval specified in the date dialog." ));
  connect(plotLast, SIGNAL(triggered()), this, SLOT(slotPlotLast()));

  plotForward = new QAction(tr("Step &Forward"), this);
  plotForward->setStatusTip(tr("Moves one time step forward."));
  plotForward->setWhatsThis(tr("Step Forward\n\nMoves one time step forward on the map. "
      "You normally would use the right arrow key for this."));
  connect(plotForward, SIGNAL(triggered()), this, SLOT(slotPlotForward()));

  plotBackward = new QAction(tr("Step &Backward"), this);
  plotBackward->setStatusTip(tr("Moves one time step backward."));
  plotBackward->setWhatsThis(tr("Step Backward\n\nMoves one time step Backward on the map. "
      "You normally would use the left arrow key for this."));
  connect(plotBackward, SIGNAL(triggered()), this, SLOT(slotPlotBackward()));

  viewRefresh = new QAction(QIcon(refreshIcon), tr("&Refresh"), this);
  viewRefresh->setStatusTip(tr("Refresh the screen display"));
  viewRefresh->setWhatsThis(tr("Refresh\n\nRefreshes the map. This deletes all the satellite tracks present."));
  connect(viewRefresh, SIGNAL(triggered()), this, SLOT(slotViewRefresh()));

  viewSwitch = new QAction(QIcon(switchIcon), tr("S&witch"), this);
  viewSwitch->setShortcut(tr("Ctrl+W"));
  viewSwitch->setStatusTip(tr("Switch satellites"));
  viewSwitch->setWhatsThis(tr("Switch\n\nSelects a new prime satellite from the list of currently selected satellites. "
      "This also lets you switch to Relative Satellite Mode"));
  connect(viewSwitch, SIGNAL(triggered()), this, SLOT(slotViewSwitch()));

  viewMap = new QAction(QIcon(mapIcon), tr("Ma&p View/Sky Map"), this);
  viewMap->setShortcut(tr("Ctrl+Y"));
  viewMap->setStatusTip(tr("Specify world map or sky map parameters."));
  viewMap->setWhatsThis(tr("Map View/Sky Map\n\nSpecifies the map parameters (background color, coordinate grid). "
      "Also switches between the world groundtrack map and different sky maps."));
  connect(viewMap, SIGNAL(triggered()), this, SLOT(slotViewMap()));

  viewText = new QAction(tr("Show Last Te&xt"), this);
  viewText->setShortcut(tr("Ctrl+X"));
  viewText->setStatusTip(tr("Switch to the last text display"));
  viewText->setWhatsThis(tr("Show Last Text\n\nSwitch to the last text display."));
  connect(viewText, SIGNAL(triggered()), this, SLOT(slotViewText()));

  viewNames = new QAction(tr("Show &Names"), this);
  viewNames->setShortcut(tr("Ctrl+N"));
  viewNames->setStatusTip(tr("Show satellites names next to their respective positions."));
  viewNames->setWhatsThis(tr("Show Names\n\nShows the names of the satellites next to their respective "
      "positions on the map. In Real Time mode or Delta Time mode the names will remain on the screen "
      "until this command is repeated."));
  viewNames->setCheckable(true);
  connect(viewNames, SIGNAL(toggled(bool)), this, SLOT(slotViewNames(bool)));

  viewPicture = new QAction(tr("Show Pict&ure"), this);
  viewPicture->setStatusTip(tr("Show loaded picture."));
  viewPicture->setWhatsThis(tr("Show Picture\n\nToggles between the loaded picture and the normal maps."));
  viewPicture->setCheckable(true);
  viewPicture->setEnabled(false);
  connect(viewPicture, SIGNAL(toggled(bool)), this, SLOT(slotViewPicture(bool)));

  viewVisibility = new QAction(QIcon(visbIcon), tr("&Visibility"), this);
  viewVisibility->setShortcut(tr("Ctrl+V"));
  viewVisibility->setStatusTip(tr("Sets the visibility parameters for AOS calculations etc."));
  viewVisibility->setWhatsThis(tr("Visibility\n\nSpecifies the parameters which govern the various SatCal "
      "visibility calculations like minimum satellite elevation, maximum Sun elevation. Here you also "
      "specify the parameters for occultation and flare calculations."));
  connect(viewVisibility, SIGNAL(triggered()), this, SLOT(slotViewVisibility()));

  viewStep = new QAction(QIcon(stepIcon), tr("&Time Step"), this);
  viewStep->setShortcut(tr("Ctrl+T"));
  viewStep->setStatusTip(tr("Specify the time step used for the delta time calculations"));
  viewStep->setWhatsThis(tr("Time Step\n\nSpecifies the time step (in minutes and seconds) used for delta "
      "time calculations. Seconds can be given as integer values or as fractions (like 0.1 for 1/10th of a second."));
  connect(viewStep, SIGNAL(triggered()), this, SLOT(slotViewStep()));

  viewMagnitude = new QAction(tr("&Magnitude"), this);
  viewMagnitude->setShortcut(tr("Ctrl+M"));
  viewMagnitude->setStatusTip(tr("Specify the parameter for the apparent magnitude of the satellite."));
  viewMagnitude->setWhatsThis(tr("Magnitude\n\nSatCal displays a number of parameters for the selected prime "
      "satellite below the map. Instead of the relative speed the apparent magnitude can be displayed. To do "
      "so a parameter called Characteristic Length must be specified and different from 0. This parameter is "
      "roughly equal to the linear dimension of the satellite if the satellite were painted white. Satellites "
      "with a darker color should be specified with correspondingly smaller values. Typical values are 0.5 for "
      "small satellites, 1 or 2 for medium satellites and 15 for the Space Station or Shuttle."));
  connect(viewMagnitude, SIGNAL(triggered()), this, SLOT(slotViewMagnitude()));

  viewFontSize = new QAction(tr("Font Si&ze"), this);
  viewFontSize->setStatusTip(tr("Set the font size for the screen display"));
  viewFontSize->setWhatsThis(tr("Font Size\n\nSpecifies the font size (8 to 12) for the screen display."));
  connect(viewFontSize, SIGNAL(triggered()), this, SLOT(slotViewFontSize()));

  viewLeft = new QAction(tr("Enable &Left Button"), this);
  viewLeft->setStatusTip(tr("Enable left mouse button clicks for map and pass details."));
  viewLeft->setWhatsThis(tr("Enable Left Button\n\nTo get map and pass details the respective positions "
      "on the screen are normally marked with a right mouse click. On some systems this results in a "
      "context menu to pop up in addition to the SatCal info. Use the left mouse button in this case "
      "if you are annoyed by this. (SatCal ignores the left button if it is not enabled)."));
  viewLeft->setCheckable(true);
  connect(viewLeft, SIGNAL(toggled(bool)), this, SLOT(slotViewLeft(bool)));

  size25Act = new QAction(tr("Set Picture Size &25%"), this);
  size25Act->setEnabled(false);
  connect(size25Act, SIGNAL(triggered()), this, SLOT(size25()));

  size50Act = new QAction(tr("Set Picture Size &50%"), this);
  size50Act->setEnabled(false);
  connect(size50Act, SIGNAL(triggered()), this, SLOT(size50()));

  normalSizeAct = new QAction(tr("Norm&al Size"), this);
  normalSizeAct->setEnabled(false);
  connect(normalSizeAct, SIGNAL(triggered()), this, SLOT(normalSize()));

  viewToolBar = new QAction(tr("Tool&bar"), this);
  viewToolBar->setStatusTip(tr("Enables/disables the toolbar"));
  viewToolBar->setWhatsThis(tr("Toolbar\n\nEnables/disables the toolbar"));
  viewToolBar->setCheckable(true);
  connect(viewToolBar, SIGNAL(toggled(bool)), this, SLOT(slotViewToolBar(bool)));
  viewToolBar->setEnabled(true);

  viewStatusBar = new QAction(tr("&Statusbar"), this);
  viewStatusBar->setStatusTip(tr("Enables/disables the statusbar"));
  viewStatusBar->setWhatsThis(tr("Statusbar\n\nEnables/disables the statusbar"));
  viewStatusBar->setCheckable(true);
  connect(viewStatusBar, SIGNAL(toggled(bool)), this, SLOT(slotViewStatusBar(bool)));
  viewStatusBar-> setEnabled(true);

  settSysConf = new QAction(tr("&Configure System"), this);
  settSysConf->setStatusTip(tr("Configure System Parameters"));
  settSysConf->setWhatsThis(tr("Configure System\n\nSet various default parameters "
      "and the tolerance for the numerical integration."));
  connect(settSysConf, SIGNAL(triggered()), this, SLOT(slotSettSysConf()));

  helpAboutApp = new QAction(tr("&About..."), this);
  helpAboutApp->setStatusTip(tr("About the application"));
  helpAboutApp->setWhatsThis(tr("About\n\nAbout the application"));
  connect(helpAboutApp, SIGNAL(triggered()), this, SLOT(slotHelpAbout()));

  aboutQtAct = new QAction(tr("About &Qt"), this);
  aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
  connect (aboutQtAct, SIGNAL(activated()), qApp, SLOT(aboutQt()));

  quickHelp = new QAction(tr("&Help..."), this);
  quickHelp->setStatusTip(tr("Quick help"));
  quickHelp->setWhatsThis(tr("Help\n\nA summary help of how to use SatCal."));
  connect(quickHelp, SIGNAL(triggered()), this, SLOT(slotQuickHelp()));

  whts = QWhatsThis::createAction(this);
  whts->setStatusTip(tr("What's This?"));

  connect(http, SIGNAL(done(bool)),this,SLOT(httpDone(bool)));
}

void SatcalApp::initMenuBar()
{
  fileMenu = menuBar()->addMenu(tr("&File"));
  fileMenu->addAction(fileOpen);
  fileMenu->addAction(fileUpdate);
  fileMenu->addSeparator();
  fileMenu->addAction(fileSave);
  fileMenu->addAction(fileSaveAs);
  fileMenu->addSeparator();
  fileMenu->addAction(fileTLESave);
  fileMenu->addAction(fileTLESaveAs);
  fileMenu->addSeparator();
  fileMenu->addAction(filePicture);
  fileMenu->addSeparator();
  fileMenu->addAction(fileQuit);

  inputMenu = menuBar()->addMenu(tr("&Input"));
  inputMenu->addAction(inputSelectSat);
  inputMenu->addAction(inputRemoveSat);
  inputMenu->addAction(inputNextSat);
  inputMenu->addSeparator();
  inputMenu->addAction(inputDate);
  inputMenu->addAction(inputLocation);
  inputMenu->addSeparator();
  inputMenu->addAction(inputIers);

  calcMenu = menuBar()->addMenu(tr("&Calculate"));
  calcMenu->addAction(calcAOS);
  calcMenu->addAction(calcAllSats);
  calcMenu->addAction(calcOrbitTrack);
  calcMenu->addSeparator();
  calcMenu->addAction(calcIridium);
  calcMenu->addSeparator();
  calcMenu->addAction(calcMatch);

  manvMenu = menuBar()->addMenu(tr("&Maneuver"));
  manvMenu->addAction(manvEdit);
  manvMenu->addSeparator();
  manvMenu->addAction(manvDeltaV);
  manvMenu->addAction(manvLaunch);
  manvMenu->addAction(manvRendz);
  manvMenu->addSeparator();
  manvMenu->addAction(manvOrbDet);

  plotMenu = menuBar()->addMenu(tr("&Plot"));
  plotMenu->addAction(plotRT);
  plotMenu->addAction(plotDelta);
  plotMenu->addAction(plotCurrent);
  plotMenu->addSeparator();
  plotMenu->addAction(plotNext);
  plotMenu->addAction(plotLast);
  plotMenu->addSeparator();
  plotMenu->addAction(plotForward);
  plotMenu->addAction(plotBackward);

  viewMenu = menuBar()->addMenu(tr("&View"));
  viewMenu->addAction(viewRefresh);
  viewMenu->addAction(viewSwitch);
  viewMenu->addSeparator();
  viewMenu->addAction(viewMap);
  viewMenu->addAction(viewText);
  viewMenu->addAction(viewNames);
  viewMenu->addAction(viewPicture);
  viewMenu->addSeparator();
  viewMenu->addAction(viewVisibility);
  viewMenu->addAction(viewStep);
  viewMenu->addAction(viewMagnitude);
  viewMenu->addSeparator();
  viewMenu->addAction(viewFontSize);
  viewMenu->addAction(viewLeft);
  viewMenu->addSeparator();
  viewMenu->addAction(size25Act);
  viewMenu->addAction(size50Act);
  viewMenu->addAction(normalSizeAct);
  viewMenu->addSeparator();
  viewMenu->addAction(viewToolBar);
  viewMenu->addAction(viewStatusBar);

  settingsMenu = menuBar()->addMenu(tr("&Settings"));
  settingsMenu->addAction(settSysConf);

  helpMenu = menuBar()->addMenu(tr("&Help"));
  helpMenu->addAction(helpAboutApp);
  helpMenu->addAction(aboutQtAct);
  helpMenu->addSeparator();
  helpMenu->addAction(quickHelp);
  helpMenu->addSeparator();
  helpMenu->addAction(whts);

}

void SatcalApp::initToolBar()
{
  fileToolbar = addToolBar(tr("operations"));
  fileToolbar->addAction(fileOpen);
  fileToolbar->addAction(fileSave);
  fileToolbar->addSeparator();
  fileToolbar->addAction(inputSelectSat);
  fileToolbar->addAction(inputNextSat);
  fileToolbar->addAction(inputDate);
  fileToolbar->addAction(inputLocation);
  fileToolbar->addSeparator();
  fileToolbar->addAction(manvEdit);
  fileToolbar->addSeparator();
  fileToolbar->addAction(calcAOS);
  fileToolbar->addAction(calcIridium);
  fileToolbar->addAction(calcMatch);
  fileToolbar->addSeparator();
  fileToolbar->addAction(viewVisibility);
  fileToolbar->addAction(viewStep);
  fileToolbar->addAction(viewSwitch);
  fileToolbar->addSeparator();
  fileToolbar->addAction(plotRT);
  fileToolbar->addAction(plotDelta);
  fileToolbar->addAction(plotNext);
  fileToolbar->addSeparator();
  fileToolbar->addAction(viewMap);
  fileToolbar->addAction(viewRefresh);
  fileToolbar->addSeparator();
  fileToolbar->addSeparator();
  fileToolbar->addAction(whts);

}

void SatcalApp::initStatusBar()
{
  statusBar()->showMessage(tr("Ready."), 2000);
}

void SatcalApp::initView()
{
  // set the main widget here

 imageLabel = new QLabel;
 imageLabel->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
 imageLabel->setScaledContents(true);

 view=new SatcalView(imageLabel);

 sv = new QScrollArea();
 sv->setWidget(view);
 setCentralWidget(sv); 

}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void SatcalApp::keyPressEvent(QKeyEvent *e)
{
 int n;

 n = 0;
 setFocus();
 switch (e->key()) {
 case Qt::Key_Down:
          slotStopTime();
          view->scrollDsp(0, 10);
          break;
 case Qt::Key_Left:
          slotStopTime();
          view->scrollDsp(10, 0);
          break;
 case Qt::Key_Up:
          slotStopTime();
          view->scrollDsp(0, -10);
          break;
 case Qt::Key_Right:
          slotStopTime();
          view->scrollDsp(-10, 0);
          break;
 case Qt::Key_PageDown:
          slotStopTime();
          slotPlotNext();
          break;
 case Qt::Key_PageUp:
          slotStopTime();
          slotPlotLast();
          break;
 case Qt::Key_Space:
          view->vLocalInf();
          break;
 case Qt::Key_Delete:
          view->vRefreshP();
          break;
 case Qt::Key_Home:
          sv->horizontalScrollBar()->setValue(0); 
          sv->verticalScrollBar()->setValue(0); 
          view->goHome();
          break;
 case '1':
          n = 1;
          break;
 case '2':
          n = 2;
          break;
 case '3':
          n = 3;
          break;
 case '4':
          n = 4;
          break;
 case '5':
          n = 5;
          break;
 case '6':
          n = 6;
          break;
 case '7':
          n = 7;
          break;
 case '8':
          n = 8;
          break;
 case '9':
          n = 9;
          break;
 case '0':
          n = 10;
          break;
 case 'a':
 case 'A':
          slotCalcAOS();
          break;
 case 'c':
 case 'C':
          slotInputLocation();
          break;
 case 'd':
 case 'D':
          slotInputDate();
          break;
 case 'e':
 case 'E':
          slotManvEdit();
          break;
 case 'f':
 case 'F':
          slotFileOpen();
          break;
 case 'g':
 case 'G':
          slotPlotDelta();
          break;
 case 'h':
 case 'H':
          slotQuickHelp();
          break;
 case 'l':
 case 'L':
          slotManvLaunch();
          break;
 case 'm':
 case 'M':
          slotViewMagnitude();
          break;
 case 'n':
 case 'N':
          viewNames->toggle();
          break;
 case 'o':
 case 'O':
          slotCalcOrbitTrack();
          break;
 case 'p':
 case 'P':
          slotInputNextSat();
          break;
 case 'r':
 case 'R':
          slotPlotRT();
          break;
 case 's':
 case 'S':
          view->vSelectSat();
          break;
 case 't':
 case 'T':
          slotViewStep();
          break;
 case 'u':
 case 'U':
          slotPlotCurrent();
          break;
 case 'v':
 case 'V':
          view->vVisibility();
          break;
 case 'w':
 case 'W':
          slotViewSwitch();
          break;
 case 'x':
 case 'X':
          slotViewText();
          view->goHome();
          break;
 case 'y':
 case 'Y':
          slotViewMap();
          break;  
 case 'z':
 case 'Z': 
          slotInputRemoveSat();
          break;
  
 default: QWidget::keyPressEvent(e);
 };

 if (n > 0) view->keyPress(n);
}

void SatcalApp::slotStopTime()
{
 if (timon)
  {
   timon = 0;
   timer->stop();
   view->vTmCheck(timon);
  };
}

void SatcalApp::slotFileOpen()
{
  QString s;
  
  statusBar()->showMessage(tr("Opening TLE file..."));

  s = QFileDialog::getOpenFileName(this,"TLE file dialog",TLEname,
                         "TLE files (*.txt *.TXT *.tle *.TLE);;all files (*.*)");
  if (!s.isEmpty())
   {
    TLEname = s;
    view->setTLEname(TLEname);
   }
  else
   {
    statusBar()->showMessage(tr("*** TLE file not selected ***"));
    return;
   }; 
        
  statusBar()->showMessage(tr("Ready."));
}


void SatcalApp::slotFileSave()
{
  statusBar()->showMessage(tr("Saving AOS times..."));

  slotStopTime();
  view->vFileSave();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotFileUpdate()
{
  QString shost, spath, sfnm;
  
  statusBar()->showMessage(tr("Online TLE file update"));

  if (wwwon)
   {
    QMessageBox::warning(this,"Online Update","Another online update still in progress.");

    return;
   };

  if (view->vUpdate(shost, spath, sfnm)) return;

  wwwsave = false;
  wwwfile.setFileName(sfnm);
  if(wwwfile.open(QIODevice::ReadOnly))   // copy file to temp for safety
   {
    QStringList lines;
    QTextStream stream( &wwwfile );
    QString line;
    while ( !stream.atEnd() )
     {
      line = stream.readLine(); // line of text excluding '\n'
      lines += line;
     }
    wwwfile.close();

    wwwsvf.setFileName("satwwwsave.tmp");
    if ( wwwsvf.open( QIODevice::WriteOnly ) )
     {
        QTextStream stream( &wwwsvf );
        for ( QStringList::Iterator it = lines.begin(); it != lines.end(); ++it )
            stream << *it << "\n";
        wwwsvf.close();
        wwwsave = true;
     }
    else wwwsvf.remove();
   };

  if(!wwwfile.open(QIODevice::WriteOnly))
   {
    QMessageBox::warning(this,"HTTP Get",tr("Cannot write file %1\n%2.")
                                          .arg(wwwfile.fileName())
                                          .arg(wwwfile.errorString()));
    if (wwwsave)
     {
      wwwsvf.remove();
      wwwsave = false; 
     };
                                                                                  
    return;
   };

  wwwon = 1;
  http->setHost(shost);
  http->get(spath, &wwwfile);
  http->close();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::httpDone(bool error)
{
 if (error)
  {
   QMessageBox::warning(this,"HTTP Get",tr("Error fetching file with "
                                                "HTTP: %1.")
                                                .arg(http->errorString()));
   if (wwwsave)  // restore the origial TLE file
    {
     if(wwwsvf.open(QIODevice::ReadOnly))  
      {
       QStringList lines;
       QTextStream stream( &wwwsvf );
       QString line;
       while ( !stream.atEnd() )
        {
         line = stream.readLine(); // line of text excluding '\n'
         lines += line;
        }
       wwwsvf.close();

       QTextStream stream2( &wwwfile );
       for ( QStringList::Iterator it = lines.begin(); it != lines.end(); ++it )
            stream2 << *it << "\n";
      };
    };  
  }                                              
 else QMessageBox::information(this,"HTTP Get",tr("Download complete. File %1.").arg(wwwfile.fileName()));

 wwwon = 0;
 wwwfile.close();
 if (wwwsave)
  {
   wwwsvf.remove();
   wwwsave = false;
  };
}

void SatcalApp::slotFileSaveAs()
{
  QString s, fn;

  statusBar()->showMessage(tr("Saving under new filename..."));

  slotStopTime();

  fn = view->getSaveName();
  s = QFileDialog::getSaveFileName(this,"Select a File Name",
                    fn,"all files (*.*)");
  if (!s.isEmpty())
   {
    view->setSaveName(s);
    view->vFileSave();
   } 
  else
   {
    statusBar()->showMessage(tr("*** Save file not selected ***"));
    return;
   };

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotFileTLESave()
{
  statusBar()->showMessage(tr("Saving the TLE's on file..."));

  slotStopTime();
  view->vTLESave();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotFileTLESaveAs()
{
  QString s, fn;
  
  statusBar()->showMessage(tr("Saving the TLE's under new filename..."));

  slotStopTime();

  fn = view->getTSaveName();
  s = QFileDialog::getSaveFileName(this,"Select a TLE filename", fn,
           "TLE files (*.txt *.TXT *.tle *.TLE);;all files (*.*)");
  if (!s.isEmpty())
   {
    view->setTSaveName(s);
    view->vTLESave();
   }
  else
   {
    statusBar()->showMessage(tr("*** TLE Save File not selected ***"));
    return;
   };

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotFilePicture()
{

  statusBar()->showMessage(tr("Opening TLE file..."));
  slotStopTime();

  QString fileName = QFileDialog::getOpenFileName(this,"Open Picture",CFDIR,"Images ( *.png *.jpg *.bmp);;all files (*.*)");

  if (!fileName.isEmpty()) 
   {
    QFileInfo *qfi = new QFileInfo(fileName);
    CFDIR = qfi->absoluteFilePath();

    const QImage image (fileName);
    if (image.isNull()) 
     {
      QMessageBox::information(this, "SatCal",
                     tr("Cannot load %1.").arg(fileName));
      return;
     };

     view->vPicture(image);
   };

  viewPicture->setEnabled(true);
  viewPicture->setChecked(true);
  size25Act->setEnabled(true);
  size50Act->setEnabled(true);
  normalSizeAct->setEnabled(true);

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotFileQuit()
{
  statusBar()->showMessage(tr("Exiting application..."));

  QDir::setCurrent(APPDIR);

  slotStopTime();

  view->vQuit();
  qApp->quit();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotInputSelectSat()
{
  statusBar()->showMessage(tr("Select satellite..."));

  view->vSelectSat();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotInputRemoveSat()
{
  statusBar()->showMessage(tr("Remove satellite..."));

  view->vRemoveSat();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotInputNextSat()
{
  int res;
  
  statusBar()->showMessage(tr("Next satellites to pass..."));

  slotStopTime();

  res = view->vNextSat();
  if (res > 1) slotPlotRT();
  else
   {
    if (res < 0)
	     QMessageBox::warning(this, "SatCal", "Unable to open TLE file", 0,
         "Quit", 0, 0, 1 );
    else   
	     QMessageBox::warning(this, "SatCal", "No suitable satellites found",
          0,  "Quit", 0, 0, 1 );

    statusBar()->showMessage(tr("Failed to find next satellites."));
    return;
   };    

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotInputDate()
{
  statusBar()->showMessage(tr("Input the Date and Time..."));

  if (view->vInputDate())
   {
    statusBar()->showMessage(tr("Input cancelled."));
    return;
   };

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotInputLocation()
{
  statusBar()->showMessage(tr("Input the location..."));

  if(view->vInputLocation())
   {
    statusBar()->showMessage(tr("Input cancelled."));
    return;
   };

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotInputIers()
{
  statusBar()->showMessage(tr("Set IERS Parameters..."));

  view->vSetIERS();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotCalcAOS()
{
  statusBar()->showMessage(tr("List satellite passes..."));

  slotStopTime();
  view->vCalcAOS();
  
  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotCalcAllSats()
{
  statusBar()->showMessage(tr("List all satellite passes..."));

  slotStopTime();
  view->vAllSat();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotCalcOrbitTrack()
{
  statusBar()->showMessage(tr("List all satellite passes..."));

  slotStopTime();
  view->vOrbitTrack();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotCalcIridium()
{
  statusBar()->showMessage(tr("List Iridium Flares..."));

  slotStopTime();
  view->vIridium();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotCalcMatch()
{
  statusBar()->showMessage(tr("Find matching satellites..."));

  slotStopTime();
  view->vMatch();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotManvEdit()
{
  statusBar()->showMessage(tr("Edit orbit elements"));

  slotStopTime();
  view->vEditOrb();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotManvDeltaV()
{
  statusBar()->showMessage(tr("Delta-V maneuver"));

  slotStopTime();
  view->vDeltaV();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotManvLaunch()
{
  statusBar()->showMessage(tr("Launch orbit"));

  slotStopTime();
  view->vLaunch();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotManvRendz()
{
  statusBar()->showMessage(tr("Rendezvous Launch orbit"));

  slotStopTime();
  view->vRendz();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotManvOrbDet()
{
  statusBar()->showMessage(tr("Orbit determination"));

  slotStopTime();
  view->vOrbDet();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotPlotRT()
{
  statusBar()->showMessage(tr("Real Time plot"));

  if (timon == 0)
   {
    timon = 1;
    timer->start(980);
   }
  else
   {  
    timer->stop();
    timon = 0;
   };
  view->vTmCheck(timon); 

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotPlotDelta()
{
  statusBar()->showMessage(tr("Delta Time plot"));
  if (timon == 0)
   {
    timon = 2;
    timer->start(1);
   }
  else
   {
    timer->stop();
    timon = 0;
   };
  view->vTmCheck(timon);

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotPlotCurrent()
{
  statusBar()->showMessage(tr("Plot current position"));

  slotStopTime();
  view->vCurrent();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotPlotNext()
{
  statusBar()->showMessage(tr("Next pass"));

  slotStopTime();
  view->vPlotNext();
      
  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotPlotLast()
{
  statusBar()->showMessage(tr("Last pass"));

  slotStopTime();
  view->vPlotLast(); 

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotPlotForward()
{
  statusBar()->showMessage(tr("Step forward"));

  slotStopTime();
  view->vForward();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotPlotBackward()
{
  statusBar()->showMessage(tr("Step backward"));

  slotStopTime();
  view->vBackward();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotViewRefresh()
{
  statusBar()->showMessage(tr("Refresh screen"));

  sv->horizontalScrollBar()->setValue(0); 
  sv->verticalScrollBar()->setValue(0);  
  view->vRefreshP();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotViewSwitch()
{
  statusBar()->showMessage(tr("Switch prime satellite"));

  view->vSwitch();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotViewMap()
{
  statusBar()->showMessage(tr("Select world map or sky map parameters."));

  view->vMapDialog();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotViewText()
{
  statusBar()->showMessage(tr("Show last text display."));

  slotStopTime();
  view->vText();
  sv->horizontalScrollBar()->setValue(0);  
  sv->verticalScrollBar()->setValue(0);  
  view->goHome();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotViewNames(bool on)
{
  statusBar()->showMessage(tr("Show satellite names."));

  view->vNames(on);

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotViewPicture(bool on)
{
  statusBar()->showMessage(tr("Show loaded picture."));

  slotStopTime();
  view->vPicSwitch(on);
  sv->horizontalScrollBar()->setValue(0);  
  sv->verticalScrollBar()->setValue(0);  
  view->goHome();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotViewLeft(bool on)
{
  statusBar()->showMessage(tr("Enable left mouse button."));

  view->vLeft(on);

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotViewVisibility()
{
  statusBar()->showMessage(tr("Set visibility parameters."));

  view->vVisibility();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotViewStep()
{
  statusBar()->showMessage(tr("Change time step."));

  view->vStep();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotViewMagnitude()
{
  statusBar()->showMessage(tr("Set apparent magnitude parameter."));

  view->vMag();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotViewFontSize()
{
  bool ok;
  int res;

  statusBar()->showMessage(tr("Specify the font size..."));

  res = QInputDialog::getInteger(this,"Font Size", "Specify font size for display", fntsz, 8, 12, 2, &ok);
  if (ok)
   {
    fntsz = res;
    view->setFont(fntsz);
   };

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotViewToolBar(bool toggle)
{
  statusBar()->showMessage(tr("Toggle toolbar..."));

  if (toggle== false)
  {
    fileToolbar->hide();
  }
  else
  {
    fileToolbar->show();
  };

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotViewStatusBar(bool toggle)
{
  statusBar()->showMessage(tr("Toggle statusbar..."));

  if (toggle == false)
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotSettSysConf()
{
  statusBar()->showMessage(tr("Set system parameters."));

  view->vSysConf();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotHelpAbout()
{
  QMessageBox::about(this,tr("About..."),
                      tr("SatCal\nVersion 11.8\n(c) March 2013 by Gerhard HOLTKAMP"
                          "\n\nSatCal is free Open Source software."
                          "\nYou can redistribute it and/or modify it under the"
                          "\nterms of the GNU General Public License as published"
                          "\nby the Free Software Foundation, version 2 or later.") );
}

void SatcalApp::slotQuickHelp()
{
  QString s;
//  s = APPDIR + "/satcalHelp.txt";
  s = ":/satcalHelp.txt";
  QFile file(s);

  statusBar()->showMessage(tr("Provide general help with SatCal"));

  StQuickDialog *dlg = new StQuickDialog();

  if(!file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
     statusBar()->showMessage(tr("*** ERROR ***  unable to open Help file"));
     return;
    };

   QTextStream in(&file);

   while (!in.atEnd())
    {
     s = in.readLine();
     dlg->setHelpLine(s);
     dlg->setHelpLine("\n");
    };

  dlg->setHelpLine("$$$");
  dlg->exec();

  statusBar()->showMessage(tr("Ready."));
}

void SatcalApp::slotTmCheck()
{
  if (timon == 1) view->RT_Plot();
  if (timon == 2)
   {
    if (view->PlotDelta())
     {
      timer->stop();
      timon = 0;
      view->vTmCheck(timon);
     };
   };   
}

void SatcalApp::size25()
{
 picsize = 3;
 view->vImage(3);
}

void SatcalApp::size50()
{
 picsize = 2;
 view->vImage(2);
}

void SatcalApp::normalSize()
{
 picsize = 0;
 view->vImage(0);
}

