/* =========================================================================
  Procedures needed for numerical integration.

  Copyright :Gerhard HOLTKAMP          17-JUL-2000   19:00
  ========================================================================= */

#include <cmath>
using namespace std;

#include "simlib.h"

/* definition of the differential equations
  The function Fun calculates the right hand sides of the
				differential equations and has to be supplied by the
				user of the RKF45 class or the RK4 class
				.
  Input Parameters for Fun : x = independent variable,
											 y = array with dependent variables
  Output Parameters for Fun : z = array with the results of
													calculating the right hand sides

 */

/*-------------------- Class RKF45 ----------------------------------------
  Runge-Kutta-Fehlberg 4(5) integration

  Construction of the RKF45 class:
	 RKF45 name(nq)  where nq is the number of equations (dependent variables)

  Public Functions:
	 void ResetNeq (int nq)
		Resets the number of equations (dependent variables) to nq .
	 void SetTolerance (double vtol, int rtol)
		Sets the tolerance to vtol.
		If rtol is 0 the tolerance is interpreted as absolute tolerance.
		If rtol is 1 the tolerance is interpreted as relative tolerance.
		Default (if SetTolerance is not called) is a relative tolerance of 1e-6.

	 void SetMaxIter (int mxi)
		Sets the maximum number of iterations per step to mxi. This serves
		to limit the loops in case there is a poor behaviour of the
		automatic stepsize control.
		If mxi is 0 the iteration loop of the stepsize control continues
		until the tolerance criterion is fulfilled.
		Default (if SetMaxIter is not called) is 0.

	 int GetConOk ()
		Returns 1 if the allocation of memory needed for the construction
		of the class was successful; 0 if unsuccessful.

	 void RKF45Step(double xin, double hin, double yin[],
							 void (*Fun)(double x, double y[], double z[]),
							 double& xout, double& hout, double yout[])
		 Performs one step in the intergration of the system of
		 differential equations suppied by the user within the function Fun.
		 See below for an explanation of the parameters.


	 For the algorithm see J.M.A.Danby "Computer Modelling from Sports
	 to Spaceflight", Willmann-Bell, Richmond Virginia, 1997, chapter 3 }

  =========================================================================
  */
RKF45::RKF45 (int nq)
 {
  /* dynamically create arrays for dependent variables and derivatives
		nq is the number of equations
	*/
  neq = nq;
  y = new double[neq];
  f = new double[6*neq];
  te = new double[neq];

  // set some defaults
  constrok = 1;
  if (!y) constrok = 0;
  if (!f) constrok = 0;
  if (!te) constrok = 0;
  maxiter = 0;
  reltol = 1;
  tol = 1.0e-6;
 }

RKF45::~RKF45 ()
 {
  delete [] y;
  delete [] f;
  delete [] te;
 }

void RKF45::ResetNeq (int nq)
 {
  // Reset the number of equations and associated arrays
  if (y) delete [] y;
  if (f) delete [] f;
  if (te) delete [] te;
  neq = nq;
  y = new double[neq];
  f = new double[6*neq];
  te = new double[neq];

  constrok = 1;
  if (!y) constrok = 0;
  if (!f) constrok = 0;
  if (!te) constrok = 0;
 }

void RKF45::SetTolerance (double vtol, int rtol)
 {
  // set tolerance and relative tolerance flag
  tol = fabs(vtol);
  if (tol < 1.0e-50) tol = 1.0e-50;  // just in case
  if (rtol) reltol = 1;
  else reltol = 0;
 }

void RKF45::SetMaxIter (int mxi)
 {
  // set maximum number of iterarations
  maxiter = mxi;
  if (maxiter < 0) maxiter = -maxiter;
 }

int RKF45::GetConOk ()
 {
  // check whether construction of class went OK
  return constrok;
 }

void RKF45::RKF45Step(double xin, double hin, double yin[],
							 void (*Fun)(double x, double y[], double z[]),
							 double& xout, double& hout, double yout[])
 {
  /*
	 Calculate one step in solving a system of neq differential equations
	 with the Runge-Kutta-Fehlberg method RKF4(5).

	 INPUT PARAMETERS :
		xin : initial value of the independent variable
		hin : original stepsize
		yin : array containing the initial values of the dependent variables
		The function Fun calculates the right hand sides of the
				differential equations and has to be supplied by the
				user of the RKF45 class
				Input Parameters for Fun : x = independent variable,
											 y = array with dependent variables
				Output Parameters for Fun : z = array with the results of
													calculating the right hand sides

	 OUTPUT PARAMETERS :
		xout : new value of the independent variable
		hout : new stepsize
		yout : array containing the new values the dependent variables.

	*/
	 //  Parameters for the Runge-Kutta-Fehlberg method of order 4/5.
	 const double a[6] = { 0, 2.0/9.0, 1.0/3.0, 3.0/4.0, 1.0, 5.0/6.0 };
	 const double ch[6] = { 47.0/450.0, 0, 12.0/25.0, 32.0/225.0, 1.0/30.0, 6.0/25.0 };
	 const double ct[6] = { -1.0/150.0, 0, 3.0/100.0, -16.0/75.0, -1.0/20.0, 6.0/25.0 };
	 const double b[6][5] = { {0, 0, 0, 0, 0},
									  {2.0/9.0, 0, 0, 0, 0},
									  {1.0/12.0, 1.0/4.0, 0, 0, 0},
									  {69.0/128.0, -243.0/128.0, 135.0/64.0, 0, 0},
									  {-17.0/12.0, 27.0/4.0, -27.0/5.0, 16.0/15.0, 0},
									  {65.0/432.0, -5.0/16.0, 13.0/16.0, 4.0/27.0, 5.0/144.0} };

	double htemp, temax, yscr, tscr, rt;
	int k, l, n, i;

	// initialize step
	if (maxiter) i = maxiter;
	else i = 1;
	h = hin;
	Fun(xin, yin, &f[0]);

	// main loop
	do
	 {
	  // basic integration steps
	  for (k=1; k<6; k++)
		{
		 x = xin + a[k]*h;
		 for (n=0; n<neq; n++)
		  {
			yscr = 0;
			for (l=0; l<k; l++) yscr = yscr + b[k][l]*f[l*neq+n];
			y[n] = yin[n] + h*yscr;
		  };  // n
		  Fun(x, y, &f[neq*k]);
		}; // k

	  // summing up the new y-values and calculating the error residuals
	  for (n=0; n<neq; n++)
		{
		 yscr = 0;
		 tscr = 0;
		 for (k=0; k<6; k++)
		  {
			l = neq*k+n;
			yscr = yscr + ch[k]*f[l];
			tscr = tscr + ct[k]*f[l];
		  }; // k
		 y[n] = yin[n] + h*yscr;
		 te[n] = fabs (h*tscr);
		};  // n

	  // checking the error tolerance and calculating a new stepsize
	  temax = tol * 1.0E-15;

	  if (reltol)   // case of relative tolerance
		{
		 for (n=0; n<neq; n++)
		  {
			rt = fabs(y[n]);
			if (rt < 1.0E-50) rt = 1.0E-50;
			rt = te[n] / rt;
			if (temax < rt) temax = rt;
		  };
		}
	  else     // case of absolute tolerance
		{
		 for (n=0; n<neq; n++)
		  if (temax < te[n]) temax = te[n];
		};
	  htemp = h;
	  h = 0.9*h*pow((tol/temax),0.2);

	  if (maxiter) i--;
	 } while ((temax > tol) && (i > 0));

	// store result in output parameters
	for (n=0; n<neq; n++) yout[n] = y[n];
	xout = xin + htemp;
	hout = h;

 }

/*-------------------- Class RK4 ------------------------------------------
  Runge-Kutta integration of fourth order

  Construction of the RK4 class:
	 RK4 name(nq)  where nq is the number of equations (dependent variables)

  Public Functions:

  =========================================================================
  */

RK4::RK4 (int nq)
 {
  /* dynamically create arrays for dependent variables and derivatives
		nq is the number of equations
	*/
  neq = nq;
  y = new double[neq];
  f = new double[4*neq];
 }

RK4::~RK4 ()
 {
  delete [] y;
  delete [] f;
 }

int RK4::GetConOk ()
 {
  // check whether construction of class went OK
  return constrok;
 }

void RK4::RK4Step(double xin, double h, double yin[],
							 void (*Fun)(double x, double y[], double z[]),
							 double& xout, double yout[])
 {
  /*
	 Calculate one step in solving a system of neq differential equations
	 with the Runge-Kutta method of order 4.

	 INPUT PARAMETERS :
		xin : initial value of the independent variable
		h : stepsize
		yin : array containing the initial values of the dependent variables
		The function Fun calculates the right hand sides of the
				differential equations and has to be supplied by the
				user of the RK4 class
				Input Parameters for Fun : x = independent variable,
											 y = array with dependent variables
				Output Parameters for Fun : z = array with the results of
													calculating the right hand sides

	 OUTPUT PARAMETERS :
		xout : new value of the independent variable
		yout : array containing the new values the dependent variables.

	 For the algorithm see Paul Hellings, "Astrophpysics With a PC",
	 Willmann-Bell, Richmond Virginia, 1994, chapter 1
	*/

	// Parameters for the Runge-Kutta method of order 4.
	const double a[4] = { 0, 0.5, 0.5, 1.0 };
	const double ch[4]= { 1.0/6.0, 2.0/6.0, 2.0/6.0, 1.0/6.0 };
	const double b[4] = { 0, 0.5, 0.5, 1.0 };

	double yscr;
	int k, n;

	// initialize step
	Fun(xin, yin, &f[0]);

	// basic integration steps
	for (k=1; k<4; k++)
	 {
	  x = xin + a[k]*h;
	  for (n=0; n<neq; n++)
		 y[n] = yin[n] + b[k]*h*f[(k-1)*neq+n];
	  Fun(x, y, &f[neq*k]);
	 };  // k

	// summing up the new y-values and calculating the error residuals
	for (n=0; n<neq; n++)
	 {
	  yscr = 0;
	  for (k=0; k<4; k++)
		 yscr = yscr + ch[k]*f[neq*k+n];
	  y[n] = yin[n] + h*yscr;
	 }; // n

	// store result in output parameters
	for (n=0; n<neq; n++) yout[n] = y[n];
	xout = xin + h;

 }


