/***************************************************************************
* Perform the calculations for the SatCal program                          *
*                                                                          *
*                                                                          *
* Author: Gerhard HOLTKAMP                     26-MAR-2013
  Open Source software. See header of main.cpp for details.
***************************************************************************/
/*------------ include files and definitions -----------------------------*/
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <ctime>

using namespace std;

#include "satccom.h"
#include "satcnum.h"
#include "astrolib.h"
#include "attlib.h"
#include "sgp4unit.h"
#include "nolocal.h"

FILE *outptr;  /* Output File */

SatPosData *satpos = new SatPosData;

double GBL_timezone;  // timezone set for system
const double degrad = M_PI / 180.0;
const char cdegs = '°';  // check whether this comes out as degree sign

elsetrec satrec;  // satellite record used for SGP4 procedures

/*---------------------- Strncmpi --------------------------------------*/

int Strncmpi (char* s1, char* s2, int slen)
 {
  // compare strings like strncmp but case insensitive
  // just return 0 if equal and 1 if not equal

  int j, res, l1, l2;

  if (slen == 0) return 1;

  l1 = strlen(s1);
  l2 = strlen(s2);
  if ((l1 < slen) || (l2 < slen))
   {
    if (l1 != l2) return 1;
    if (l1 <= 0) return 1;
   };

  res = 0;
  for (j=0; j<slen; j++)
   {
    if (tolower(s1[j]) != tolower(s2[j]))
     {
      res = 1;
      break;
     };
   };

  return res;
 }

/*---------------------- function GetMonth --------------------------------*/
void GetMonth (int mm, char* mchr)
{
// mm is the number of the month
// mchr is a char[4] array to receive the three letters of the month

 switch (mm)
  {
   case 1 : strcpy(mchr,"JAN");
			  break;
   case 2 : strcpy(mchr,"FEB");
 			  break;
   case 3 : strcpy(mchr,"MAR");
			  break;
   case 4 : strcpy(mchr,"APR");
			  break;
   case 5 : strcpy(mchr,"MAY");
			  break;
   case 6 : strcpy(mchr,"JUN");
			  break;
   case 7 : strcpy(mchr,"JUL");
 			  break;
   case 8 : strcpy(mchr,"AUG");
			  break;
   case 9 : strcpy(mchr,"SEP");
			  break;
   case 10 : strcpy(mchr,"OCT");
			  break;
   case 11 : strcpy(mchr,"NOV");
			  break;
   case 12 : strcpy(mchr,"DEC");
			  break;
   default: strcpy(mchr,"ERR");
  };
}

/*----------------------- function dtmstr -----------------------------*/
void dtmstr(double jdmoon, char *dts)
  /*
    Convert the Modified Julian Date jdmoon into a corresponding string
    dts which has the format "MMM DD HH:MM"
  */
  {
   int dd, mm, yy, deg, mnt;
   double hh, sec;
   char mchr[4];

   /* convert MJD into corresponding date and time */
   caldat(jdmoon, dd, mm, yy, hh);
   dms (hh,deg,mnt,sec);
   if (sec>30.0) {mnt++;};
   if (mnt>59)
    {
     deg++;
     mnt=0;
    };

   /* store data in their positions */
   GetMonth (mm, mchr);
   dts[0]=mchr[0];
   dts[1]=mchr[1];
   dts[2]=mchr[2];
   dts[3]=' ';
   sprintf(&dts[4],"%2i %2i:%02i", dd, deg, mnt);
   dts[12]='\0';
  }

/*--------------------- Function PutNxtLn -------------------------------*/
void PutNxtLn(WoutData* outdata, char *otxt)  // copy line into buffer
 {
  int i, j;
  int maxlen = GBL_MAXCPL;

  decdot(otxt);

  i = outdata->curlines;
  if (outdata->curlines < GBL_MAXLIN)
   {
    for (j = 0; j<maxlen; j++) outdata->wbuf[i][j] = otxt[j];
 	 outdata->curlines++;
   };
 }

void PutNxtLnT(WoutData* outdata, char *otxt)  // copy line into buffer and mark for Tab
 {
  int i, j;
  int maxlen = GBL_MAXCPL;

  decdot(otxt);

  i = outdata->curlines;
  if (outdata->curlines < GBL_MAXLIN)
   {
    for (j = 0; j<maxlen; j++) outdata->wbuf[i][j] = otxt[j];
    outdata->wbuf[i][maxlen-1] = '~';
    outdata->curlines++;
   };
  }

/*--------------------- Fuction PutSpace ----------------------------------*/
void PutSpace(WoutData* outdata, int nspace = 1)  // write "nspace" space lines
 {
  char otxt[GBL_MAXCPL] = " ";

  for (int i = 0; i<nspace; i++) PutNxtLn(outdata, otxt);
 }

/*------------------------- hdfdms ---------------------------------*/

double hdfdms (double h)
 {
  // convert degrees from d.fff -> d.mm where .mm are the minutes
  int mm, deg;
  double hh, sec;

  hh = fabs(h);

  dms (hh,deg,mm,sec);
  if (sec>30.0) {mm++;};
  if (mm>59)
   {
    deg++;
    mm=0;
   };
  hh = double(deg);
  sec = double(mm)/100.0;
  hh += sec;
  if (h < 0) hh = -hh;

  return hh;
 }


/*------------------------- DegFDms ---------------------------------*/

double DegFDms (double h)
 {
  // convert degrees from d.fff -> d.mmsss where .mm are the minutes
  //  and sss are seconds (and fractions of seconds).
  int mm, deg;
  double hh, t, sec;

  hh = fabs(h);
  dms (hh,deg,mm,sec);
  if (sec>=59.5)
   {
    mm++;
    sec = 0;
   };
  if (mm>59)
   {
    deg++;
    mm=0;
   };
  hh = double(deg);
  t = double(mm)/100.0;
  hh += t;
  t = sec/10000.0;
  hh += t;
  if (h < 0) hh = -hh;

  return hh;
 }

/*------------------------- DmsDegF ---------------------------------*/

double DmsDegF (double h)
 {
  /* convert degrees from d.mmsss -> d.fff where .mm are the minutes
     and sss are seconds (and fractions of seconds).
     Returned is the angle in decimal degrees.
  */
  int mm, deg;
  double hh, t, sec, dlt;

  hh = fabs(h);
  dlt = hh*3.33e-16;
  hh += dlt;
  deg = int(hh);
  t = fmod(hh,1.0)*100.0;
  mm = int(t + 0.1e-12);
  sec = fmod(hh*100.0,1.0)*100.0;
  hh = ddd(deg,mm,sec);
  if (h < 0) hh = -hh;

  return hh;
 }

/*********************************************************************/

/*------------------------- getcfg ---------------------------------*/

int getcfg (WinpData* indata)
 {
  /* get the initial configuration from the file SATCAL.CFG
     and store the saved values for the timezone and the
     geographic location in indata.

     RETURN: 1 if reading the cfg file was successful,
             2 if file does not exist, 0 if error reading file

     NOTE: If timezone = 16 automatic system timezone will be assumed.
  */
  int fsc, fsb, k, j, vis, mgrey, mgrid, swmax;
  int mfsz, fbold, leftm, sidxcn;
  double lng, lat, ht, tz, msun, msat, yrepoch;
  double dura;
  double dF107, dFB107, dKp, dCr, dCd, dAm;
  double vard[5];
  char sname[21];
  char gname[31];
  char tlname[105];
  char wwwname[105];
  FILE* cfle;

  dFB107 = indata->dFB107;
  dF107 = indata->dF107;;
  dKp = indata->dKp;
  dCr = indata->dCr;
  dCd = indata->dCd;
  dAm = indata->dAm;
  dura = indata->duration;

  strcpy(sname, "");
  strcpy(gname, "");
  strcpy(tlname, "");
  strcpy(wwwname, "");

  swmax=0;
  tz = 16.0;

  cfle = fopen ("satcal.cfg","r");

  if (cfle != NULL)
   {
    fsc = 1;
//    fscanf(cfle,"%lf \n%lf \n%lf \n%lf\n",&tz, &lat, &lng, &ht);
//    if (ferror(cfle)) fsc = 0;
    fsc = Fscanf(cfle, 1, vard);
    if (fsc) 
     {
      tz = vard[0];
      fsc = Fscanf(cfle, 1, vard);
      if (feof(cfle)) fsc = 0;
     };
    if (fsc) 
     {
      lat = vard[0];
      fsc = Fscanf(cfle, 1, vard);
      if (feof(cfle)) fsc = 0;
     };
    if (fsc) 
     {
      lng = vard[0];
      fsc = Fscanf(cfle, 1, vard);
      if (feof(cfle)) fsc = 0;
     };
    if (fsc) 
     {
      ht = vard[0];
      if (feof(cfle)) fsc = 0;
     };
    if (fsc)
     {
      if (fgets(gname, 30, cfle) == NULL) fsc = 0;
      else
       {
        j = strlen(gname);
        if (j>0)
         {
          if(gname[j-1] == '\n') gname[j-1] = '\0';
         };
       };
      };
    if (feof(cfle)) fsc = 0;
    if (fsc) fscanf(cfle,"%d\n %d\n %d\n",&swmax, &mgrid, &mgrey);
    if (feof(cfle)) fsc = 0;
    if (fsc)
     {
      if (fgets(tlname, 104, cfle) == NULL) fsc = 0;
      else
       {
        j = strlen(tlname);
        if (j>0)
         {
          if(tlname[j-1] == '\n') tlname[j-1] = '\0';
         };
       };
     };
    if (feof(cfle)) fsc = 0;
//    if (fsc) fscanf(cfle,"\n%d \n%lf \n%lf", &vis, &msun, &msat);
    if (fsc) fscanf(cfle,"\n%d ", &vis);
    if (feof(cfle)) fsc = 0;
    fsc = Fscanf(cfle, 1, vard);
    if (fsc) 
     {
      msun = vard[0];
      fsc = Fscanf(cfle, 1, vard);
      if (feof(cfle)) fsc = 0;
      else msat = vard[0];
     };

//    if (fsc) fscanf(cfle,"\n%lf", &yrepoch);
    fsc = Fscanf(cfle, 1, vard);
    if (fsc) 
     {
      yrepoch = vard[0];
      if (feof(cfle)) fsc = 0;
     }; 

    if (fsc) fscanf(cfle,"\n%d", &mfsz);
    if (feof(cfle)) fsc = 0;
    if (fsc) fscanf(cfle,"\n%d", &fbold);
    if (feof(cfle)) fsc = 0;
    if (fsc) fscanf(cfle,"\n%d\n", &leftm);
    if (feof(cfle)) fsc = 0;
    if (fsc)
     {
      if (fgets(wwwname, 104, cfle) == NULL) fsc = 0;
      else
       {
        j = strlen(wwwname);
        if (j>0)
         {
          if(wwwname[j-1] == '\n') wwwname[j-1] = '\0';
         };
       };
     };
//    if (feof(cfle)) fsc = 0;
//    if (fsc) fscanf(cfle,"\n%lf %lf %lf %lf", &dura, &dF107, &dFB107, &dKp);
//    if (fsc) fscanf(cfle,"\n%lf %lf %lf", &dCd, &dCr, &dAm);

    if (fsc) fsc = Fscanf(cfle, 4, vard);
    if (fsc) 
     {
      dura = vard[0];
      dF107 = vard[1];
      dFB107 = vard[2];
      dKp = vard[3];
     }; 
    if (feof(cfle)) fsc = 0;

    if (fsc) fsc = Fscanf(cfle, 3, vard);
    if (fsc) 
     {
      dCd = vard[0];
      dCr = vard[1];
      dAm = vard[2];
     }; 

    if (feof(cfle)) fsc = 0;
    if (fsc) fscanf(cfle,"\n%d", &sidxcn);
    if (feof(cfle)) fsc = 0;

    if (sidxcn > GBL_MXSAT) sidxcn = 0;
    if (sidxcn > 0)
     {
      fsb = 1;
      for (k=0; k<sidxcn; k++)
       {
        if (fsb)
         {
          fscanf(cfle, "\n");
          if (fgets(sname, 20, cfle) == NULL) fsb = 0;
          else
           {
            j  = strlen(sname);
            if (j>0)
             {
              if(sname[j-1] == '\n') sname[j-1] = '\0';
              strcpy(indata->orbelm.sname[k],sname);
             };
           };
         };  // fsb

        if (feof(cfle))
         {
          fsb = 0;
          break;
         };
       }; // for k
      if (fsb == 0) sidxcn = k;
     };    // sidxcn > 0

    if (ferror(cfle))
     {
      fsc = 0;
      clearerr(cfle);
     };
    fclose(cfle);
   }
  else fsc = 2;

  if (fsc == 1)
   {
    if (tz != 16.0) indata->tzauto = 0;
    else indata->tzauto = 1;
    if ((tz > 15.0) || (tz < -15.0)) tz = 0;
    indata->tzone = tz;
    indata->geolat = lat;
    indata->geolong = lng;
    indata->geoheight = ht;
    if(strlen(gname) > 0) strcpy(indata->geoname,gname);
    if(strlen(tlname) > 0) strcpy(indata->tlename,tlname);
    if(strlen(wwwname) > 5) strcpy(indata->wwwname,wwwname);
    if (swmax != 8) swmax = 10;
    indata->dfsz = swmax;
    if ((mgrid < 0) || (mgrid > 2)) mgrid = 1;
    if ((mgrey < 0) || (mgrey > 2)) mgrey = 0;
    indata->mapbuf.emagrid = mgrid;
    indata->mapbuf.emagrey = mgrey;
    indata->visflg = vis;
    indata->minsunel = msun;
    indata->minsatel = msat;
    indata->dF107 = dF107;
    indata->dFB107 = dFB107;
    indata->dKp = dKp;
    indata->dCd = dCd;
    indata->dCr = dCr;
    indata->dAm = dAm;
    indata->yrepoch = yrepoch;
    indata->mfsz = mfsz;
    indata->fbold = fbold;
    indata->leftm = (leftm == 1);
    indata->sidxcn = sidxcn;
    indata->duration = dura;
    if (dura < 1.0) indata->duration = 1.0;
    if (dura > 14.0) indata->duration = 14.0;
   };

  return fsc;
 }

/*------------------------- putcfg ---------------------------------*/

void putcfg (WinpData* indata)
 {
  /* Save configuration data to the file SATCAL.CFG
     NOTE: If automatic system timezone, timezone will be set to 16.
  */
  double tz, dura;
  int j;
  char bufn[100];
  FILE* cfle;

  if (indata->tzauto) tz = 16.0;
  else tz = indata->tzone;
  dura = indata->duration;
  if (dura < 1.0) dura = 1.0;
  if (dura > 14.0) dura = 14.0;

  cfle = fopen ("satcal.cfg","w");
  if (cfle != NULL)
   {
    sprintf(bufn,"%lf \n%lf \n%lf \n%lf \n",
                 tz, indata->geolat, indata->geolong, indata->geoheight);
    decdot(bufn);
    fputs(bufn, cfle);
    fputs(indata->geoname, cfle);
    fprintf(cfle,"\n%d \n%d \n%d \n",indata->dfsz, indata->mapbuf.emagrid,
                 indata->mapbuf.emagrey);
    fputs(indata->tlename, cfle);
    sprintf(bufn,"\n%d \n%lf \n%lf", indata->visflg, indata->minsunel,
                  indata->minsatel );
    decdot(bufn);
    fputs(bufn, cfle);
    sprintf(bufn,"\n%lf", indata->yrepoch);
    decdot(bufn);
    fputs(bufn, cfle);
    fprintf(cfle,"\n%d", indata->mfsz);
    fprintf(cfle,"\n%d", indata->fbold);
    if (indata->leftm) j = 1;
    else j = 0;
    fprintf(cfle,"\n%d \n", j);
    fputs(indata->wwwname, cfle);
    sprintf(bufn,"\n%g %7.1lf %7.1lf %7.1lf", dura, indata->dF107,
              indata->dFB107, indata->dKp);
    decdot(bufn);
    fputs(bufn, cfle);
    sprintf(bufn,"\n%13.5le %13.5le %13.5le", indata->dCd,
              indata->dCr, indata->dAm);
    decdot(bufn);
    fputs(bufn, cfle);
    fprintf(cfle,"\n%d \n", indata->sidxcn);
    if (indata->sidxcn > 0)
     {
      for (j=0; j<indata->sidxcn; j++)
       {
        fputs(indata->orbelm.fsname[j], cfle);
        fprintf(cfle, "\n");
       };
     };
    fclose(cfle);
   };
 }

/*-------------------------- ElvRefr --------------------------------*/

double ElvRefr(WinpData* indata, double el)
/* Correct for refraction
   The very same elevation is needed with and without refraction
   in different parts of the program. That is why ElvRefr will be
   called for the same elevation each time a correction is need
   rather than only once when the elevation is calculated first.

   el is elevation in degrees
   return the corrected elevation in degrees
 */
{
 double res;

 res = el;
 if (indata->refract)
  {
   if ((indata->relsat == 0) && (el > -2.0) && (el < 85.0))
    {
     if (el < 10.0)
      {
       if (el > -1.0)
        res = 3.5689 * (0.1594 + 0.0196*el + 0.0002*el*el)
                     / (1.0 + 0.505*el + 0.0845*el*el);
       else res = 0.84446937 * (2.0 + el);
      }
     else res = 1.521154622e-2 / tan(el*degrad);
     res += el;
   };
  };

 return res;
}

/*------------------------- SetTtUt ---------------------------------*/
double SetTtUt (WinpData* indata, int yy)
// set difference TT - UTC in sec
{
 double res;

 if (indata->IERSflg)
  {
   res = 32.184 - double(indata->UtcTai);
  }
 else res = DefTdUt(yy);

 return res;
}

/*------------------------- DefDate ---------------------------------*/

void DefDate (int &dd, int &mm, int &yr)  // Get Default Date
 {
  time_t tt;
  double jd, hr;

  tt = time(NULL);                  // this is more compatible
  jd = 40587.0 + tt/86400.0; // seconds since 1-JAN-1970
  caldat(jd, dd, mm, yr, hr);
 };

/*------------------------- DefTime ---------------------------------*/

void DefTime (WinpData* indata)  // Get System Time and Date
 {
  time_t tt;
  int hh, mm, ss;
  double jd, hr, jloc, ss1;
  struct tm *tblock;

  tt = time(NULL);    // seconds UTC since 1-JAN-1970
  jd = 40587.0 + tt/86400.0;
  caldat(jd, hh, mm, ss, hr);
  indata->year = ss;
  indata->month = mm;
  indata->day = hh;
  dms(hr, hh, mm, ss1);
  indata->hour = hh;
  indata->minute = mm;
  indata->second = int(ss1 + 0.5);

  // now get the timezone
  tblock = localtime(&tt);
  jloc = double(tblock->tm_hour) + double(tblock->tm_min)/60.0 + double(tblock->tm_sec)/3600.0;
  hh = tblock->tm_mday;
  mm = tblock->tm_mon + 1;
  ss = 1900 + tblock->tm_year;
  jloc = mjd(hh, mm, ss, jloc);
  GBL_timezone = (jd - jloc) * 86400.0;
 };

 /*------------------------- setNewMJD ---------------------------------*/

void setNewMjd (WinpData* indata)  // set MJD from new time input
 {
  double hour;
  int yy;

  if (indata->tzauto) indata->tzone = -GBL_timezone/3600.0;
  hour = double(indata->hour) - indata->tzone
			 + double(indata->minute)/60.0 + double(indata->second)/3600.0;
  indata->lastmjd = mjd(indata->day, indata->month, indata->year, hour);
  indata->lorbmjdep = 0;  // as a warning to PassDetailDLG to renew pass calcs
  indata->passact = 0;    // reset a possible pass activation
 // indata->del_tdut = DefTdUt(indata->year);

  if (indata->timer_del)
   {
    indata->timer_del = 0;
    indata->timer_on = 0;
   };

  yy = indata->year;
  indata->del_tdut = SetTtUt(indata, yy);
 };

 /*------------------------- RT_MJD ---------------------------------*/

double RT_MJD (WinpData* indata)  // Get Real Time MJD (UT)
 {
  int dd, mm, yr, hh, mn, ss;
  double hour;

  // save current values
  yr = indata->year;
  mm = indata->month;
  dd = indata->day;
  hh = indata->hour;
  mn = indata->minute;
  ss = indata->second;

  DefTime (indata);

  hour = double(indata->hour) + double(indata->minute)/60.0
                + double(indata->second)/3600.0;
  if (indata->rt_offset != 0) hour += indata->rt_offset;
  hour = mjd(indata->day, indata->month, indata->year, hour);

  // restore
  indata->year = yr;
  indata->month = mm;
  indata->day = dd;
  indata->hour = hh;
  indata->minute = mn;
  indata->second = ss;

  return hour;
 }

/**************************** tleread ************************************/
/* The Two-Line-Element set for a satellite consists of the following entries :

   First line : Satellite name  (from Pos 1 on)
                The program assumes that this line is < 30 characters long.
   Second Line : Pos.1     : '1'
                 Pos.10-17 : International Launch Designator
                 Pos.19-20 : Last 2 digits of epoch year
                 Pos.21-32 : Epoch days  ddd.ffffffff
                 Pos.34-43 : Ndot2 term
                 Pos.45-52 : Ndot6 term  +XXXXX-X
                 Pos.54-61 : BSTAR term  +XXXXX-X
                 Pos.63    : Ephemeris Type
   Third Line :  Pos.1     : '2'
                 Pos. 9-16 : Inclination
                 Pos.18-25 : Right Asc.Ascending Node
                 Pos.27-33 : Eccentricity (fractional part)
                             or 26 - 33 for hyperbolic orbits
                 Pos.35-42 : Argument of Perigee
                 Pos.44-51 : Mean Anomaly
                 Pos.53-63 : Mean Motion (perigee for hyperbolic)
                 Pos.64-68 : Revolution Number at Epoch
 */


/*----------------------- function cnvepoch -----------------------------*/

void cnvepch (WinpData* indata, int year, double fday, char* epoch)
 {
  /* Convert epoch given as ddd.ffffffff yyyy into the format
     dd-MMM-yyyy   hh:mm:ss in string epoch.
     year : year, fday : day of year and fraction of day.
     Also calculated is the MHD of epoch which will be stored in
     indata->orbelm.mjdep
  */
  int day, month, hour, min, sc;
  double dr, sec;
  char mchr[4];
  char ibf[20];

  day = int(fday);
  dr = 24.0 * fmod(fday,1.0);
  indata->orbelm.mjdep[GBL_MXSAT] = mjd (day, 1, year, dr);

  caldat (indata->orbelm.mjdep[GBL_MXSAT], day, month, year, sec);
  dms (dr, hour, min, sec);
  sc = int(sec+0.5);

  sprintf(epoch, "%2i-", day);
  GetMonth (month, mchr);
  strcat(epoch, mchr);
  sprintf(ibf, "-%4i   %2i:%2i:%2i", year, hour, min, sc);
  strcat(epoch, ibf);
 }

/*----------------------- function getsax -----------------------------*/

double getsax (int aflg, double n0, double ecc, double inc)
 {
  /* Calculate semimajor axis a (in km) from mean motion n0 (in
     revolutions / day).
     aflg = 1 if SGP4 model, 0 if simple model
  */
  const double gm = 2.9755364003E15;  // Gravitation Constant G*Mearth
  const double re = 6378.137;  // Earth equatorial radius
  const double j2 = 1.082628E-3;   // J2-Term

  double a, n, c;

  if (n0 < 0.0001) n0 = 0.0001;  // just in case
  //  preliminary a
  n0 = 2.0 * M_PI * n0;
  n = n0 * n0;
  a = pow (gm/n,1.0/3.0);

  // correct for J2 term
  n = 1.0 - ecc*ecc;
  n = pow (n, -1.5);
  c = sin (inc*degrad);
  n = -1.5*j2*re*re/(a*a) * n * (1.0 - 1.5*c*c);

  // choose between one of the two ways to calculate a depending on model
  // 2nd way (gets smaller values for a)
  if (aflg) a = a * (1.0 + 1.0/3.0 * n * (1.0 - n));
  else
   {     // 1st way (gets larger values for a)
    n = n0 / (1.0-n);
    n = n*n;
    a = pow (gm/n,1.0/3.0);
   };

  if (a < 0.001) a = 0.001;

  return a;
 }

/*----------------------- function getn0 -----------------------------*/

double getn0 (double a, double ecc, double inc)
 {
  /* Calculate mean motion n0 (in revolutions / day)
     from semi-major axis a with eccentricity ecc
     and inclination inc (in deg).
  */
  const double gm = 2.9755364003E15;  // Gravitation Constant G*Mearth
  const double re = 6378.137;  // Earth equatorial radius
  const double j2 = 1.082628E-3;   // J2-Term

  double n, c, w;

  //  preliminary n
  if (a == 0) a = 1.0; // just in case
  if (a < 0) a = -a;   // just in case
  n = sqrt (gm / (a*a*a));

  // correct for J2 term
  w = 1.0 - ecc*ecc;
  if (w > 0)
   {
    w = pow (w, -1.5);
    c = sin (inc*degrad);
    n = n*(1.0 + 1.5*j2*re*re/(a*a) * w * (1.0 - 1.5*c*c));
   }
  else n = 1.0;   // do something to avoid a domain error

  n = n / (2.0*M_PI);
  if (n > 1000.0) n = 1000.0;  // avoid possible errors

  return n;
 }

/*----------------------- function tlecpy -----------------------------*/

void tlecpy (char* buf2, char* buf1, int s, int n)
 {
  // copy n characters to buf2 from buf1 starting with character # s
  // in buf1. A NULL character will be added to buf2
  int i;

  for (i=0; i<n; i++) buf2[i] = buf1[i+s-1];
  buf2[n] = '\0';
 }

/*----------------------- function convldsn -----------------------------*/

void convldsn (WinpData* indata, bool tlsc)
{
 // convert International Launch Designator
 // tlsc : true if conversion from TLE format into standard format

 int year;
 char cyr[3];

 if (tlsc)
  {
   cyr[0] = indata->idsgn[0];
   if (cyr[0] == ' ')
    {
     for (year=0; year<11; year++) indata->ldsgn[year] = ' ';
     indata->ldsgn[11] = '\0';

     return;
    };
   cyr[1] = indata->idsgn[1];
   cyr[2] = '\0';
   year = atoi(cyr);  // if last two digits of year > 56, 19xx is assumed
   if (year > 56)
    {
     indata->ldsgn[0] = '1';
     indata->ldsgn[1] = '9';
    }
   else
    {
     indata->ldsgn[0] = '2';
     indata->ldsgn[1] = '0';
    };
   indata->ldsgn[2] = indata->idsgn[0];
   indata->ldsgn[3] = indata->idsgn[1];
   indata->ldsgn[4] = '-';
   indata->ldsgn[5] = indata->idsgn[2];
   indata->ldsgn[6] = indata->idsgn[3];
   indata->ldsgn[7] = indata->idsgn[4];
   indata->ldsgn[8] = indata->idsgn[5];
   indata->ldsgn[9] = indata->idsgn[6];
   indata->ldsgn[10] = indata->idsgn[7];
   indata->ldsgn[11] = '\0';
  }
 else
  {
   indata->idsgn[0] = indata->ldsgn[2];
   indata->idsgn[1] = indata->ldsgn[3];
   indata->idsgn[2] = indata->ldsgn[5];
   indata->idsgn[3] = indata->ldsgn[6];
   indata->idsgn[4] = indata->ldsgn[7];
   indata->idsgn[5] = indata->ldsgn[8];
   indata->idsgn[6] = indata->ldsgn[9];
   indata->idsgn[7] = indata->ldsgn[10];
   indata->idsgn[8] = '\0';
   for (year=0; year<8; year++)
    {
     if (indata->idsgn[year] == '\0') indata->idsgn[year] = ' ';
    };
  };
}

/*----------------------- function gettle -----------------------------*/

int gettle(WinpData* indata, FILE* satfle, char* bufin, char* otxt, int& leof)
 {
  /* Get 2-line element set and output it
     leof = 1 if eof encountered

     RETURN: 1 if everything successful, 0 otherwise
  */
  const double re = 6378.137;  // Earth equatorial radius

  double inc, a, apo, peri, ecc;  // Kepler elements
  double n0, nd2, cx, cy, cz, c1, c2, c3;
  long int repoc, nd6;
  int ne6, j;
  int tlok;  // 1, if TLE set is formally correct
  char epoch[25];
  char buffy[45];
  char staxt[25];
  int year;     // year of epoch
  double fday;  // day or year and fraction of day
  int sx;
  bool irops;
  double vard[7];
  Vec3 rr1, vv1;

  sx = GBL_MXSAT;  // assigned to have less to write with index

  tlok = 1;
  if (bufin[0] != '1')
   {
    if (fgets(bufin, 80, satfle) == NULL) leof = 1;  // first TLE line
    if (leof) tlok = 0;
    else
     {
      if (bufin[0] != '1') tlok = 0;
     };
   };

  if (strlen(bufin) < 63) tlok = 0;  // buffer too short

  if (tlok)  // first TLE line should be correct
   {
    tlecpy(indata->idsgn, bufin, 3, 5);
    indata->orbelm.norad[sx] = atoi(indata->idsgn);
    tlecpy(indata->idsgn, bufin, 10, 8);
    tlecpy(epoch, bufin, 19, 2);
    year = atoi(epoch);  // if last two digits of year > 56, 19xx is assumed
    if (year > 56) year += 1900;
    else year += 2000;
    tlecpy(epoch, bufin, 21, 12);
    fday = Atof (epoch);
    cnvepch (indata, year, fday, epoch);   // convert epoch into MJD and text
    convldsn(indata, true);  // convert International Launch Designator
    strcpy (indata->orbelm.ldsgn[sx], indata->ldsgn);

    tlecpy(buffy, bufin, 40, 19);
    strcpy (staxt, "+++++STATEVCT1+++++");
    if (Strncmpi(buffy, staxt,19) == 0) indata->orbelm.numflg[sx] = 1;
    else indata->orbelm.numflg[sx] = 0;

    if (indata->orbelm.numflg[sx])  // read the four lines for numeric case
     {
      tlecpy(buffy, bufin, 34, 5);
      repoc = atoi (buffy);

      // second TLE line for numeric case

      tlok = 1;

//      fscanf(satfle,"%lf %lf %lf %i \n",&cx, &cy, &cz, &j);
//      if (ferror(satfle)) tlok = 0;
      tlok = Fscanf(satfle, 4, vard);
      if (tlok) 
       {
        cx = vard[0];
        cy = vard[1];
        cz = vard[2];
        j = int(vard[3]);
        if (feof(satfle)) tlok = 0;
       }; 

      if (tlok)
       {
        indata->orbelm.x0[sx] = cx;
        indata->orbelm.y0[sx] = cy;
        indata->orbelm.z0[sx] = cz;
        rr1[0] = cx;
        rr1[1] = cy;
        rr1[2] = cz;

//        fscanf(satfle,"%lf %lf %lf %i\n",&cx, &cy, &cz, &j);
//        if (ferror(satfle)) tlok = 0;
        tlok = Fscanf(satfle, 4, vard);
        if (feof(satfle)) tlok = 0;
       };
      if (tlok)
       {
        cx = vard[0];
        cy = vard[1];
        cz = vard[2];
        j = int(vard[3]);
        indata->orbelm.vx0[sx] = cx;
        indata->orbelm.vy0[sx] = cy;
        indata->orbelm.vz0[sx] = cz;
        vv1[0] = cx;
        vv1[1] = cy;
        vv1[2] = cz;

//        fscanf(satfle,"%lf %lf %lf %lf %lf %lf \n",
//                        &cx, &cy, &cz, &c1, &c2, &c3);
//        if (ferror(satfle)) tlok = 0;

        tlok = Fscanf(satfle, 6, vard);
        if (feof(satfle)) tlok = 0;
       };
      if (tlok)
       {
        cx = vard[0];
        cy = vard[1];
        cz = vard[2];
        c1 = vard[3];
        c2 = vard[4];
        c3 = vard[5];
        indata->orbelm.F107[sx] = cx;
        indata->orbelm.FB107[sx] = cy;
        indata->orbelm.Kp[sx] = cz;
        indata->orbelm.Cd[sx] = c1;
        indata->orbelm.Cr[sx] = c2;
        indata->orbelm.Am[sx] = c3;
        indata->orbelm.deltv[sx] = 0.0;
        indata->orbelm.dyaw[sx] = 0.0;
        indata->orbelm.dpitch[sx] = 0.0;
        indata->orbelm.ionflg[sx] = false;
        indata->orbelm.ionsun[sx] = true;
        indata->orbelm.updepoch[sx] = false;

        indata->orbelm.x[sx] = indata->orbelm.x0[sx];
        indata->orbelm.y[sx] = indata->orbelm.y0[sx];
        indata->orbelm.z[sx] = indata->orbelm.z0[sx];
        indata->orbelm.vx[sx] = indata->orbelm.vx0[sx];
        indata->orbelm.vy[sx] = indata->orbelm.vy0[sx];
        indata->orbelm.vz[sx] = indata->orbelm.vz0[sx];
        indata->orbelm.mjdcur[sx] = indata->orbelm.mjdep[sx];

        // get the corresponding Kepler elements
        nd2 = 0.0;
        indata->orbelm.ndot[sx] = nd2;
        indata->orbelm.n6dot[sx] = 0.0;
        indata->orbelm.bstar[sx] = 0.0;
        indata->orbelm.aflg[sx] = 0;
        cx = indata->orbelm.mjdep[sx];
        rr1 /= 1000.0;
        vv1 /= 1000.0;
        oscelm (398600.4415, cx, rr1, vv1,
                     cy, cz, a, ecc, c1, c2, inc);
        indata->orbelm.ra[sx] = c1;
        indata->orbelm.aper[sx] = c2;
        indata->orbelm.m0[sx] = cz;

        if (ecc < 1.0) n0 = getn0(a, ecc, inc);
        else
         {
          n0 = a * (1.0 - ecc);
         };
        indata->orbelm.n0[sx] = n0;
       };
     }

    else  // continue with the standard TLE case
     {
      // get NDOT2
      tlecpy(buffy, bufin, 34, 10);
      nd2 = Atof (buffy);
      indata->orbelm.ndot[sx] = nd2;

      // get NDOT6 and BSTAR
      tlecpy(buffy, bufin, 45, 6);
      nd6 = atol (buffy);
      tlecpy(buffy, bufin, 51, 2);
      ne6 = atoi (buffy);
      n0 = float(nd6) * 1.0E-5;
      indata->orbelm.n6dot[sx] = n0 * pow(10.0,ne6);
      tlecpy(buffy, bufin, 54, 6);
      nd6 = atol (buffy);
      tlecpy(buffy, bufin, 60, 2);
      ne6 = atoi (buffy);
      n0 = float(nd6) * 1.0E-5;
      indata->orbelm.bstar[sx] = n0 * pow(10.0,ne6);
      indata->orbelm.aflg[sx] = (bufin[62] != '9');  // ephemeris type

      // second TLE line
      if (fgets(bufin, 80, satfle) == NULL) leof = 1;
      if (leof) tlok = 0;
      else
       {
        if (strlen(bufin) < 63) tlok = 0;  // buffer too short
        if (bufin[0] != '2') tlok = 0;   // not the second line
       }

      if (tlok)  // second TLE line should be correct
       {
        tlecpy(buffy, bufin, 9, 8);  // inclination
        inc = Atof (buffy);
        tlecpy(buffy, bufin, 18, 8);  // Right Asc.Ascending Node
        indata->orbelm.ra[sx] = Atof (buffy);
        tlecpy(buffy, bufin, 35, 8);  // Argument of Perigee
        indata->orbelm.aper[sx] = Atof (buffy);
        tlecpy(buffy, bufin, 44, 8);  // Mean Anomaly
        indata->orbelm.m0[sx] = Atof (buffy);
        tlecpy(buffy, bufin, 53, 11);  // Mean Motion

        n0 = Atof (buffy);
        indata->orbelm.n0[sx] = n0;

        if (bufin[27] == '.') // hyperbolic orbit
         {
          tlecpy(buffy, bufin, 26, 8);  // eccentricity
         }
        else
         {
          bufin[25] = '.';
          bufin[24] = '0';
          tlecpy(buffy, bufin, 25, 9);  // eccentricity
         };
        ecc = Atof (buffy);
        if (strlen(bufin) < 68) repoc = -1;
        else
         {
          tlecpy(buffy, bufin, 64, 5);
          repoc = atoi (buffy);
         };
       };
     };  // end of standart TLE input

    // calculate remaining data

    if (tlok)
     {
      if (n0 < 1E-30) n0 = 1E-30;  // avoid numerical crash
      if (ecc < 1.0)
       {
        a = getsax (indata->orbelm.aflg[sx], n0, ecc, inc);
        indata->orbelm.adot[sx] = -4.0/3.0 * a/n0 * nd2;
        apo = a * (1.0 + ecc) - re;
        peri = a * (1.0 - ecc) - re;
       }
      else
       {
        a = 1.0 - ecc;
        if (a < 0) a = n0 / a;
        else a = 1.0;
        apo = 1.0;
        peri = 1.0;
       };

      // check special cases
      if (indata->orbelm.lflg == 1) // check for very low orbits
       {
        if (n0 < 16.088) tlok = 0;  // < 250 km altitude
	else
	 {            // do not consider TLEs more than a week old.
	   if (indata->orbelm.mjdep[sx] < (indata->lastmjd - 7.0)) tlok = 0;
	 };
       };

      if ((indata->orbelm.nflg == 1) && (repoc != 0))   // check for new S/C
       {
        if (repoc >= 100) tlok = 0; // < 100 rev.
        if ((n0 < 4) && (repoc >= 10)) tlok = 0; // < 10 rev. for higher orbits
       };
     };
   };

  if (tlok)  // copy the corresponding values
   {
    if(indata->adjflg && (indata->orbelm.numflg[sx] == 0)) // adjust old orbit elements
     {
      indata->orbelm.mjdep[sx] = indata->orbelm.mjdep[sx] - indata->adjtim/86400.0;
      if (indata->adjn0 != 0)
       {
        irops = true;
        if ((indata->iridflg == 3) && indata->iridops)
         {
          if((a < 7150.0) || (a > 7159.9)) irops = false;
         };
        if (irops)  // don't adjust for this special Iridium case
         {
          indata->orbelm.n0[sx] = indata->adjn0;
          a = getsax (0, indata->adjn0, ecc, inc);
         };
       };
      indata->orbelm.aflg[sx] = 0;
      indata->orbelm.adot[sx] = 0;
      indata->orbelm.ndot[sx] = 0;
      indata->orbelm.n6dot[sx] = 0;
      indata->orbelm.ra[sx] = indata->orbelm.ra[sx] + indata->adjra;
     };

    if (indata->orbelm.nrflg)  // put IDSGN in stead of name
     {
      year = strlen(indata->orbelm.fsname[sx]);
      if (year < 2)
       {
        if (year == 1)
         {
          if (indata->orbelm.fsname[sx][0] == ' ') year = 0;
         };
        if (year < 1) strcpy(indata->orbelm.fsname[sx], indata->ldsgn);
       };
     };

    indata->orbelm.a[sx] = a;
    indata->orbelm.inc[sx] = inc;
    indata->orbelm.ecc[sx] = ecc;
    // check that there are no line feeds and carriage returns in sat name
    bool nck;
    nck = true;
    for (j=0; j<20; j++)
     {
      if (indata->orbelm.fsname[sx][j] == '\0') nck = false;
      if ((indata->orbelm.fsname[sx][j] == '\n')||(indata->orbelm.fsname[sx][j] == '\r'))
       {
        if (nck)
         {
          indata->orbelm.fsname[sx][j] = '\0';
          nck = false;
         };
       }
     };
    strcpy(otxt,indata->orbelm.fsname[sx]);
    strcat(otxt,"\n================================");
    sprintf(buffy,"\nNORAD#\t\t:%6i\nInt.Launch#\t: ", indata->orbelm.norad[sx]);
    strcat(otxt,buffy);
    strcat(otxt,indata->orbelm.ldsgn[sx]);
    strcat(otxt,"\n\nEpoch\t: ");
    strcat(otxt,epoch);
    sprintf(buffy,"\n\nInclination\t: %8.4lf", inc);
    decdot(buffy);
    strcat(otxt,buffy);
    sprintf(buffy,"\nRight Ascension\t: %8.4lf", indata->orbelm.ra[sx]);
    decdot(buffy);
    strcat(otxt,buffy);
    sprintf(buffy,"\nArg. of Perigee\t: %8.4lf", indata->orbelm.aper[sx]);
    decdot(buffy);
    strcat(otxt,buffy);

    if (ecc < 1.0)  // elliptic orbit
     {
      sprintf(buffy,"\nMean Anomaly\t: %8.4lf", indata->orbelm.m0[sx]);
      decdot(buffy);
      strcat(otxt,buffy);
      sprintf(buffy,"\n\nEccentricity\t: %9.7lf", indata->orbelm.ecc[sx]);
      decdot(buffy);
      strcat(otxt,buffy);
      sprintf(buffy,"\nSemi-Major Axis a\t: %10.3lf km", indata->orbelm.a[sx]);
      decdot(buffy);
      strcat(otxt,buffy);
      sprintf(buffy,"\nChange in a\t: %10.3lf km/day",indata->orbelm.adot[sx]);
      decdot(buffy);
      strcat(otxt,buffy);
      sprintf(buffy,"\nPerigee\t\t: %10.3lf km", peri);
      decdot(buffy);
      strcat(otxt,buffy);
      sprintf(buffy,"\nApogee\t\t: %10.3lf km", apo);
      decdot(buffy);
      strcat(otxt,buffy);

      if (repoc > 0) sprintf(buffy,"\n\nRevolution at Epoch: %li", repoc);
      else strcpy(buffy,"\n");
      strcat(otxt,buffy);
      sprintf(buffy,"\nRevolutions per day: %12.8lf", n0);
      decdot(buffy);
      strcat(otxt,buffy);
      if (fabs(n0) > 0.001)
       {
        nd2 = 1440.0 / n0;  // rev period in min
        sprintf(buffy,"\n\nPeriod (min):%10.2lf", nd2);
        decdot0(buffy);
        strcat(otxt,buffy);
       };
     }
    else    // hyperbolic orbit
     {
      sprintf(buffy,"\n\nEccentricity\t: %8.5lf", indata->orbelm.ecc[sx]);
      decdot(buffy);
      strcat(otxt,buffy);
      sprintf(buffy,"\nSemi-Major Axis a\t: %10.3lf km", indata->orbelm.a[sx]);
      decdot(buffy);
      strcat(otxt,buffy);
      sprintf(buffy,"\nPerigee Distance q\t:%11.3lf km", indata->orbelm.n0[sx]);
      decdot(buffy);
      strcat(otxt,buffy);
     };

    // get state vector at epoch in case it's needed for numeric calcs
    if (indata->orbelm.numflg[sx] == 0) EpochState (indata, sx, true);
    else
     {
      strcpy(buffy, "\n+++ Numeric Propagation +++");
      strcat(otxt, buffy);
     };
   }
  else otxt[0] = '\0';

  return tlok;
 }

/*----------------------- function tleread -----------------------------*/

int tleread(WinpData* indata, FILE* satfle, char* otxt, int morflg)
 {
  /*
  Extraction of two-line element set from TLE file and output the
  elements as classical Kepler elements to otxt.

  Two different ways of calculating a from the mean motion n are
  built into the program. Which one to use depends on the orbit model
  used to calculate the ephemeris. If you are not happy with the default
  which is method #1 (leading to a slightly lower value of a) you can enter
  '#2' instead of the satellite name when prompted for that name. Entering
  '#1' will switch back to the first method. The constant AFLG sets at
  the default at the beginning (TRUE for #1 and FALSE for #2).

  INPUT : File satfle  containing the two-line elements. The file is
			 assumed to be open.
			 morflg = 1 if search next matches, 0 otherwise

  OUTPUT : If no formal error was encountered, the orbit elements,
			  together with the name of the satellite will be written
			  to otxt. Multiple fits for a satellite name are possible.

  RETURN: 0 if no matching satellite found,
			 1 if satellite found, 2 if more matches pending
  */

  int found, leof;
  int sx, j;
  int svnrflg;
  size_t lsatname, buflen;  // length of the search string and buffer
  char bufin[82];
  char nordck[6];
  static char nextsat[21];

  sx = GBL_MXSAT;  // assigned to have less to write with index
  indata->lastsx = 9999;  // reset sx marker for SGP4 initialization

  // the following serves to pacify the Windows compiler
  svnrflg = indata->orbelm.nrflg;
  if (indata->orbelm.sname[sx][0] == '*') indata->orbelm.nrflg = 1;

  found = 0;
  lsatname = strlen(indata->orbelm.sname[sx]);

  for (j=0; j<20; j++) indata->orbelm.fsname[sx][j] = ' ';
  indata->orbelm.fsname[sx][20] = '\0';

  if (feof(satfle) == 0) leof = 0;
  else leof = 1;

  while (!leof && !found)
   {
    if (fgets(bufin, 80, satfle) == NULL) leof = 1;  // get one TLE line

    if (morflg)  // search for next match
     {
      if ((indata->orbelm.nrflg == 0) && (indata->orbelm.sname[sx][0] != '*'))
       {
        found = 1;  // we've already found match in previous search
        strcpy (indata->orbelm.fsname[sx], nextsat);
       };
     };

    buflen = strlen(bufin);
    if ((buflen > 59) && ((bufin[0] == '1') || (bufin[0] == '2'))) // 2-line data
     {
      if ((indata->orbelm.nrflg) && (bufin[0] == '1'))   // NORAD# search
       {
        if (indata->orbelm.nrflg == 1)
         {
          tlecpy(nordck, bufin, 3, 5);  // check NORAD#
          if (indata->orbelm.norad[sx] == atoi(nordck)) found = 1;
          if (indata->orbelm.sname[sx][0] == '*') found = 1; // all satellites
         }
        else  // check International Launch Designator
         {
          if (indata->orbelm.sname[sx][0] == '*') found = 1; // all satellites
          else
           {
            tlecpy(indata->idsgn, bufin, 10, 8);
            convldsn(indata, true);
            if(Strncmpi(indata->orbelm.sname[sx], indata->ldsgn, lsatname) == 0)
                       found = 1;
           };
         };
       };
     }
    else     // line with satellite name
     {
      if (indata->orbelm.nrflg == 0)   // normal name (not NORAD#) search
       {
        if ((Strncmpi(indata->orbelm.sname[sx], bufin, lsatname) == 0)
                                       || (indata->orbelm.sname[sx][0] == '*'))
         {
          found = 1;  // satellite found by name
          strncpy(indata->orbelm.fsname[sx],bufin,20);
         };
       }
      else strncpy(indata->orbelm.fsname[sx],bufin,20); // save it for NORAD search

     };

    if (found)      // process found satellite
     {
      found = gettle(indata, satfle, bufin, otxt, leof);  // get TLEs
     };

    if (found)      // process found satellite
     {
      // check for further occurrences
      if (indata->orbelm.nrflg == 0)   // normal name (not NORAD#) search
       {
        if (indata->orbelm.sname[sx][0] == '*') found = 2;
        else
         {
          while (!leof)
           {
            if (fgets(bufin, 80, satfle) == NULL) leof = 1;
            if ((leof == 0) && (Strncmpi(indata->orbelm.sname[sx], bufin, lsatname) == 0))
             {
              found = 2;  // next match found
              strncpy(nextsat,bufin,20);
              leof = 1;
             };
           };  // end while
         };
       }  // indata->orbelm.nrflg
      else  // NORAD search
       {
        if (indata->orbelm.sname[sx][0] == '*') found = 2;
       };
     };
   };  // while not eof, found

  if (found < 1)
   {
    if (indata->orbelm.nrflg == 1)
       sprintf (otxt, "\n+++ No match found for NORAD# %i +++", indata->orbelm.norad[sx]);
    else
     {
      strcpy (otxt,"\n+++ No match found for ");
      strcat (otxt, indata->orbelm.sname[sx]);
      strcat (otxt, " +++");
     };
   };

  if ((indata->orbelm.sname[sx][0] == '*') && (found == 2))
   {
    long int ofs;    // avoid an error message at eof

    ofs = ftell (satfle);
    if (fgets(bufin, 80, satfle) == NULL) found = 1;
    else fseek (satfle, ofs, SEEK_SET);
   };

  indata->orbelm.nrflg = svnrflg; // this was for the Windows compiler

  return found;
 }

/********************************************************************/

/*------------------------- GetRVMat ------------------------------*/

Mat3 GetRVMat (Vec3& r1, Vec3& v1, double yaw, double pitch, int inpl)
 {
  /* Get matrix mx to transform from the original reference (equatorial)
     into a coordinate system which is defined by the r-v orbit plane
     (r1 : position vector, v1 : velocity vector)
     with x pointing into the direction which results when, starting
     from +v (and assuming that one sits in an airplane with the
     bottom of the plane pointing toward the center of the Earth)
     one first yaws then pitches by the respective yaw-pitch
     angles (given in degrees). Yaw positive for turn to right;
     Pitch positive for upward turn.

     if inpl = 0, the transformation is into the r-v plane with x
        pointing to +v; if inpl = 1 as described above.
     if inpl = 2 turn x like for inpl=1 but perpendicular to r

     Returns the matrix mx.
  */

  double al, tmpa;
  Vec3 u1, u2, u3;
  Mat3 mx, m2;

  // transformation matrix from equatorial to orbit plane
  u2 = vnorm (r1);
  u3 = vnorm (v1);

  if (inpl == 2) tmpa = acos(dot(u2,u3)) - M_PI*0.5;
  u1 = u2 * u3;  // vector normal to orbit plane

  // check if r and v colinear. In this case define orbit plane

  u1 = vnorm (u1);
  if (abs(u1) == 0)
   {             // this normally should never happen
    u1[1] = 1.0;
    u1 = u2 * u1;
    u1 = vnorm (u1);
    if (abs(u1) == 0)
     {
      u1[0] = 1.0;
      u1 = u2 * u1;
      u1 = vnorm (u1);
     };
    if (abs(u1) == 0) u1[2] = 1.0;  // last resort
   };

  if (u1[2] > 1.0) u1[2] = 1.0; // to be safe
  if (u1[2] < -1.0) u1[2] = -1.0; // to be safe

  al = atan20(u1[1],u1[0]) + M_PI;
  mx = zrot (al);
  al = -acos (u1[2]);
  m2 = yrot (al);
  mx = m2 * mx;   // conversion into orbit plane
  u2 = mxvct (mx, v1);
  if ((u2[1] == 0) && (u2[0] == 0)) al = 0;   // just in case
  else al = atan20(u2[1],u2[0]);
  m2 = zrot (al);
  mx = m2 * mx;	 // x points to velocity-direction

  // rotate to LVLH-plane
  if (inpl)
   {
    m2 = xrot(M_PI*0.5);
    mx = m2 * mx;
    if (inpl == 2)   // turn x perpendicular to r
     {
      m2 = yrot(tmpa);
      mx = m2 * mx;
     };
   };

  // now yaw and pitch
  if (yaw != 0)
   {
    m2 = zrot(-yaw*degrad);
    mx = m2 * mx;
   };

  if (pitch != 0)
   {
    m2 = yrot(-pitch*degrad);
    mx = m2 * mx;
   };

  return mx;
 }

Mat3 GetYSMat (WinpData* indata, Vec3& r1, Vec3& v1, double yaw, double pitch)
 {
  /* Get matrix mx to transform from the original reference (equatorial)
     into a coordinate system which is defined by the Local Yaw Steering
     law. This is similar to the local attitude defined for GetRVMat
     except that the 0-yaw is pointing into the direction of the
     velocity as projected onto the moving earth surface.
     Returns the matrix mx.
  */
  int sx;
  double psi, phi, vs, sp, cs, c2, c3;
  Vec3 u1;
  Mat3 mx, m2;
  const double mp2 = 2.0*M_PI;
  const double finv = 1.0 / 298.257223563;

  mx = GetRVMat (r1, v1, 0, 0, 2);
  sx = indata->satidx;
  vs = indata->orbelm.n0[sx];
  if (vs <= 0) return mx;  // should never happen!
  vs = 1.0 / vs;
  u1 = vnorm (r1);
  phi = 1.0 - u1[2]*u1[2];
  if (phi < 0) phi = 0;
  else phi = sqrt(phi);  // cos(latitude)
  c3 = phi;
  cs = indata->orbelm.inc[sx];
  cs = cos(cs*degrad);
  if ((phi > 0) && (phi > fabs(cs)))
   {
    sp = cs / phi;  // sin(psi)
    psi = asin(sp);
    cs = cos(psi);  // cos(psi)
    if (cs > 0)
     {
      c2 = psi;
      if (v1[2] < 0) c2 = M_PI - c2;
      if (c2 > mp2) c2 -= mp2;
      if (c2 < 0) c2 += mp2;  // course from North
      c3 = acos(c3);
      c3 = fabs(finv*sin(2.0*c3));  // correction geodetic - geocentric latitude
      if (r1[2] < 0) c3 = -c3;   // southern hemisphere
      m2 = zrot(c2);
      mx = m2 * mx;
      m2 = yrot(c3); // correct for geodetic horizontal
      mx = m2 * mx;
      m2 = zrot(-c2);
      mx = m2 * mx;

      // now the yaw correction part
      cs = (sp - vs*phi) / cs;
      if (cs < M_PI*0.5)
       {
        cs = atan(cs);  // course corrected with regard to moving surface
        cs -= psi;  // delta-yaw for yaw steering
        if (v1[2] < 0) cs = -cs;  // descending orbit
        cs = cs / degrad;
        yaw += cs;
       };
     };
   };

  // now yaw and pitch
  if (yaw != 0)
   {
    m2 = zrot(-yaw*degrad);
    mx = m2 * mx;
   };
  if (pitch != 0)
   {
    m2 = yrot(-pitch*degrad);
    mx = m2 * mx;
   };

  return mx;
 }

/*------------------------- GetSatVis ------------------------------*/

void GetSatVis (SatPosData* satpos, double sa, int sx)
{
  /* get the circle of visibility of satellite */
  const double re = 6378.137;  // Earth equatorial radius
  const double mp2 = 2.0*M_PI;

  double cd, sb, ca, al, da, sd, cl;
  int j;

  cd = satpos->height[sx] + re;
  if (cd > re) cd = re / cd;
  else cd = 1.0;

  sd = sin(acos(cd));
  ca = cos(sa);
  sa = sin(sa);
  al = 0.0000001;  // avoid an occasional error in longitude if al = 0
  satpos->nsatvp = 64;
  da = mp2 / double(satpos->nsatvp);
  for (j = 0; j < satpos->nsatvp; j++)
   {
    sb = cd*sa + sd*ca*cos(al);
    if (fabs(sb) > 1.0) sb = 0;
    satpos->satvlat[sx][j] = asin(sb);
    cl = ca * cos(satpos->satvlat[sx][j]);
    if (fabs(cl) > 1e-50) cl = (cd - sa*sb) / cl;
    else cl = 1.0;
    if (fabs(cl) <= 1.0) cl = acos(cl);
    else cl = 0;
    if (al > M_PI) cl = -cl;

    satpos->satvlng[sx][j] = cl + satpos->lng[sx]*degrad;
    if (satpos->satvlng[sx][j] > mp2) satpos->satvlng[sx][j] -= mp2;
    if (satpos->satvlng[sx][j] < -M_PI) satpos->satvlng[sx][j] += mp2;
    al += da;
   };
}

/*------------------------- GetOccTrans -----------------------------*/

void GetOccTrans(double jd, WinpData* indata, SatPosData* satpos, int sx,
							double ep2, Vec3& r1, Vec3& rk)
 {
  /* Calculate occultation or transit groundtrack.
     INPUT:
           jd : MJD time (UT1)
           sx: satellite index
           ep2: correction for apparent sidereal time
           r1: satellite position in Earth radii
           rk: apparent position of Sun (Moon; RA/DEC) in Earth radii

     OUTPUT:
           The geographic latitude and longitude of the place where
           occultation or transit results will be put into occlat and
           occlng in satpos. (If there is no intersection the latitude
           will be set to 100.0)
           r1 will be re-set to the unit vector from the satellite to the
           intersection point.

           In order to account for atmospheric refraction the procedure is
           run then checked whether the angle of incidence at the target
           site warrents a correction and then possibly run a second time
           with this correction for refraction. Thus only one iteration is
           considered.
  */

  const double flat = 0.996633;  // flatting of the Earth
  int atcorr;
  double lamda, phi, s0, s, dlt, r2, r0;
  double atcang;
  Vec3 eshadow, rint, ve, r1sv, rksv;

  atcorr = 2;
  atcang = 1.0;
  r1sv = r1;
  rksv = rk;

  while (atcorr)
   {
    // adjust for refraction
    if (atcorr == 1)
     {
      s0 = asin(atcang) / degrad;
      if (s0 < 1.4) s0 = 1.4;  // limit it for very small angles
      s0 = 8.40429926257 * pow(s0, -2.01365949653);
     }
    else s0 = 0;

    // adjust for altitude of observing site
    s = indata->geoheight * 0.001 + s0;
    if ((s > -1.0) && (s < 20.0))
     {
      s = 6378.137 / (6378.137 + s);
      r1 *= s;
      rk *= s;
     };

    // adjust for flatting of the Earth
    r1[2]/=flat;
    if (indata->occflg != 3) rk[2]/=flat;
    lamda = 0;

    // intersect shadow axis with Earth
    if (indata->occflg != 3) eshadow = r1 - rk;
    else
     {
      eshadow[0] = -rk[0];
      eshadow[1] = -rk[1];
      eshadow[2] = -rk[2]/flat;
     };
    eshadow = vnorm(eshadow);    // direction vector of shadow
    s0 = - dot(r1, eshadow);   // distance S/C - fundamental plane

    r2 = dot (r1,r1);
    dlt = s0*s0 + 1.0 - r2;
    r0 = 1.0 - dlt;
    if (r0 >= 0) r0 = sqrt (r0);
    else r0 = 2.0;      // distance center of Earth - shadow axis

    // get intersection coordinates

    if (r0 < 1.0)
     {
      if (dlt > 0) dlt = sqrt(dlt);
      else dlt = 0;
      s = s0 - dlt;  // distance S/C - fundamental plane
      if (s >= 0)
       {
        rint = r1 + s * eshadow;
        r1 = rint - r1; // this is needed later for the iridium flare mag
        r1[2] *= flat;
        rint[2] *= flat;    // vector to intersection
        ve = carpol(rint);
        lamda = ve[1] - lsidtim(jd,0,ep2)*0.261799387799; // geogr.coord.
        if (lamda > M_PI) lamda -= 2.0*M_PI;
        if (lamda < (-M_PI)) lamda += 2.0*M_PI;
        phi = sqrt(rint[0]*rint[0] + rint[1]*rint[1])*0.993305615;
        phi = atan2(rint[2],phi);
       }
      else phi = 100.0;
     }
    else phi = 100.0; // no intersection

    // check whether an iteration for refraction is necessary
    if ((atcorr == 2) && (phi < 100.0))
     {
      rint[0] = -rint[0];
      rint[1] = -rint[1];
      rint[2] = -rint[2];
      rint = vnorm(rint);
      eshadow[2] *= flat;
      eshadow = vnorm(eshadow);
      atcang = dot(rint, eshadow);  // sin of elevation
      atcang = fabs(atcang);
      if (indata->refract && (atcang < 0.26)) // correct for angles < 15° if refraction activated
       {
        atcorr = 1;
        r1 = r1sv;
        rk = rksv;
       }
      else atcorr = 0;
     }
    else atcorr = 0;
   };  // while atcorr

  if (phi < 100.0)
   {
    satpos->occlat[sx] = phi / degrad;

    // calculate along-the-surface distance of spot
    if (sx == indata->satidx)
     {
      s = sin(indata->geolat*degrad)*sin(phi)
                     + cos(indata->geolat*degrad)*cos(phi)
                     * cos(indata->geolong*degrad - lamda);
      if (fabs(s) <= 1.0) s = acos(s) * 6378.137;
      satpos->mag = s;
     };
   }
  else satpos->occlat[sx] = 100.0;
  satpos->occlng[sx] = lamda / degrad;
 }

/*------------------------- GetSatPos ------------------------------*/

void GetSatPos (double t, WinpData* indata, SatPosData* satpos, int sx, int rngflg)
 {
  /* Get Position of Satellite at MJD-time t (UTC) with orbit
    elements in indata->orbelm.

    sx is the satellite index
    rngflg = 1 if visibility circle has to be calculated, 0 otherwise
    Place the results in satpos
  */

  const double gm = 2.9755364003E15;  // Gravitation Constant G*Mearth
  const double re = 6378.137;  // Earth equatorial radius in km
  const double j2 = 1.082628E-3;   // J2-Term
  const double mp2 = 2.0*M_PI;
  const double ae = 23454.79106062;  // 1AE -> Earth Radii
  const double f = 1.0 / 298.257223563;  // Earth flattening

  double dt, m0, ran, aper, inc, a, ecc, n0;
  double fac, e, c, s, k, ep2, sa, sh, sj;
  double glat, glong, gheight, slat, slng, shgt, mgsv, flmgsv;
  double iroccra, iroccdec, irmagx, irxflrx, irbeta;
  double qrs, qrm, qd, qeclmg, xi, qs1, qs2, qa;
  double b1, b2, b3, tut1;
  int irflaryst, irix, irxflg;
  int j, sx2, rstflg;
  int dd, mm, yy;
  bool irsgo;

  Sun200 sun;
  Moon200 moon;

  Vec3 r1, rs, rm, v1, s1, s2, s3, u1, gps, ssx;
  Vec3 irs2, rg1;
  Mat3 m1, m2, mx, pmx;

  tut1 = t;
  if (indata->IERSflg) tut1 += indata->Ut1Utc/86400.0;

  qeclmg = 0;  // only needed to correct magnitude for solar eclipse

  // check whether relative satellite positions are to be calculated
  rstflg = 0;

  if (indata->relsat)
   {
    if ((indata->satidx == sx) && (indata->sidxcn > 1)
           && (indata->occflg == 0) && (indata->flarflg == 0))
     {
      rstflg = 1;

      glat = indata->geolat;
      glong = indata->geolong;
      gheight = indata->geoheight;
      sx2 = sx + 1;
      if (sx2 >= indata->sidxcn) sx2 = 0;
      slat = satpos->lat[sx2];
      slng = satpos->lng[sx2];
      shgt = satpos->height[sx2];
      GetSatPos (t, indata, satpos, sx2, 0);
      indata->geolat = satpos->lat[sx2];
      indata->geolong = satpos->lng[sx2];
      indata->geoheight = satpos->height[sx2] * 1000.0;
      satpos->ssun2 = satpos->sunang[sx2];
     };
   };

  flmgsv = 99.0;

  // get sun vector and nutation matrix (and moon vector if moonshine option)

  caldat(t, dd, mm, yy, s);
  dt = t + SetTtUt(indata, yy)/86400.0;
  dt = julcent (dt);
  rs = sun.position(dt);
  rs = eclequ(dt,rs);
  mx = nutmat(dt, ep2, indata->IERSflg);   // nutation
  rs = mxvct(mx,rs);    // apparent coordinates
  rs = aberrat(dt,rs);
  rs *= ae;

  // get moon vector if needed
  if (indata->moonflg || (indata->occflg == 2)
        || indata->eclflg || indata->orbelm.numflg[sx])
   {
    rm = moon.position(dt);
    rm = eclequ(dt,rm);
    rm = mxvct(mx,rm);    // apparent coordinates
   };

  // precession matrix date -> epoch
  if (indata->yrepoch != 0)
   {
    a = (indata->yrepoch - 2000.0) / 100.0;
    pmx = pmatequ (dt, a);
   };

  // prepare orbit calculation

  dt = t - indata->orbelm.mjdep[sx];

  if (indata->orbelm.numflg[sx])
                     NumProp(t, indata, sx, rs, rm, r1, v1);
  else     // analytic models
   {
    if (indata->orbelm.aflg[sx])
     {
      Sgp4(indata, dt, sx, r1, v1);
      m1 = zrot(-ep2/13750.9870831);  // convert from TEME into Mean of Date
      r1 = mxvct (m1, r1);
      v1 = mxvct (m1, v1);
     }

    else  // simple orbit model
     {
      ecc = indata->orbelm.ecc[sx];
      a = indata->orbelm.a[sx];
      if (ecc < 1.0) // elliptic orbit
       {
        n0 = mp2 * indata->orbelm.n0[sx];
        if (a < 1.0) a = 1.0;  // avoid possible crashes later on

        // get current orbit elements ready
        aper = (re / a) / (1.0 - ecc*ecc);
        aper = 1.5 * j2 * aper*aper * n0;
        m0 = indata->orbelm.inc[sx]*degrad;
        ran = -aper * cos(m0) * dt;
        m0 = sin (m0);
        aper = aper * (2.0 - 2.5*m0*m0) * dt;
        ran = indata->orbelm.ra[sx] * degrad + ran;
        aper = indata->orbelm.aper[sx] * degrad + aper;
        if (indata->orbelm.n6dot[sx] == 0) m0 = indata->orbelm.m0[sx]*degrad +
                          (n0 + mp2*indata->orbelm.ndot[sx]*dt)*dt;
        else m0 = indata->orbelm.m0[sx]*degrad +
                    (n0 + mp2*(indata->orbelm.ndot[sx] + indata->orbelm.n6dot[sx]*dt)*dt)*dt;
        inc = indata->orbelm.inc[sx] * degrad;

        // solve Kepler equation
        a = a + indata->orbelm.adot[sx]*dt;
        if (a < 1.0) a = 1.0;  // avoid possible crashes later on
        e = eccanom(m0, ecc);
        fac = sqrt(1.0 - ecc*ecc);
        c = cos(e);
        s = sin(e);
        r1.assign (a*(c - ecc), a*fac*s, 0.0);

        m0 = 1.0 - ecc*c;
        k = sqrt(gm/a);
        v1.assign (-k*s/m0, k*fac*c/m0, 0.0);

        /* convert into reference plane */
        m1 = zrot (-aper);
        m2 = xrot (-inc);
        m1 *= m2;
        m2 = zrot (-ran);
        m2 = m2 * m1;
        r1 = mxvct (m2, r1);
        v1 = mxvct (m2, v1);
        v1 /= 86400.0;
       }
      else  // hyperbolic orbit
       {
        inc = indata->orbelm.inc[sx];
        aper = indata->orbelm.aper[sx];
        c = indata->orbelm.mjdep[sx];
        n0 = indata->orbelm.n0[sx];
        ran = 1.0 - ecc;
        if (ran < 0) n0 /= ran;
        ran = indata->orbelm.ra[sx];

        kepler (gm, c, t, 0, n0, ecc, ran, aper, inc, r1, v1);
        v1 /= 86400.0;
       };
     };  // end of calc for simple orbit model

    // save state vector for numerical calc if needed

    indata->orbelm.x[sx] = r1[0]*1000.0;
    indata->orbelm.y[sx] = r1[1]*1000.0;
    indata->orbelm.z[sx] = r1[2]*1000.0;
    indata->orbelm.vx[sx] = v1[0]*1000.0;
    indata->orbelm.vy[sx] = v1[1]*1000.0;
    indata->orbelm.vz[sx] = v1[2]*1000.0;
    indata->orbelm.mjdcur[sx] = t;

    indata->lndist = 0.0;
   }; // end of analytical model

  r1 = mxvct (mx, r1);  // correct for nutation
  v1 = mxvct (mx, v1);

  if (indata->yrepoch != 0)  // add nutation to precession
   {
    mx = mxtrn(mx);
    pmx = pmx * mx;
   };

  // get range rate and R.A. and Decl.
  if (rstflg == 0)
   {
    satpos->r2 = r1;   // save it for range rate calc
    satpos->v2 = v1;
   };
  s = indata->geolat*degrad;
  if (fabs(s) < 1.53589) s = atan(tan(indata->geolat*degrad)/1.00674);
  s1[0] = 0;
  s1[1] = re + indata->geoheight*0.001;
  if (s1[1] < 0.0) s1[1] = 0.0;
  s1[1] = s1[1]*mp2*cos(s) / 86164.09054; // rotational speed of obs site
  s1[2] = 0;
  m1 = zrot(-lsidtim (tut1, indata->geolong, ep2)*M_PI/12.0);
  s1 = mxvct(m1, s1);

  gps = GeoPos (tut1, ep2, indata->geolat*degrad, indata->geolong*degrad,
                        indata->geoheight, indata->IERSxp, indata->IERSyp) * re;
  s2 = r1 - gps;
  if (indata->IERSflg)  // correct for light time
   {
    c = abs(s2);
    s3 = s2 - v1*c/299792.458;
   }
  else s3 = s2;

  if (indata->yrepoch != 0) s3 = mxvct(pmx, s3);  // adjust for epoch
  s3 = carpol(s3);
  s2 = vnorm(s2);
  ssx = s2;
  satpos->rrate[sx] = dot(v1,s2) - dot(s1,s2);
  if (s3[1] < 0) s3[1] += mp2;
  satpos->RA[sx] = DegFDms(12.0*s3[1]/M_PI);
  satpos->Dec[sx] = DegFDms(s3[2]/degrad);

  // get sub solar point
  s2 = carpol(rs);
  satpos->sunlat = s2[2];   // just preliminary
  satpos->sunlng = s2[1] - lsidtim (tut1, 0, ep2)*M_PI/12.0;
  if (satpos->sunlng > mp2) satpos->sunlng -= mp2;
  if (satpos->sunlng < -M_PI) satpos->sunlng += mp2;
  if (satpos->sunlng > M_PI) satpos->sunlng -= mp2;
  if (fabs(satpos->sunlat) < 1.53589) satpos->sunlat = atan(1.00674*tan(satpos->sunlat));
  satpos->sunlat /= degrad;
  satpos->sunlng /= degrad;

  // get groundtrack
  s3 = r1 / re;
  AppPos (tut1, ep2, indata->geolat*degrad, indata->geolong*degrad,
                     indata->geoheight, 1, s3, satpos->azim[sx], satpos->elev[sx],
                     satpos->range[sx]);

  // Beta angle
  u1 = vnorm(r1)*vnorm(v1);
  satpos->beta[sx] = dot(u1, vnorm(rs));
  if (satpos->beta[sx] > 1.0) satpos->beta[sx] = 1.0;
  if (satpos->beta[sx] < -1.0) satpos->beta[sx] = -1.0;
  satpos->beta[sx] = asin(satpos->beta[sx]) / degrad;

  // check for solar eclipse
  if (indata->eclflg && (sx == indata->satidx))
   {
    indata->eclpow = 0;
    qd = dot(vnorm(rs-s3),vnorm(rm-s3));
    if (fabs(qd) > 1.0) qd = 1.0;  // just in case
    qd = acos(qd);     // distance Sun - Moon
    qrs = 109.123 / abs(rs);
    qrm = abs(rm-s3);
    if (qrm < 1e-9) qrm = abs(rm);
    qrm = 0.272493 / qrm;

    indata->eclang = qd / degrad;

    if (qd < (qrm + qrs))       // we have an eclipse
     {
      indata->eclmag = (qrs + qrm - qd) / (2.0*qrs);
      indata->ecltyp = 1;
      if ((qd <= fabs(qrm - qrs)) || (qd < 1.745e-6))
       {
        if (qrm >= qrs)
         {
          indata->ecltyp = 2; // total
          qeclmg = 20.0;  // just a high value for mag to make it invisible
         }
        else
         {
          indata->ecltyp = 3;           // annular
          indata->eclmag = qrm / qrs;
          qeclmg = qrm / qrs;
          qeclmg = 1.0 - qeclmg*qeclmg;
          if (qeclmg > 1e-12)
           {
            indata->eclpow = qeclmg;
            qeclmg = -2.5 * log10(qeclmg);
           }
          else qeclmg = 20.0;
         };
       }

      else  // get mag difference for partial eclipse
       {
        xi = (qrs*qrs - qrm*qrm + qd*qd) / (2.0*qd);
        qeclmg = fabs(qd - xi)/qrm;

        if ((xi <= -qrs) || (qeclmg > 1.0)) qeclmg = 20.0;
        else
         {
          if (fabs(xi/qrs) > 1.0) qeclmg = 0;
          else
           {
            qa = 2.0*acos(qeclmg);
            qs2 = qrm*qrm*(qa - sin(qa))*0.5;
            if((qd<xi) && (qrm<qrs)) qs2 = M_PI * qrm*qrm - qs2;
            qa = 2.0*acos(xi/qrs);
            qs1 = qrs*qrs*(qa - sin(qa))*0.5;
            qeclmg = M_PI*qrs*qrs - qs1 - qs2;
            qeclmg = qeclmg / (M_PI*qrs*qrs);

            if (qeclmg > 1e-12)
             {
              indata->eclpow = qeclmg;
              qeclmg = -2.5 * log10(qeclmg);
             }
            else qeclmg = 20.0;
           };
         };
       };
     }
    else   // no eclipse
     {
      indata->eclmag = 0.0;
      indata->ecltyp = 0;
     };
   };

  // get the groundtrack coordinates

  rg1 = mxvct(zrot(lsidtim (tut1, 0, ep2)*M_PI/12.0),r1);
  if((indata->IERSxp != 0.0) || (indata->IERSyp != 0.0))
    rg1 = mxvct(PoleMx(indata->IERSxp, indata->IERSyp),rg1);

  s2 = carpol(rg1);
  satpos->lat[sx] = s2[2];   // just preliminary
  sa = satpos->lat[sx];  // keep the geocentric latitude for the GetSatVis call
  satpos->lng[sx] = s2[1];
  if (satpos->lng[sx] > mp2) satpos->lng[sx] -= mp2;
  if (satpos->lng[sx] < -M_PI) satpos->lng[sx] += mp2;
  if (satpos->lng[sx] > M_PI) satpos->lng[sx] -= mp2;

  // get height above ellipsoid and geodetic latitude
  if (abs(r1) > 0.1)
   {
    c = f*(2.0 - f);
    s = r1[0]*r1[0] + r1[1]*r1[1];
    sh = c*r1[2];

    for (j=0; j<4; j++)
     {
      b1 = r1[2] + sh;
      b3 = sqrt(s+b1*b1);
      if (b3 < 1e-5) b3 = sin(satpos->lat[sx]);  // just in case
      else b3 = b1 / b3;
      b2 = 6378.137 / sqrt(1.0 - c*b3*b3);
      sh = b2 * c * b3;
     };

    sh = r1[2] + sh;
    satpos->lat[sx] = atan20(sh,sqrt(s));
    sh = sqrt(s+sh*sh) - b2;
    satpos->height[sx] = sh;
   }
  else satpos->height[sx] = 0;  // this should never happen

  // get sun angles
  AppPos (tut1, ep2, indata->geolat*degrad, indata->geolong*degrad,
                     indata->geoheight, 1, rs, s, satpos->obssun, c);
  satpos->obssun /= degrad;
  satpos->osunaz = s;
  if (satpos->height[sx] > 0) sh = satpos->height[sx]*1000.0;
  else sh = 0;
  AppPos (tut1, ep2, satpos->lat[sx], satpos->lng[sx], sh,
                        1, rs, s, satpos->sunang[sx], c);

  satpos->sunazm[sx] = s;
  s = abs(r1);
  if (s > re) s = asin(re / s);
  else s = M_PI * 0.5;
  s = M_PI * 0.5 - s;
  satpos->sunang[sx] = (satpos->sunang[sx] + s);

  if (indata->moonflg)   // satellite moon angle
   {
     AppPos (tut1, ep2, satpos->lat[sx], satpos->lng[sx], sh,
                         1, rm, sj, satpos->moonang[sx], c);
     satpos->moonang[sx] = (satpos->moonang[sx] + s);
   };

  satpos->range[sx] *= re;
  if (satpos->azim[sx] < 0) satpos->azim[sx] += mp2;
  satpos->azim[sx] /= degrad;
  satpos->elev[sx] /= degrad;

  // get the apparent magnitude of the prime satellite
  if ((indata->maglng > 0) && (sx == indata->satidx))
   {
    dd = 1;
    mgsv = -30.0;
    if (indata->moonflg && (satpos->sunang[sx] < 0))
     {
      if(satpos->moonang[sx] > 0) dd = 0;
     };

    if(dd)   // sun visibility
     {
      s2 = rs*re;
      satpos->mag = -26.74;  // sun's magnitude at earth
     }
    else    // moon visibility
     {
      s2 = rm * re;
      s1 = rs * re;
      s3 = r1 - s2;
      c = abs(s3) / (re * ae);  // distance moon-sat in AU
      s1 -= s2;
      s1 = vnorm(s1);
      s3 = vnorm(s3);
      s = dot(s1, s3);  // cos of phase angle Sun-Moon-Satellite
      if (fabs(s) <= 1.0) s = acos(s) / degrad;
      else s = 180.0;
      s = exp(-s/41.0041888);
      if (c <= 0) c = 1e-30;  // should never happen.
      satpos->mag = 0.23 + 5.0*log10(c) - 2.5*log10(s);  // moon's mag
      mgsv = satpos->mag;  // save magnitude for Iridium Flare calc
     };

    u1 = gps - r1;   // get the phase angle for mag
    s2 -= r1;
    u1 = vnorm(u1);
    s2 = vnorm(s2);
    s = dot(u1, s2);
    s = 0.5 * (1.0 + s);
    if (s < 1e-30) s = 1.0;
    if (satpos->range[sx] > 0)
     {
      satpos->mag += 5.0*log10(satpos->range[sx]*1000.0/indata->maglng)
                      - 2.5*log10(s) + 0.74 + qeclmg; // the 0.74 adjust for geom.albedo
      if ((satpos->elev[sx] > 0.01) && (satpos->elev[sx] < 33.0) && indata->atmxtin) // add extinction:
                                        satpos->mag += 2.9935 - 0.828865 * log(satpos->elev[sx]);
     }
    else satpos->mag = 0;
    flmgsv = satpos->mag;
   };

  satpos->lat[sx] /= degrad;
  satpos->lng[sx] /= degrad;
  satpos->sunang[sx] /= degrad;
  if (indata->moonflg) satpos->moonang[sx] /= degrad;

  // get the circle of visibility
  if (rngflg && (indata->occflg == 0) && (indata->flarflg == 0)) GetSatVis (satpos, sa, sx);

  // calculate occultation track
  if (indata->occflg)
   {
    // select the corresponding vectors
    if (indata->occflg == 1) u1 = rs;
    if (indata->occflg == 2)
     {
      AppPos (tut1, ep2, indata->geolat*degrad, indata->geolong*degrad,
                          indata->geoheight, 1, rm, satpos->omoonaz, satpos->omoonel, c);
      if (c > 0.1) c = 0.272493 / c;
      else c = 0.00452121;
      if (c <= 1.0) c = asin(c) / degrad;  // Moon radius in degrees
      else c = 0.265;
      satpos->omoonrd = c;

      if (indata->sunmplt)
       {
        u1 = rm - rs;
        u1 = vnorm(u1);
        s3 = vnorm(rm);
        satpos->moonphs = dot(u1, s3);  // cos of phase angle of moon
       };
      u1 = rm;
     };
    if (indata->occflg == 3)
     {
      s3[0] = 1.0;
      s3[1] = 15.0*DmsDegF(indata->occra)*degrad;
      s3[2] = DmsDegF(indata->occdec)*degrad;
      s3 = polcar(s3);
      if (indata->yrepoch != 0)
       {
        pmx = mxtrn(pmx);
        u1 = mxvct(pmx, s3);
       }
      else u1 = s3;
     };
    s2 = r1 / re;
    GetOccTrans(tut1, indata, satpos, sx, ep2, s2, u1);
   };

  // calculate flare track or angles
  irsgo = true;
  if ((indata->iridflg == 3) && (sx == indata->satidx) &&
                (indata->flarflg == 0))
   {
    if (indata->iridsol) irxflg = 3;
    else irxflg = 2;
   }
  else irxflg = 0;

  irxflrx = 360.1;  // minimum iridium flare angle
  if (indata->flarflg || irxflg)
   {
    dd = 0;
    satpos->occlat[sx] = 100.0;  // no reflection
    if (indata->moonflg == 0)
     {
      if (satpos->sunang[sx] > 0)
       {
        s2 = rs;
        dd = 1;
       };
     }
    else
     {
      if (satpos->moonang[sx] > 0)
       {
        s2 = rm;   // take the moon for illumination
        dd = 1;
       };
     };

    if (dd)   // got to be illuminated!
     {
      s3 = r1 / re;
      s2 = s2 - s3;  // correct for parallax at S/C position

      if (irxflg)
       {
        irflaryst = indata->flaryst;
        indata->flaryst = 0;
        iroccra = indata->occra;
        indata->occra = 0;
        iroccdec = indata->occdec;
        indata->occdec = -40.0;
        irmagx = satpos->mag;
        irs2 = s2;
       };

      ssx[0] = -ssx[0];
      ssx[1] = -ssx[1];
      ssx[2] = -ssx[2];

      if ((indata->flarmod > 0) || irxflg)
       {
        for (irix = 0; irix <= irxflg; irix++)
         {
          if (irix > 0)
           {
            s2 = irs2;
            if (irix == 1) indata->occra = -120.0;
            else indata->occra = 120.0;
           };

          // calculate track with known surface angle
          s3[0] = 1.0;
          if ((abs(indata->flarmod) == 1) || irxflg)
           {
            // local attitude
            if (indata->flaryst)
                   m1 = GetYSMat (indata, r1, v1, indata->occra, indata->occdec);
            else
             {
              if ((irix == 3) || indata->iridsock) // check Iridium solar panel flare
               {
                irbeta = fabs(satpos->beta[sx]);
                if ((irbeta >= 40.0) && (irbeta <= 65.0)) // Mode 2 Iridium solar panel
                 {
                  m1 = GetRVMat (r1, v1, 90.0, -90.0, 1);
                  s3 = vnorm(rs);
                  s3 = mxvct(m1, s3);
                  s3 = carpol(s3);
                  iroccra = 360.0 - s3[1] / degrad;
                  if (s3[2] > 0.0) iroccdec = 40.0;
                  else iroccdec = -40.0;
                  m2 = zrot(-iroccra*degrad) * m1;
                  m1 = yrot(-iroccdec*degrad)*m2;
                  s3[0] = 1.0;
                 }
                else irsgo = false; // no go for Iridium solar panel flare
               }

              else m1 = GetRVMat (r1, v1, indata->occra, indata->occdec, 1);
             };

            if (irsgo)
             {
              m2 = mxtrn(m1);
              s3[1] = 0;
              s3[2] = 0;
              u1 = mxvct(m2, s3);
             };

            // get the orientation of the Iridium solar panel in local system
            if ((irix == 3) && irsgo)
             {
              m1 = GetRVMat (r1, v1, 0, 0, 1);
              s3 = mxvct(m1, u1);
              s3 = vnorm(s3);
              s3 = carpol(s3);
              iroccra = 360.0 - s3[1] / degrad;
              iroccdec = s3[2] / degrad;
              s3[0] = 1.0;
              s3[1] = 0;
              s3[2] = 0;
             };
           }

          else
           {
            // inertial attitude
            s3[1] = 15.0*DmsDegF(indata->occra)*degrad;
            s3[2] = DmsDegF(indata->occdec)*degrad;
            s3 = polcar(s3);
            if (indata->yrepoch != 0)
             {
              pmx = mxtrn(pmx);
              u1 = mxvct(pmx, s3);
             }
            else u1 = s3;
           };

          // u1 now points toward the mirror normal in apparent equatorial coord.
          if (irsgo)
           {
            s2 = vnorm(s2);
            u1 = vnorm(u1);
            a = dot(s2, u1);
           };
          if ((a >= 0) && (a <= 1.0) && irsgo)
           {
            a = acos(a);  // angle of reflection
            m1 = GetRVMat (s2, u1, 0, 0, 0);
            s3[0] = 1.0;
            s3[1] = 0;
            s3[2] = 0;
            m2 = zrot(-a);
            s3 = mxvct(m2, s3);
            m1 = mxtrn(m1);
            u1 = mxvct(m1, s3);
            u1 = vnorm(u1);  // u1 now points in the direction of the reflection
            if (sx == indata->satidx)  // get flare angle
             {
              indata->flrang = dot(u1,ssx);
              if (indata->flrang > 1.0) indata->flrang = 1.0;
              if (indata->flrang < -1.0) indata->flrang = -1.0;
              indata->flrang = acos(indata->flrang) / degrad;
              if (irxflg)
               {
                if(irxflrx > indata->flrang) irxflrx = indata->flrang;
                else indata->flrang = irxflrx;
               };
             };
            u1[0] = -u1[0];
            u1[1] = -u1[1];
            u1[2] = -u1[2];
            s2 = r1 / re;
            indata->occflg = 3;
            GetOccTrans(tut1, indata, satpos, sx, ep2, s2, u1);
            indata->occflg = 0;

            // get mag for iridium flares or known flares
            if (((indata->iridflg == 3) || indata->flrmagf) && (sx == indata->satidx))
             {
              if (satpos->elev[sx] > 0.00001)
               {
                satpos->mag = 5.0*log10(satpos->range[sx]) -14.4;
                if ((satpos->elev[sx] < 33.0) && indata->atmxtin) // add extinction:
                            satpos->mag += 2.9935 - 0.828865 * log(satpos->elev[sx]);
                a = cos(a);  // illuminated surface
                if (a <= 0) a = 0.0001;
                satpos->mag -= 2.5 * log10(a);
                s2 = vnorm(s2);
                a = dot(s2,ssx);  // flare angle
                if (a <= 0) a = 0.0001;
                if (a >= 1.0) a = 0.9999999999;

                if ((irix == 3) || indata->iridsock) // Iridium solar panel flare
                 {
                  a = acos(a)/degrad;
                  if (a < 3.5) a = -3.867 + 0.993*a; // it's what Iridflar says;
                  else a = -5.6415 + 1.5*a;  // my whimsical addition
                  satpos->mag += a - 0.7;
                 }
                else // normal MMA flares
                 {
                  a = 5.71785 + 3.2437 * log(acos(a)/degrad);
                  if (a < -0.2) a = -0.2; // limit flare angle; sun is only 0.5 deg!
                  satpos->mag += a - 8.7;
                 };
                if (indata->moonflg)
                 {
                  if (mgsv > -30.0) satpos->mag += 26.74 + mgsv;
                  else satpos->mag = 99.0;
                 };
               }
              else satpos->mag = 99.0;  // satellite below horizon
              if (indata->flrmagf && (indata->iridflg < 3))
               {
                if (satpos->mag < 99.0) satpos->mag = satpos->mag + 8.7 + indata->flrmagx;
                if (flmgsv < satpos->mag) satpos->mag = flmgsv;
               };
              if ((irxflg) && (irmagx > satpos->mag)) irmagx = satpos->mag;
             }; // end mag iridium
           };
         };   // for irix

       }  // flarmod > 0
      else
       {
        if (sx == indata->satidx)      // calculate flare angles
         {
          satpos->occlat[sx] = 99.0; // we get reflections
          s3 = gps - r1;
          s3 = vnorm(s3);
          s2 = vnorm(s2);
          a = dot(s2, s3);
          if (fabs(a) <= 1.0) a = acos(a) * 0.5;
          else a = 0;
          m1 = GetRVMat(s2, s3, 0, 0, 0);
          m2 = zrot(a);
          u1[0] = 1.0;
          u1[1] = 0;
          u1[2] = 0;
          u1 = mxvct(m2, u1);
          m1 = mxtrn(m1);
          u1 = mxvct(m1, u1);

          if (abs(indata->flarmod) == 1)
           {
            // local attitude
            if (indata->flaryst) m1 = GetYSMat (indata, r1, v1, 0, 0);
            else m1 = GetRVMat (r1, v1, 0, 0, 1);
            u1 = mxvct(m1, u1);
            u1 = vnorm(u1);
            s3 = carpol(u1);
            indata->occra = 360.0 - s3[1] / degrad;
            indata->occdec = s3[2] / degrad;
           }
          else
           {
            // inertial attitude
            if (indata->solflar)
             {
              qa = dot(vnorm(u1),vnorm(rs));
              if(fabs(qa) > 1.0) qa = 1.0;
              qa = acos(qa);
              indata->occdec = DegFDms(qa/degrad);
             }
            else
             {
              s3 = carpol(u1);
              if (s3[1] < 0) s3[1] += mp2;
              indata->occra = DegFDms(12.0*s3[1]/M_PI);
              indata->occdec = DegFDms(s3[2]/degrad);
             };
           };
          if (indata->cylflg)  // get flare angle for cylindrical flares
           {
            if (indata->flarmod == -2)
             {
              a = 15.0*DmsDegF(indata->occra)*degrad;
              s3[2] = DmsDegF(indata->occdec)*degrad;
              s3[1] = 15.0*DmsDegF(indata->cylra)*degrad;
              s3[0] = DmsDegF(indata->cyldec)*degrad;
             }
            else
             {
              a = indata->occra*degrad;
              s3[2] = indata->occdec*degrad;
              s3[1] = indata->cylra*degrad;
              s3[0] = indata->cyldec*degrad;
             };
            a = a - s3[1];
            a = sin(s3[0])*sin(s3[2]) + cos(s3[0])*cos(s3[2])*cos(a);
            if (a > 1.0) a = 1.0;
            if (a < -1.0) a = -1.0;
            indata->flrang = acos(a) / degrad;
           };  // cylindrical flares
         };  // sx == satidx
       };
      if (irxflg)
       {
        indata->flaryst = irflaryst;
        indata->occra = iroccra;
        indata->occdec = iroccdec;
        satpos->mag = irmagx;

        if ((satpos->occlat[sx] == 100.0) && (indata->maglng == 0))
         {   // make sure flare angle gets displayed
          if (irxflrx < 90.0) satpos->occlat[sx] = 99.0;
         };
        if (satpos->elev[sx] < 0) satpos->occlat[sx] = 100.0;
       };
     };  // if dd
   };

  // restore values if recursive call was done for relative sats
  if (rstflg)
   {
    satpos->ssun1 = satpos->sunang[sx];
    satpos->RA[sx] = satpos->lat[sx2] ; // save position of target for display
    satpos->Dec[sx] = satpos->lng[sx2];
    satpos->sunang[sx] = satpos->height[sx2];
    satpos->lat[sx2] = slat;
    satpos->lng[sx2] = slng;
    satpos->height[sx2] = shgt;
    indata->geolat = glat;
    indata->geolong = glong;
    indata->geoheight = gheight;

    // get range rate correct
    s2 = r1 - satpos->r2;
    s2 = vnorm(s2);
    satpos->rrate[sx] = dot(v1,s2) - dot(satpos->v2,s2);
   };
 }

void EpochState (WinpData* indata, int sx, bool nw)
 // get state vector at epoch in case it's needed for numerical calc

 {
  int savrel;
  double t;

  savrel = indata->relsat;
  indata->relsat = 0;
  t = indata->orbelm.mjdep[sx];

  indata->lastsx = 9999;  // force new initialization of SGP4
  GetSatPos (t, indata, satpos, sx, 0);

  indata->orbelm.x0[sx] = indata->orbelm.x[sx];
  indata->orbelm.y0[sx] = indata->orbelm.y[sx];
  indata->orbelm.z0[sx] = indata->orbelm.z[sx];
  indata->orbelm.vx0[sx] = indata->orbelm.vx[sx];
  indata->orbelm.vy0[sx] = indata->orbelm.vy[sx];
  indata->orbelm.vz0[sx] = indata->orbelm.vz[sx];
  indata->orbelm.numflg[sx] = 0;

  if (nw)  // new orbit set
   {
    indata->orbelm.F107[sx] =  indata->dF107;
    indata->orbelm.FB107[sx] = indata->dFB107;
    indata->orbelm.Kp[sx] = indata->dKp;
    indata->orbelm.Cd[sx] = indata->dCd;
    indata->orbelm.Cr[sx] = indata->dCr;
    indata->orbelm.Am[sx] = indata->dAm;
    indata->orbelm.deltv[sx] = 0.0;
    indata->orbelm.dyaw[sx] = 0.0;
    indata->orbelm.dpitch[sx] = 0.0;
    indata->orbelm.ionflg[sx] = false;
    indata->orbelm.ionsun[sx] = true;
    indata->orbelm.updepoch[sx] = false;
   };

  indata->relsat = savrel;
 }

/********************************************************************/

/*------------------------- swelcm ---------------------------------*/

void swelcm(WinpData* indata, WoutData* outdata)
 {
  // prepare welcoming message and display initial input values
  char outbuf[110];
  char otxt[541];
  double jdm, hh;
  int j, k, sdx;

  FILE* satfle;

  // default data for indata

  indata->tzone = 0;
  indata->tzauto = 1;
  indata->dspyear = 0;
  indata->hdyear = 0;
  DefTime(indata);
  indata->lastmjd = double(indata->hour) + double(indata->minute)/60.0
                                         + double(indata->second)/3600.0;
  indata->lastmjd=mjd(indata->day,indata->month,indata->year,indata->lastmjd);
  indata->duration = 3.1;
  indata->tstep = 1.0 / 5760.0;  // 15 sec
  indata->rt_offset = 0;
  indata->tstpmin = 0;
  indata->tstpsec = 15.0;
  indata->del_tdut = DefTdUt(indata->year);
  indata->mrefmjd = 0;
  indata->geolat = 0;
  indata->geolong = 0;
  indata->geoheight = 0;
  strcpy(indata->geoname,"Default");
  indata->lnchlat = 0;
  indata->lnchlng = 0;
  indata->tmeco = 8.8;
  indata->lnchflg = 0;
  indata->lsouthflg = 0;
  indata->lsitex = 0;
  indata->rendzs = -1;
  indata->rendzt = 0;
  indata->rendtim = 1.3;
  indata->occflg = 0;
  indata->yrepoch = 2000.0;
  indata->occra = 0;
  indata->occdec = 0;
  indata->cylflg = false;
  indata->cylra = 0;
  indata->cyldec = 0;
  indata->az1 = 0;
  indata->el1 = 0;
  indata->az2 = 0;
  indata->el2 = 0;
  indata->deltm = 0;
  indata->azra1flg = 0;
  indata->azra2flg = 0;
  indata->sdaz1 = 0;
  indata->sdel1 = 0;
  indata->sdmxang = 2.0;
  indata->sddirt = 0;
  indata->sddeltm = 1.0;
  indata->sdra1flg = 1;
  indata->sddrctflg = 0;
  indata->sdmvis = false;
  indata->odtmatch = false;
  indata->pastro = false; 
  indata->pnstars = 0; 
  indata->picsize = 0;

  strcpy(indata->svfname,"*satsave.txt");
  strcpy(indata->tlename,"./visual.txt");
  strcpy(indata->tlesave,"*satcal.tle");
  strcpy(indata->wwwname,"www.celestrak.com/NORAD/elements/visual.txt");
  strcpy(indata->idsgn,"      ");
  strcpy(indata->ldsgn,"      ");

  indata->dmsflg = 0;
  indata->filsflg = 0;
  indata->svfirst = 1;
  indata->svtlefst = 1;
  indata->invalflg = 0;
  indata->timer_on = 0;
  indata->timer_del = 0;
  indata->relsat = 0;
  indata->passact = 0;
  indata->satidx = 0;
  indata->lastsx = 9999;
  indata->sidxcn = 0;
  indata->visflg = 1;
  indata->namflg = 0;
  indata->moonflg = 0;
  indata->flarflg = 0;
  indata->flarmod = 1;
  indata->flaryst = 0;
  indata->sunmplt = 0;
  indata->lstgraph = 0;
  indata->ra4dflg = 0;
  indata->minsunel = -6.0;
  indata->minsatel = 10.0;
  indata->maglng = 0;
  indata->frequ = 0;
  indata->mpnight = 0;
  indata->mapbuf.emagrid = 0;
  indata->mapbuf.emagrey = 2;
  indata->skymap = 0;
  indata->skyprj = 0;
  indata->mapprj = 3;
  indata->physmap = false;
  indata->satsky = false;
  indata->mfsz = 10;
  indata->dfsz = 10;
  indata->fbold = true;
  indata->leftm = false;
  indata->skazimuth = 0;
  indata->centmer = 0;
  indata->graphflg = 0;
  indata->npass = 0;
  indata->iridflg = 0;
  indata->iridmag = 1.0;
  indata->iridops = 1;
  indata->iridsol = false;
  indata->iridsock = false;
  indata->iridmess = false;
  indata->iridkm = 50.0;
  indata->wgs84 = 0;
  indata->usesdp4 = true;
  indata->adjflg = false;
  indata->adjtim = 0;
  indata->adjn0 = 0;
  indata->adjra = 0;
  indata->solflar = false;
  indata->eclflg = false;
  indata->ecltyp = 0;
  indata->eclang = 0.0;
  indata->eclmag = 0.0;
  indata->eclpow = 0.0;
  indata->flrang = 0.0;
  indata->flrmagf = false;
  indata->flrmagx = -7.5;

  for (j=0; j<GBL_MXSAT+2; j++)
   {
    indata->orbelm.norad[j] = 0;
    indata->orbelm.aflg[j] = 1;
    for (k=0; k<20; k++) indata->orbelm.fsname[j][k] = ' ';
    indata->orbelm.fsname[j][20] = '\0';
    for (k=0; k<20; k++) indata->orbelm.sname[j][k] = ' ';
    indata->orbelm.sname[j][20] = '\0';
    for (k=0; k<11; k++) indata->orbelm.ldsgn[j][k] = ' ';
    indata->orbelm.ldsgn[j][11] = '\0';
   };

  indata->orbelm.lflg = 0;
  indata->orbelm.nflg = 0;
  indata->orbelm.nrflg = 0;
  indata->orbelm.paflg = 0;
  indata->lorbnorad = 0;
  indata->lorbmjdep = 0;
  indata->lorbrelsat = 0;

  indata->RKFmxitr = 20;
  indata->RKFtol = 1.0e-8;
  indata->dF107 = 160.0;
  indata->dFB107 = 155.0;
  indata->dKp = 4.0;
  indata->dCd = 2.3;
  indata->dCr = 1.3;
  indata->dAm = 0.01;
  indata->orbelm.F107[0] =  indata->dF107;
  indata->orbelm.FB107[0] = indata->dFB107;
  indata->orbelm.Kp[0] = indata->dKp;
  indata->orbelm.Cd[0] = indata->dCd;
  indata->orbelm.Cr[0] = indata->dCr;
  indata->orbelm.Am[0] = indata->dAm;
  indata->orbelm.deltv[0] = 0.0;
  indata->orbelm.dyaw[0] = 0.0;
  indata->orbelm.dpitch[0] = 0.0;
  indata->orbelm.ionflg[0] = false;
  indata->orbelm.ionsun[0] = true;
  indata->orbelm.updepoch[0] = false;

  indata->IERSxp = 0.0;
  indata->IERSxp = 0.0;
  indata->Ut1Utc = 0.0;
  indata->UtcTai = -int(indata->del_tdut - 32.184 + 0.0001);
  indata->IERSflg = false;
  indata->tmje = 1;
  indata->moongrav = true;
  indata->sungrav = true;
  indata->refract = true;
  indata->atmxtin = true;
  indata->lndist = 0.0;
  indata->liftdrag = 0.15;

  sprintf (outbuf, "   *** Welcome to SatCal Version 11.8 ***");
  PutNxtLn (outdata, outbuf);
  PutSpace(outdata);

  // read the configuration data
  if (getcfg (indata) < 1)
   {
    sprintf (outbuf,"Problems reading Configuration File. Using defaults instead.");
    PutNxtLn (outdata, outbuf);
    PutSpace(outdata);
   };

  if (indata->tzauto) indata->tzone = -GBL_timezone/3600.0;
  jdm = indata->lastmjd + indata->tzone/24.0;
  caldat(jdm, indata->day, indata->month, indata->year, hh);
  dms (hh, indata->hour, indata->minute, jdm);
  indata->second = int(jdm);
  sprintf (outbuf, "The startup input values are:");
  PutNxtLn (outdata, outbuf);
  PutSpace(outdata);
  // check for system timezone
  hh = -GBL_timezone/3600.0;
  sprintf(outbuf,"Timezone: %g",indata->tzone);
  if (hh != indata->tzone) strcat
                 (outbuf, " *** Timezone different from set system time! ***");
  PutNxtLn (outdata, outbuf);
  sprintf (outbuf,"Geographic Location:");
  PutNxtLn (outdata, outbuf);
  PutNxtLn (outdata, indata->geoname);
  sprintf(outbuf,"Lat.: %g  Long.: %g  Height: %g",
                 indata->geolat, indata->geolong, indata->geoheight);
  PutNxtLn (outdata, outbuf);
  PutSpace(outdata);

  sprintf(outbuf, "Name of the Orbit Element File:");
  PutNxtLn (outdata, outbuf);
  satfle = fopen (indata->tlename,"r");
  if (satfle != NULL)
   {
    strcpy(outbuf, indata->tlename);
    fclose(satfle);
   }
  else
   {
    indata->tlename[0] = '*';
    indata->tlename[1] = '\0';
    sprintf(outbuf, "*** No valid TLE file selected ***");
   };
  PutNxtLn(outdata, outbuf);
  PutSpace(outdata);

  sprintf(outbuf, "Maximum Sun Elev.: %g", indata->minsunel);
  PutNxtLn (outdata, outbuf);
  sprintf(outbuf, "Minimum Sat Elev.: %g", indata->minsatel);
  PutNxtLn (outdata, outbuf);
  if (indata->visflg) sprintf (outbuf, "Only visible passes");
  else sprintf (outbuf, "All passes");
  PutNxtLn (outdata, outbuf);
  PutSpace(outdata);
  if (indata->mapbuf.emagrey == 0) strcpy(outbuf, "Map background: White");
  if (indata->mapbuf.emagrey == 1) strcpy(outbuf, "Map background: Grey");
  if (indata->mapbuf.emagrey == 2) strcpy(outbuf, "Map background: Black");
  if (indata->mapbuf.emagrid == 0) strcat(outbuf, "   No coordinate grid");
  if (indata->mapbuf.emagrid == 1) strcat(outbuf, "   30 deg coordinate grid");
  if (indata->mapbuf.emagrid == 2) strcat(outbuf, "   10 deg coordinate grid");
  PutNxtLn (outdata, outbuf);
  PutSpace(outdata);
  if (indata->yrepoch < 1900.0) indata->yrepoch = 0;  // just in case
  sprintf(outbuf, "Epoch for R.A./Dec.: %g", indata->yrepoch);
  PutNxtLn (outdata, outbuf);

  // try to load satellites
  if (indata->sidxcn)
   {
    sdx = indata->sidxcn;
    indata->sidxcn = 0;
    satfle = fopen (indata->tlename,"r");
    if (satfle == NULL) return;

    for (j=0; j<sdx; j++)
     {
      fseek (satfle, 0L, SEEK_SET);

      strcpy(indata->orbelm.sname[GBL_MXSAT], indata->orbelm.sname[j]);
      if (tleread(indata, satfle, otxt, 0) >= 1)
       {
        if (indata->sidxcn >= GBL_MXSAT) indata->sidxcn = GBL_MXSAT-1;
        ShiftElm(indata, indata->sidxcn, GBL_MXSAT);
        indata->sidxcn++;
        if (indata->sidxcn > GBL_MXSAT) indata->sidxcn = GBL_MXSAT;
        indata->orbelm.fsname[GBL_MXSAT][0] = '#';
       };
     };
    fclose(satfle);
   };

  if (indata->sidxcn)
   {
    PutSpace(outdata);
    sprintf(outbuf, "Selected Satellites:");
    PutNxtLn (outdata, outbuf);
    PutSpace (outdata);
    for (j=0; j<indata->sidxcn; j++)
     {
      PutNxtLn (outdata,indata->orbelm.fsname[j]);
     };
   };
 }

/***********************************************************************/

/*----------------------- function aosstr -----------------------------*/

void aosstr(double jdm, double azim, double elev, char *dts, int dmflg)
  /*
    Convert the Modified Julian Date jdm into a corresponding string
    *dts which has the format "MMM DD HH:MM:SS  (azim, elev)" (if dmflg = 0)
    or "HH:MM:SS  (azim, elev)"  (if dmflg > 0)
  */
 {
  int dd, mm, yy, deg, mnt, sd;
  double hh, sec;
  char mchr[28];

  // convert MJD into corresponding date and time
  caldat(jdm, dd, mm, yy, hh);
  dms (hh,deg,mnt,sec);
  sd = int(sec+0.4999);

  // store data in their positions
  if (dmflg == 0)
   {
    GetMonth (mm, dts);
    sprintf (mchr," %2i ", dd);
    strcat (dts, mchr);
   }
  else strcpy (dts, " ");

  if (sd == 60)
   {
    if (mnt < 59)
     {
      mnt++;
      sd = 0;
     }
    else
     {
      if (deg < 24)
       {
        deg++;
        mnt = 0;
        sd = 0;
       };
     };
   };

  sprintf(mchr," %2i:%02i:%02i (%3.0lf,%3.0lf)", deg, mnt, sd, azim, elev);
  strcat(dts, mchr);
 }

/*------------------------- cksatok ---------------------------------*/

int cksatok(WinpData* indata, SatPosData* satpos, int sx)
 {
  /* check whether the satellite visibility conditions are met
     RETURN: 1 if conditions are met, 0 otherwise
   */

  const double re = 6378.137;  // Earth equatorial radius

  double cd, ca, sa, sd;
  int satclr;

  satclr = 0;   // check whether AOS conditions met

  if (indata->relsat && (indata->sidxcn > 1) && (indata->occflg == 0)
                                            && (indata->flarflg == 0))
   {     // special case of relative satellite visibility
    cd = satpos->height[sx] + re;
    if (cd > re) cd = re / cd;
    else cd = 1.0;
    cd = acos(cd);
    ca = satpos->sunang[sx] + re;
    if (ca > re) ca = re / ca;
    else ca = 1.0;
    ca = acos(ca);
    ca += cd;   // maximum central angle for line-of-sight
    sa = satpos->lat[sx]*degrad;   // calculate cos of actual sat angle
    sd = satpos->RA[sx]*degrad;
    sa = sin(sa)*sin(sd)
           + cos(sa)*cos(sd)*cos((satpos->lng[sx]-satpos->Dec[sx])*degrad);
    if (cos(ca) < sa)
     {
      satclr = 1;
      if (indata->visflg)
       {
        if ((satpos->ssun1 < 0) || (satpos->ssun2 < 0)) satclr = 0;
       };
     };
   }
  else   // normal case of ground station visibility
   {
    if ((satpos->elev[sx] >= indata->minsatel)
                          && (satpos->obssun < indata->minsunel))
     {
      satclr = 1;
      if (indata->visflg)
       {
        if (indata->moonflg)
         {
          if (satpos->sunang[sx] >= 0) satclr = 0;
          if (satpos->moonang[sx] < 0) satclr = 0;
         }
        else
         {
          if (satpos->sunang[sx] < 0) satclr = 0;
         };
       };  // if visflg
     };
   };  // normal case

  return satclr;
 }

/*------------------------- gpassend ---------------------------------*/

double gpassend (WinpData* indata, SatPosData* satpos, int sx, double tb, double ts)
 {
  /* find the end of the current pass
     sx : index of currently active satellite
     tb : start time
     ts : maximum time of search

     RETURN: the (MJD) time of the end of the pass
  */
  double const dtm = 1.0/1440.0;   // add one minute

  int nend;
  double t;

  t = tb;
  do
   {
    GetSatPos (t, indata, satpos, sx, 0);
    nend = cksatok (indata, satpos, sx);   // check whether AOS conditions met
    if (nend) t += dtm;
    if (t > ts) nend = 0;
   } while (nend);

  return t;
 }

/*------------------------- nextpass ---------------------------------*/

void CkIrid(double t, WinpData* indata, int sx, int k, int *ant,
                         double *tt, double *mgx, double *el2, double *az2)
 {
  /* Check minimum Transit or Flare site or maximum Iridium Flare for k-th field.
     set ant = 1 if first success, set ant = 0 if no further cases
   */
  if (ant[k])
   {
    GetSatPos (t, indata, satpos, sx, 0);

    if (satpos->occlat[sx] < 100.0)
     {
      if (ant[k] == 2)
       {
        ant[k] = 1;
        tt[k] = t;
        mgx[k] = satpos->mag;
        el2[k] = satpos->elev[sx];
        az2[k] = satpos->azim[sx];
       }
      else
       {
        if (ant[k] == 3) ant[k] = 1;
        if (satpos->mag < mgx[k])
         {
          tt[k] = t;
          mgx[k] = satpos->mag;
          el2[k] = satpos->elev[sx];
          az2[k] = satpos->azim[sx];
         };
       };
     }
    else
     {
      if (ant[k] == 1) ant[k] = 0;
     };
   };
 }

int GetIrid(WinpData* indata, int sx, int ir0, double *ti, double *el, double *az)
 {
  /* Get minimum Transit or Flare site or maximum Iridium Flare
     between times ti[0] and ti[2];
     These parameters are like those for nextpass below.

     ir0 : 1 if the Iridium front mirror or the event is OK; 0 otherwise

     OUTPUT:
            ti[1] = MJD of event
            az[1], el[1] : azimuth and elevation (in deg) of satellite at that time
            az[2] : minimum distance or maximum flare
            el[0], el[2] will be set to sat latitude for determination of
                                   ascending or descending Iridium passes.

     RETURN: 1 if a suitable event was found, 2 otherwise
   */
  double const dtm = 1.0/1440.0;   // add one minute
  double const dtsc = 1.157407407e-5;  // add 1 sec

  int satok, j, k;
  int ant[3];
  double t, t2, magmax;
  double tt[3], mgx[3], el2[3], az2[3];

  satok = 2;
  magmax = 1e5;

  t = ti[0];
  t2 = ti[2];
  for (j=0; j<3; j++)
   {
    ant[j] = 2;
    mgx[j] = magmax;
   };
  if (indata->iridflg < 3)
   {
    if (ir0 == 0) return satok;
   }
  else
   {
    if (ir0 == 0) ant[0] = 0;
   };
  el[0] = 100.0;

  // find flares to the minute and decide which Iridium mirror
  while (t < t2)
   {
    if ((indata->iridflg < 3) || indata->iridsock)
                      CkIrid(t, indata, sx, 0, ant, tt, mgx, el2, az2);
    else
     {
      indata->occdec = -40.0;
      indata->occra = 0.0;
      CkIrid(t, indata, sx, 0, ant, tt, mgx, el2, az2); // Iridium front mirror
      indata->occra = -120.0;
      CkIrid(t, indata, sx, 1, ant, tt, mgx, el2, az2); // Iridium left mirror
      indata->occra = 120.0;
      CkIrid(t, indata, sx, 2, ant, tt, mgx, el2, az2); // Iridium right mirror
      if (el[0] > 99.0) el[0] = satpos->lat[sx];
     };
    t += dtm;
   };  // while

  t = t2;  // last point
  if ((indata->iridflg < 3) || indata->iridsock)
   {
    CkIrid(t, indata, sx, 0, ant, tt, mgx, el2, az2);
    if (mgx[0] >= magmax) return satok;
   }
  else
   {
    indata->occra = 0;
    CkIrid(t, indata, sx, 0, ant, tt, mgx, el2, az2);
    indata->occra = -120.0;
    CkIrid(t, indata, sx, 1, ant, tt, mgx, el2, az2);
    indata->occra = 120.0;
    CkIrid(t, indata, sx, 2, ant, tt, mgx, el2, az2);
    k = 3;
    for (j=0; j<3; j++)  // which is the best mirror?
     {
      if (mgx[j] < magmax)
       {
        magmax = mgx[j];
        k = j;
       };
     };

    if (k > 2) return satok;
    if (k)
     {
      if (k == 1) indata->occra = -120.0;  // left mirror
      else indata->occra = 120.0;          // right mirror
      tt[0] = tt[k];
      mgx[0] = mgx[k];
      el2[0] = el2[k];
      az2[0] = az2[k];
     }
    else indata->occra = 0;               // front mirror
   };

  t = tt[0] - dtsc;  // find out which side of time to search for max
  if (indata->iridflg == 3)
   {
    el[2] = satpos->lat[sx];
    if (el[0] > 99.0)  // this should rarely happen
     {
      GetSatPos (t, indata, satpos, sx, 0);
      el[0] = satpos->lat[sx];
     };
   };
  if (t < ti[0])
   {
    t2 = t + dtm;
    t = tt[0];
   }
  else
   {
    ant[0] = 1;
    magmax = mgx[0];
    CkIrid(t, indata, sx, 0, ant, tt, mgx, el2, az2);
    t = tt[0] + 2.0 * dtsc;
    t2 = t + dtm - dtsc;
    if (ant[0])
     {
      if (mgx[0] < magmax)
       {
        t = t - dtm + 2.0*dtsc;
        t2 = tt[0];
       };
     };
   };
  if (t < ti[0]) t = ti[0];
  if (t2 > ti[2]) t2 = ti[2];
  ant[0] = 3;
  satok = 1;

  // find minimum distance or maximum flare
  while (t < t2)
   {
    CkIrid(t, indata, sx, 0, ant, tt, mgx, el2, az2);
    if (ant[0] == 0) t = t2;
    t += dtsc;
   };  // while

  // check whether condition was met
  if (indata->iridflg < 3)
   {
    if (mgx[0] > indata->iridkm) satok = 2;
   }
  else
   {
    if (mgx[0] > indata->iridmag) satok = 2;
   };

  if (satok == 1)  // get it accurate to 0.1 sec
   {
    ant[0] = 3;
    t = tt[0];
    t2 = t;
    magmax = dtsc * 0.1;
    for (j=0; j<9; j++)
     {
      t += magmax;
      t2 -= magmax;
      CkIrid(t, indata, sx, 0, ant, tt, mgx, el2, az2);
      CkIrid(t2, indata, sx, 0, ant, tt, mgx, el2, az2);
     };
    ti[1] = tt[0];
    el[1] = el2[0];
    az[1] = az2[0];

    // get transit distance more accurately
    if ((indata->iridflg < 3) && (mgx[0] < 10.0))  // only for < 10 km needed
     {
      t = ti[1] + magmax;
      t2 = ti[1] - magmax;
      GetSatPos (t, indata, satpos, sx, 0);
      if (satpos->occlat[sx] < 100.0)
       {
        mgx[1] = satpos->mag;
        el2[1] = satpos->occlat[sx];
        az2[1] = satpos->occlng[sx];
        GetSatPos (t2, indata, satpos, sx, 0);
        if (satpos->occlat[sx] < 100.0)
         {
          mgx[2] = satpos->mag;
          el2[2] = satpos->occlat[sx];
          az2[2] = satpos->occlng[sx];
          az2[1] -= indata->geolong;
          az2[2] -= indata->geolong;
          el2[1] -= indata->geolat;
          el2[2] -= indata->geolat;
          if ((fabs(indata->geolat)< 80.0)&&(fabs(az2[1])< 2.0)
                    &&(fabs(az2[2])< 2.0)&&(fabs(el2[1])< 1.0)&&(fabs(el2[2])< 1.0))
           {
            el2[0] = indata->geolat*degrad;
            az2[1] = az2[1]*degrad * cos(el2[0]);
            az2[2] = az2[2]*degrad * cos(el2[0]);
            el2[1] *= degrad;
            el2[2] *= degrad;

            t = az2[1] - az2[2];
            t2 = el2[1] - el2[2];
            t = t*t + t2*t2;
            if (t > 1e-100)
             {
              t = sqrt(t);
              mgx[0] = (az2[1]*el2[2] - az2[2]*el2[1]) / t;
              if (mgx[0] < 0) mgx[0] = -mgx[0];
              mgx[0] = mgx[0] * (6378.14 + indata->geoheight*0.001);
             };
           }
          else    // alternative way if too close to the poles or dateline
           {
            t = 8.0 * (mgx[1]*mgx[1] + mgx[2]*mgx[2] - 2.0*mgx[0]*mgx[0]);
            if (t > 1e-50)
             {
              t2 = mgx[1]*mgx[1] - mgx[2]*mgx[2];
              t = mgx[0]*mgx[0]  - t2*t2 / t;
              if (t >= 0) mgx[0] = sqrt(t);
             };
           };
         };
       };
     };  // mgx[0] < 5.0

    az[2] = mgx[0];
   };

  return satok;
 }

int nextpass(WinpData* indata, int sx, double t, double ts, double *ti, double *el, double *az)
 {
  /* Find the next pass consistent with the visibility conditions
     from (MJD) time t on up to a maximum of ts.
     sx = index for satellite in question

     OUTPUT:
            ti[0], ti[1], ti[2] = start, max and end times of the pass
            az[i], el[i] = azimuth and elevation (in deg) of satellite
                             at the above times.

     RETURN: 1 if a pass was found,
             2 if pass was found but iridflg NOT successful
             0 otherwise
   */
  double const dtm = 1.0/1440.0;   // add one minute
  double const dtsc = 1.157407407e-5;  // add 1 sec

  int satok, satclr, ir0, j;
  double dt, told, t2, e1, e2;
  double tmax, elmax, azmax;
  int nrlsflg;  // =1 if NOT relative satellite mode

  told = t;
  satok = 0;
  tmax = t - 1.0;
  elmax = -100.0;
  azmax = 0;
  ir0 = 0;
  if (indata->iridflg == 3) indata->occra = 0;  // Iridium front mirror

  // first find the end of current pass (if active)
  if (indata->passact) t = gpassend (indata, satpos, sx, t, ts);

  if (indata->relsat && (indata->sidxcn > 1) && (indata->occflg == 0)
                                             && (indata->flarflg == 0)) nrlsflg = 0;
  else nrlsflg = 1;

  do         // find next contact times accurate to the minute
   {
    GetSatPos (t, indata, satpos, sx, 0);
    satclr = cksatok (indata, satpos, sx);   // check whether AOS conditions met
    if (satok)
     {
      if (satpos->occlat[sx] < 100.0) ir0 = 1;
      if (satclr)   // check for maximum
       {
        if (elmax < satpos->elev[sx])
         {
          elmax = satpos->elev[sx];
          azmax = satpos->azim[sx];
          tmax = t;
         };
       }
      else         // end of pass found
       {
        ti[2] = t;
        if (nrlsflg)
         {
          el[2] = satpos->elev[sx];
          az[2] = satpos->azim[sx];
         }
        else
         {
          el[2] = satpos->lat[sx];
          az[2] = satpos->lng[sx];
         };
        t = ts + 1.0;
       };
     }
    else
     {
      if (satclr)     // start of pass found
       {
        if (satpos->occlat[sx] < 100.0) ir0 = 1;
        satok = 1;
        elmax = satpos->elev[sx];
        azmax = satpos->azim[sx];
        tmax = t;
        ti[0] = t;
        if (nrlsflg)
         {
          el[0] = satpos->elev[sx];
          az[0] = satpos->azim[sx];
         }
        else
         {
          el[0] = satpos->lat[sx];
          az[0] = satpos->lng[sx];
         };
       };
     };

    dt = dtm;
    if((indata->minsatel >= 0) && nrlsflg == 1)
     {
      if(satpos->elev[sx] < -60.0) dt = 0.0208;  // 30 min
      if(satpos->elev[sx] < -25.0) dt = 0.0069;  // 10 min
      if((satpos->obssun - 15.0) > indata->minsunel) dt = 0.0416667; // 1 hour
     };
    t += dt;
   } while (t <= ts);

  if (satok)
   {
    ti[1] = tmax;
    el[1] = elmax;
    az[1] = azmax;
    if (satclr)    // the end of pass extends beyond the interval
     {
      ti[2] = ts;
      if (nrlsflg)
       {
        el[2] = satpos->elev[sx];
        az[2] = satpos->azim[sx];
       }
      else
       {
        el[2] = satpos->lat[sx];
        az[2] = satpos->lng[sx];
       };
     };

    // now get the times accurate to the second
    if (ti[0] > told)      // start time
     {
      dt = dtm * 0.5;
      t = ti[0] - dt;

      for (j=0; j<6; j++)
       {
        dt *= 0.5;
        GetSatPos (t, indata, satpos, sx, 0);
        if(cksatok (indata, satpos, sx))
         {
          if (satpos->occlat[sx] < 100.0) ir0 = 1;
          ti[0] = t;
          if (nrlsflg)
           {
            el[0] = satpos->elev[sx];
            az[0] = satpos->azim[sx];
           }
          else
           {
            el[0] = satpos->lat[sx];
            az[0] = satpos->lng[sx];
           };
          t -= dt;
         }
        else t += dt;
       };
     };

    if ((ti[2] > told) && (ti[2] < ts))  // end time
     {
      dt = dtm * 0.5;
      t = ti[2] - dt;
      for (j=0; j<6; j++)
       {
        dt *= 0.5;
        GetSatPos (t, indata, satpos, sx, 0);
        if(cksatok (indata, satpos, sx)) t += dt;
        else
         {
          if (satpos->occlat[sx] < 100.0) ir0 = 1;
          ti[2] = t;
          if (nrlsflg)
           {
            el[2] = satpos->elev[sx];
            az[2] = satpos->azim[sx];
           }
          else
           {
            el[2] = satpos->lat[sx];
            az[2] = satpos->lng[sx];
           };
          t -= dt;
         };
       };
     };

    if (indata->iridflg) satok = GetIrid(indata, sx, ir0, ti, el, az);  // transit, flares, iridium flares
    else    // normal max elevation
     {
      if (ti[1] > ti[2])    // max time at end
       {
        ti[1] = ti[2];
        el[1] = el[2];
        az[1] = az[2];
       }
      else
       {
        if (ti[0] > ti[1])  // max time at beginning
         {
          ti[1] = ti[0];
          el[1] = el[0];
          az[1] = az[0];
         }
        else               // max time in-between
         {
          t = ti[1] - dtsc;
          GetSatPos (t, indata, satpos, sx, 0);
          e1 = satpos->elev[sx];
          t2 = ti[1] + dtsc;
          GetSatPos (t2, indata, satpos, sx, 0);
          e2 = satpos->elev[sx];

          if (nrlsflg && (e1 != e2))
           {
            if (e1 > e2)
             {
              t = ti[1] - dtm;
              t2 = ti[1];
              if (t < ti[0]) t = ti[0];
             }
            else
             {
              t = ti[1];
              t2 = ti[1] + dtm;
              if (t2 > ti[2]) t2 = ti[2];
             };

            elmax = -100.0;
            while (t < t2)
             {
              GetSatPos (t, indata, satpos, sx, 0);
              if (satpos->elev[sx] > el[1])
               {
                ti[1] = t;
                el[1] = satpos->elev[sx];
                elmax = el[1];
                az[1] = satpos->azim[sx];
               };
              if (elmax > satpos->elev[sx]) t = t2;
              else t += dtsc;
             };  // while
           };  // e1 != e2
         };  // max time in between
       };  // max time
     }; // if iridflg
   };  // if satok

  return satok;
 };

/*------------------------- lasttpass ---------------------------------*/

double gpassbeg (WinpData* indata, SatPosData* satpos, int sx, double tb, double ts)
 {
  /* find the beginning of the current pass
     sx : index of currently active satellite
     tb : start time
     ts : end time of search

     RETURN: the (MJD) time of the end of the pass
  */
  double const dtm = 1.0/1440.0;   // add one minute

  int nend;
  double t;

  t = tb;

  do
   {
    GetSatPos (t, indata, satpos, sx, 0);
    nend = cksatok (indata, satpos, sx);   // check whether AOS conditions met
    if (nend) t -= dtm;
    if (t < ts) nend = 0;
   } while (nend);

  return t;
 }

int lastpass(WinpData* indata, int sx, double t, double ts, double *ti, double *el, double *az)
 {
  /* Find the last pass consistent with the visibility conditions
     from (MJD) time t on down to a minimum of ts.
     sx = index for satellite in question

     OUTPUT:
            ti[1], ti[2], ti[3] = start, max and end times of the pass
            az[i], el[i] = azimuth and elevation (in deg) of satellite
                             at the above times.

    RETURN: 1 if a pass was found, 0 otherwise
  */
  double const dtm = 1.0/1440.0;   // add one minute
  double const dtsc = 1.157407407e-5;  // add 1 sec

  int satok, satclr, j;
  double dt, told, t2, e1, e2;
  double tmax, elmax, azmax;
  int nrlsflg;  // =1 if NOT relative satellite mode

  told = t;
  satok = 0;
  tmax = t + 1.0;
  elmax = -100.0;
  azmax = 0;

  // first find the beginning of current pass (if active)
  if (indata->passact) t = gpassbeg (indata, satpos, sx, t, ts);
  if (indata->relsat && (indata->sidxcn > 1) && (indata->occflg == 0)
                                      && (indata->flarflg == 0)) nrlsflg = 0;
  else nrlsflg = 1;

  do         // find next contact times accurate to the minute
   {
    GetSatPos (t, indata, satpos, sx, 0);
    satclr = cksatok (indata, satpos, sx);   // check whether AOS conditions met
    if (satok)
     {
      if (satclr)   // check for maximum
       {
        if (elmax < satpos->elev[sx])
         {
          elmax = satpos->elev[sx];
          azmax = satpos->azim[sx];
          tmax = t;
         };
       }
      else         // end of pass found
       {
        ti[2] = t;
        if (nrlsflg)
         {
          el[2] = satpos->elev[sx];
          az[2] = satpos->azim[sx];
         }
        else
         {
          el[2] = satpos->lat[sx];
          az[2] = satpos->lng[sx];
         };
        t = ts - 1.0;
       };
     }
    else
     {
      if (satclr)     // start of pass found
       {
        satok = 1;
        elmax = satpos->elev[sx];
        azmax = satpos->azim[sx];
        tmax = t;
        ti[0] = t;
        if (nrlsflg)
         {
          el[0] = satpos->elev[sx];
          az[0] = satpos->azim[sx];
         }
        else
         {
          el[0] = satpos->lat[sx];
          az[0] = satpos->lng[sx];
         };
       };
     };

    dt = dtm;
    if((indata->minsatel >= 0) && nrlsflg == 1)
     {
      if(satpos->elev[sx] < -60.0) dt = 0.0208;  // 30 min
      if(satpos->elev[sx] < -25.0) dt = 0.0069;  // 10 min
     };
    t -= dt;
   } while (t >= ts);

  if (satok)
   {
    ti[1] = tmax;
    el[1] = elmax;
    az[1] = azmax;
    if (satclr)    // the end of pass extends beyond the interval
     {
      ti[2] = ts;
      if (nrlsflg)
       {
        el[2] = satpos->elev[sx];
        az[2] = satpos->azim[sx];
       }
      else
       {
        el[2] = satpos->lat[sx];
        az[2] = satpos->lng[sx];
       };
     };

    // now get the times accurate to the second
    if (ti[0] < told)      // start time
     {
      t = ti[0] + dtm;
      for (j=0; j<60; j++)
       {
        GetSatPos (t, indata, satpos, sx, 0);
        if(cksatok (indata, satpos, sx))
         {
          ti[0] = t;
          if (nrlsflg)
           {
            el[0] = satpos->elev[sx];
            az[0] = satpos->azim[sx];
           }
          else
           {
            el[0] = satpos->lat[sx];
            az[0] = satpos->lng[sx];
           };
          break;
         };

        t -= dtsc;  // subtract 1 sec
       };
     };

    if ((ti[2] < told) && (ti[2] > ts))  // end time
     {
      t = ti[2] + dtm;
      for (j=0; j<60; j++)
       {
        GetSatPos (t, indata, satpos, sx, 0);
        if(cksatok (indata, satpos, sx) == 0)
         {
          ti[2] = t;
          if (nrlsflg)
           {
            el[2] = satpos->elev[sx];
            az[2] = satpos->azim[sx];
           }
          else
           {
            el[2] = satpos->lat[sx];
            az[2] = satpos->lng[sx];
           };
          break;
         };
        t -= dtsc;  // subtract 1 sec
       };
     };

    if (ti[1] < ti[2])    // max time at end
     {
      ti[1] = ti[2];
      el[1] = el[2];
      az[1] = az[2];
     }
    else
     {
      if (ti[0] < ti[1])  // max time at beginning
       {
        ti[1] = ti[0];
        el[1] = el[0];
        az[1] = az[0];
       }
      else               // max time in-between
       {
        t = ti[1] + dtsc;
        GetSatPos (t, indata, satpos, sx, 0);
        e1 = satpos->elev[sx];
        t2 = ti[1] - dtsc;
        GetSatPos (t2, indata, satpos, sx, 0);
        e2 = satpos->elev[sx];
        if (nrlsflg && (e1 != e2))
         {
          if (e1 > e2)
           {
            t = ti[1] + dtm;
            t2 = ti[1];
            if (t > ti[0]) t = ti[0];
           }
          else
           {
            t = ti[1];
            t2 = ti[1] - dtm;
            if (t2 < ti[2]) t2 = ti[2];
           };

          elmax = -100.0;
          while (t > t2)
           {
            GetSatPos (t, indata, satpos, sx, 0);
            if (satpos->elev[sx] > el[1])
             {
              ti[1] = t;
              el[1] = satpos->elev[sx];
              elmax = el[1];
              az[1] = satpos->azim[sx];
             };
            if (elmax > satpos->elev[sx]) t = t2;
            else t -= dtsc;
           };  // while
         };  // e1 != e2
       };
     };
   };  // if satok

  return satok;
 };

/*------------------------- aospass ---------------------------------*/

void ADisDir (double az1, double el1, double az2, double el2, double &d, double &s)
 {
  /* calculate great circle distance d and direction s from point
     (az1,el1) to (az2,el2).
     az1, el1, az2, el2 are given in radians.
     d will be given by its cosine and s is given as an angle in degrees.
  */

  d = sin(el2)*sin(el1) + cos(el2)*cos(el1)*cos(az2 - az1);
  s = 1.0 - d*d;
  if (s > 0) s = sqrt(s);  // sin
  else s = 0;
  s = s * cos(el1);
  if (fabs(s) > 1e-40)
   {
    s = (sin(el2) - sin(el1)*d) / s;
    if (fabs(s) <= 1.0)
     {
      s = acos(s) / degrad;
      if (sin(az2 - az1) < 0) s = 360.0 - s;
     };
   }
  else s = 0;
 };

void SunMoonTr (WinpData* indata, double ti, int sx, double lckm,
                                  int cklc, int &lcok, char* mtxt)
 {
  /* Calculate duration and direction of Sun/Moon transit at time ti
     as well as the width of the zone where it can be seen
     and output result to mtxt.
     lckm is the minimum distance to the local site.
     If cklc is 1 it will be checked whether lckm is less than the width
     of the zone in which case lcok will be set to 1. If cklc is 1 and
     lckm is greater than the width lcok will be set to 2.
  */
  int relsav;
  double latsav, lngsav;
  double el1, el2, az1, az2, s, d, t2, ht1, rn2;
  double ox1, ox2, oy1, oy2;

  relsav = indata->relsat;
  latsav = indata->geolat;
  lngsav = indata->geolong;

  GetSatPos (ti, indata, satpos, sx, 0);

  // do the calculations at the occultation site
  indata->geolat = satpos->occlat[sx];
  indata->geolong = satpos->occlng[sx];
  GetSatPos (ti, indata, satpos, sx, 0);
  el1 = satpos->elev[sx] * degrad;
  az1 = satpos->azim[sx] * degrad;
  ox1 = satpos->occlng[sx];
  oy1 = satpos->occlat[sx];
  rn2 = satpos->range[sx];
  t2 = ti + 1.0 / 86400.0;  // add one second
  GetSatPos (t2, indata, satpos, sx, 0);
  el2 = satpos->elev[sx] * degrad;
  az2 = satpos->azim[sx] * degrad;
  ox2 = satpos->occlng[sx];
  oy2 = satpos->occlat[sx];

  // width of zone
  if ((oy1 < 100.0) && (oy2 < 100.0))
   {
    ox1 *= degrad;
    oy1 *= degrad;
    ox2 *= degrad;
    oy2 *= degrad;
    ADisDir (ox1, oy1, ox2, oy2, d, s);
    ht1 = s * degrad;
   }
  else ht1 = az1;

  ht1 -= az1;  // angle between shadow path and azimuth of the Sun
  if (indata->occflg == 1) d = 0.00462516;  // tan of half a solar diameter
  else d = tan(satpos->omoonrd * degrad);
  d *= rn2;
  s = sin(el1);
  if (s > 1.0e-10) s = d / s;
  else s = d;
  d = d * cos(ht1);    // take an ellipse in the plane as projected shadow
  s = s * sin(ht1);
  ht1 = sqrt(d*d + s*s);
  if (ht1 > 50.0) ht1 = 50.0;  // beyond the limit the calc is not reliable

  // speed and direction
  ADisDir (az1, el1, az2, el2, d, s);
  if (fabs(d) < 1.0) d = acos(d) / degrad;
  else d = 0;
  if (fabs(d) > 1e-40) d = 0.53 / d;
  if ((ht1 >= 1000.0) || (d < 0.05) || (d > 30.0)) strcpy (mtxt, " ");
  else
   {
    sprintf(mtxt,"   %4.1lf km %4.1lf sec %4.0lf", ht1, d, s);
    decdot(mtxt);
   };

  if (ht1 >= 50.0) mtxt[3] = '>';
  if (cklc)
   {
    if (lckm <= ht1) lcok = 1;
    else lcok = 2;
   };
  indata->relsat = relsav;
  indata->geolat = latsav;
  indata->geolong = lngsav;
 };

void aospass(WinpData* indata, WoutData* outdata)
{
 /* Calculate the AOS times and times of maximum elevation for the
    various passes of the satellites(s) and place them in outdata
  */
  int sx, j, satok;
  int rlsflg, sx2, cklc;
  double t, ts;
  double ti[3], el[3], az[3], msunsav, irkmsav;
  char otxt[GBL_MAXCPL];
  char mtxt[61];

  indata->npass = 0;
  indata->lorbnorad = 0;
  indata->lorbmjdep = 0;
  indata->lorbrelsat = indata->relsat;
  outdata->curlines = 0;
  msunsav = indata->minsunel;
  irkmsav = indata->iridkm;

  if (indata->sidxcn < 1)
   {
    PutSpace(outdata);
    sprintf (otxt, " *** No satellite has been selected ***");
    PutNxtLn (outdata, otxt);
    PutSpace(outdata);
    sprintf (otxt, " Use the Input / Select Satellite menu");
    PutNxtLn (outdata, otxt);
    sprintf (otxt, " (and don't forget to press the SELECT button)");
    PutNxtLn (outdata, otxt);
    PutSpace(outdata);
    return;
   }

  cklc = 0;
  if ((indata->iridflg==1) && ((indata->occflg==1) || (indata->occflg==2)))
   {
    if (irkmsav <= 0)
     {
      indata->iridkm = 35.0;
      cklc = 1;
     };
   };
  if ((indata->iridflg > 0) && (indata->iridkm == 0))
   {
    PutSpace(outdata);
    sprintf (otxt, " *** Need distance > 0 for occultations or flares ***");
    PutNxtLn (outdata, otxt);
    sprintf (otxt, " *** except for Sun or Moon Transits ***");
    PutNxtLn (outdata, otxt);
    PutSpace(outdata);
    return;
   };

  strcpy (otxt," Geographic Location: ");
  strcat (otxt,indata->geoname);
  PutNxtLn (outdata, otxt);
  sprintf(otxt," Lat.: %g   Long.: %g   Height: %g",
                                 indata->geolat, indata->geolong, indata->geoheight);
  PutNxtLn (outdata, otxt);
  sx = indata->satidx;

  if (indata->relsat && (indata->sidxcn > 1) && (indata->occflg == 0)
                                             && (indata->flarflg == 0))
   {
    rlsflg = 1;
    sx2 = sx + 1;
    if (sx2 >= indata->sidxcn) sx2 = 0;
   }
  else rlsflg = 0;

  PutSpace(outdata);
  if (indata->iridflg == 1)
   {
    if (indata->occflg == 1)
     {
      strcpy(otxt, " >>> Sun Transit");
      indata->minsunel = 90.0;  // the sun must be up for this anyway!
     };
    if (indata->occflg == 2) strcpy(otxt, " >>> Moon Transit");
    if (indata->occflg == 3) sprintf(otxt,
                      " >>> Occultation  R.A.: %g  Decl.: %g", indata->occra, indata->occdec);
   };

  if (indata->iridflg == 2)
   {
    if(indata->flarmod==1) sprintf(otxt," >>> Flares. Yaw: %g  Pitch: %g",indata->occra,indata->occdec);
    else sprintf(otxt, " >>> Flares. R.A.: %g  Decl.: %g", indata->occra, indata->occdec);
   };

  if (indata->iridflg) PutNxtLn (outdata, otxt);
  strcpy(otxt, " Satellite: ");
  strcat(otxt, indata->orbelm.fsname[sx]);
  PutNxtLn (outdata, otxt);
  if (rlsflg)
   {
    strcpy(otxt, " ");
    strcat(otxt, indata->orbelm.fsname[sx2]);
    PutNxtLn (outdata, otxt);
   };

  sprintf(otxt,
      " =======================================================================");
  PutNxtLn (outdata, otxt);
  indata->lorbnorad = indata->orbelm.norad[sx];
  indata->lorbmjdep = indata->orbelm.mjdep[sx];
  if (rlsflg) strcpy (otxt, " ");
  else sprintf(otxt," min.Sat.Elev.: %g   max.Sun Elev.: %g",indata->minsatel,indata->minsunel);
  if (indata->visflg)
   {
    if (indata->moonflg) strcat(otxt,"   Moonshine passes only");
    else strcat(otxt,"   visible passes only");
   };

  PutNxtLn (outdata, otxt);
  sprintf(otxt," Year: %4i  +++ Timezone: %g +++", indata->year, indata->tzone);
  PutNxtLn (outdata, otxt);
  PutSpace(outdata);
  if (rlsflg) sprintf (otxt,"       Start (Long.,Lat.)     End (Long.,Lat.)");
  else
   {
    if (indata->iridflg) sprintf (otxt,"     Event Time (Azim.,Elev.)   Distance");
    else sprintf (otxt,
           "    Start (Azim.,Elev.)    Maximum (Azim.,Elev.)  End (Azim.,Elev.)");
   };
  PutNxtLnT (outdata, otxt);

  ts = double(indata->hour) - indata->tzone
                     + double(indata->minute)/60.0 + double(indata->second)/3600.0;
  t = mjd(indata->day, indata->month, indata->year, ts);
  ts = t + indata->duration;

  do
   {
    satok = nextpass(indata, sx, t, ts, ti, el, az);
    if (satok)
     {
      if (indata->iridflg)
       {
        if (satok < 2)
         {
          aosstr((ti[1]+indata->tzone/24.0), az[1], el[1], mtxt, 0);
          if (indata->iridflg < 3) strcpy(otxt, ".");
          else strcpy(otxt, " ");
          strcat(otxt, mtxt);
          sprintf(mtxt," %8.1lf km", az[2]);
          decdot(mtxt);
          strcat(otxt, mtxt);
          if((indata->iridflg==1) && ((indata->occflg==1) || (indata->occflg==2)))
           {
            SunMoonTr (indata, ti[1], sx, az[2], cklc, satok, mtxt);
            strcat(otxt, mtxt);
           };
         };
       }
      else
       {
        strcpy (otxt, ".");
        for (j=0; j<3; j++)
         {
          if ((j != 1) || (rlsflg == 0))
           {
            aosstr((ti[j]+indata->tzone/24.0), az[j], el[j], mtxt, j);
            strcat(otxt, mtxt);
           };
         };
       };  // iridflg

      if (satok == 1)
       {
        if (indata->npass % 10 == 0) PutSpace(outdata);
        PutNxtLnT (outdata, otxt);
        if (indata->npass < GBL_MAXPASS)
         {
          indata->tpass[indata->npass] = ti[0];
          indata->npass++;
         };
       };  // satok == 1
      t = ti[2] + 1.0/1440.0;   // add one minute
      if ((indata->orbelm.n0[sx] > 14.0) && (indata->relsat == 0))
      t += 1.0/24.0;  // add one hour
     };  // if satok

    if (t > ts) satok = 0;
   } while (satok);

  if (indata->npass < 1)
   {
    PutSpace(outdata);
    sprintf (otxt, " *** No suitable pass found within time interval ***");
    PutNxtLn (outdata, otxt);
    PutSpace(outdata);
   };

  indata->minsunel = msunsav;
  indata->iridkm = irkmsav;

  // open the output file
  if (indata->filsflg == 1)    // print the window if asked
   {
    if (indata->svfirst) outptr = fopen (indata->svfname,"w");
    else outptr = fopen (indata->svfname,"a");
    indata->svfirst = 0;
    fprintf(outptr,"\n\n\n");
    for (j=0; j<outdata->curlines; j++)
     {
      fputs (outdata->wbuf[j], outptr);
      fprintf(outptr,"\n");
     };
   };

  if (outptr != NULL)
   {
    fclose(outptr);
    outptr = NULL;
   };
}

/*------------------------- allsataos ---------------------------------*/

void ShiftElm(WinpData* indata, int sx1, int sx2)
 {
  // shift orbit elements from position sx2 to sx1
  indata->lastsx = 9999;  // reset sx marker for SGP4 initialization
  indata->orbelm.norad[sx1] = indata->orbelm.norad[sx2];

  strcpy(indata->orbelm.fsname[sx1],indata->orbelm.fsname[sx2]);
  strcpy(indata->orbelm.ldsgn[sx1],indata->orbelm.ldsgn[sx2]);
  indata->orbelm.mjdep[sx1] = indata->orbelm.mjdep[sx2];
  indata->orbelm.a[sx1] = indata->orbelm.a[sx2];
  indata->orbelm.ra[sx1] = indata->orbelm.ra[sx2];
  indata->orbelm.aper[sx1] = indata->orbelm.aper[sx2];
  indata->orbelm.inc[sx1] = indata->orbelm.inc[sx2];
  indata->orbelm.ecc[sx1] = indata->orbelm.ecc[sx2];
  indata->orbelm.m0[sx1] = indata->orbelm.m0[sx2];
  indata->orbelm.adot[sx1] = indata->orbelm.adot[sx2];
  indata->orbelm.n0[sx1] = indata->orbelm.n0[sx2];
  indata->orbelm.ndot[sx1] = indata->orbelm.ndot[sx2];
  indata->orbelm.aflg[sx1] = indata->orbelm.aflg[sx2];
  indata->orbelm.n6dot[sx1] = indata->orbelm.n6dot[sx2];
  indata->orbelm.bstar[sx1] = indata->orbelm.bstar[sx2];
  indata->orbelm.numflg[sx1] = indata->orbelm.numflg[sx2];

  indata->orbelm.x0[sx1] = indata->orbelm.x0[sx2];
  indata->orbelm.y0[sx1] = indata->orbelm.y0[sx2];
  indata->orbelm.z0[sx1] = indata->orbelm.z0[sx2];
  indata->orbelm.vx0[sx1] = indata->orbelm.vx0[sx2];
  indata->orbelm.vy0[sx1] = indata->orbelm.vy0[sx2];
  indata->orbelm.vz0[sx1] = indata->orbelm.vz0[sx2];
  indata->orbelm.mjdcur[sx1] = indata->orbelm.mjdcur[sx2];
  indata->orbelm.x[sx1] = indata->orbelm.x[sx2];
  indata->orbelm.y[sx1] = indata->orbelm.y[sx2];
  indata->orbelm.z[sx1] = indata->orbelm.z[sx2];
  indata->orbelm.vx[sx1] = indata->orbelm.vx[sx2];
  indata->orbelm.vy[sx1] = indata->orbelm.vy[sx2];
  indata->orbelm.vz[sx1] = indata->orbelm.vz[sx2];

  indata->orbelm.F107[sx1] =  indata->orbelm.F107[sx2];
  indata->orbelm.FB107[sx1] = indata->orbelm.FB107[sx2];
  indata->orbelm.Kp[sx1] = indata->orbelm.Kp[sx2];
  indata->orbelm.Cd[sx1] = indata->orbelm.Cd[sx2];
  indata->orbelm.Cr[sx1] = indata->orbelm.Cr[sx2];
  indata->orbelm.Am[sx1] = indata->orbelm.Am[sx2];
  indata->orbelm.deltv[sx1] = indata->orbelm.deltv[sx2];
  indata->orbelm.dyaw[sx1] = indata->orbelm.dyaw[sx2];
  indata->orbelm.dpitch[sx1] = indata->orbelm.dpitch[sx2];
  indata->orbelm.ionflg[sx1] = indata->orbelm.ionflg[sx2];
  indata->orbelm.ionsun[sx1] = indata->orbelm.ionsun[sx2];
  indata->orbelm.updepoch[sx1] = indata->orbelm.updepoch[sx2];
 }

void CpMMM(char* tmfld, char* lin1)
 {
  // copy the first 16 characters from lin1 to tmfld by
  // adding a character according to the month (first three chars)
  // this serves to get the month sorted right
  int j;
  char mchr[4];
  char tmtmp[17];

  strncpy(tmtmp, lin1, 16);
  for (j=0; j<16; j++) tmfld[j+1] = tmtmp[j];
  mchr[0] = tmtmp[0];
  mchr[1] = tmtmp[1];
  mchr[2] = tmtmp[2];
  mchr[3] = '\0';
  tmfld[0] = 'X';  // just in case

  if (strcmp(mchr, "JAN") == 0) tmfld[0] = 'A';
  if (strcmp(mchr, "FEB") == 0) tmfld[0] = 'B';
  if (strcmp(mchr, "MAR") == 0) tmfld[0] = 'C';
  if (strcmp(mchr, "APR") == 0) tmfld[0] = 'D';
  if (strcmp(mchr, "MAY") == 0) tmfld[0] = 'E';
  if (strcmp(mchr, "JUN") == 0) tmfld[0] = 'F';
  if (strcmp(mchr, "JUL") == 0) tmfld[0] = 'G';
  if (strcmp(mchr, "AUG") == 0) tmfld[0] = 'H';
  if (strcmp(mchr, "SEP") == 0) tmfld[0] = 'I';
  if (strcmp(mchr, "OCT") == 0) tmfld[0] = 'J';
  if (strcmp(mchr, "NOV") == 0) tmfld[0] = 'K';
  if (strcmp(mchr, "DEC") == 0) tmfld[0] = 'M';

  tmfld[17] = '\0';
 }

void SortAos(WinpData* indata)
 {
  // sort the output file for the All Sats AOS according to time
  int ntf, leof, lncnt, scf;
  char lin1[160];
  char tmfld[18], tmmin[18], tmfd2[18];
  FILE* tmpfile;

  outptr = fopen (indata->svfname,"r");
  if (outptr == NULL) return;
  tmpfile = fopen ("satcal.tmp","w+");
  if (tmpfile == NULL) return;

  // check AOS output file for content and get lowest line
  ntf = 0;
  strcpy(tmmin, "XXXXXXXXXXXXXXXXX");

  if (feof(outptr) == 0) leof = 0;
  else leof = 1;

  while (!leof)
   {
    if (fgets(lin1, 160, outptr) == NULL) leof = 1;  // get one line
    if (!leof && strlen(lin1) > 2)
     {
      if ((lin1[0] == 'G') || (lin1[0] == 'm') || (lin1[0] == 'Y')
                           || (lin1[0] == '=') || (lin1[0] == ' '))
       {
        fprintf(tmpfile, lin1);
       }
      else
       {
        if (strlen(lin1) > 16)
         {
          ntf++;
          CpMMM (tmfld, lin1);
          if (strcmp(tmfld, tmmin) < 0) strcpy (tmmin, tmfld);
         };
       };
     };
   };  // leof

  if (ntf > 1)
   {
    lncnt = 5;
    while ((strcmp(tmmin, "XXXXXXXXXXXXXXXXX") < 0) && (ntf > 0))
     {
      strcpy(tmfd2, "XXXXXXXXXXXXXXXXX");
      rewind(outptr);
      leof = 0;
      while (!leof)
       {
        if (fgets(lin1, 160, outptr) == NULL) leof = 1;  // get one line
        if (!leof && strlen(lin1) > 16)
         {
          if ((lin1[0] != 'G') && (lin1[0] != 'm') && (lin1[0] != 'Y')
                               && (lin1[0] != '=') && (lin1[0] != ' '))
           {
            CpMMM (tmfld, lin1);
            scf = strcmp(tmfld, tmmin);
            if (scf == 0)
             {
              if (lncnt > 4)
               {
                fprintf(tmpfile, "\n");
                lncnt = 0;
               };
              fprintf(tmpfile, lin1);
              lncnt++;
             }
            else
             {
              if (scf > 0)
               {
                if (strcmp(tmfld, tmfd2) < 0) strcpy(tmfd2, tmfld);
               };
             };
           };
         };
       };  // leof
      strcpy(tmmin, tmfd2);
      ntf--;
     };  // strcmp(tmmin, tmfd2)

    // copy the sorted file back
    fclose (outptr);
    outptr = fopen (indata->svfname,"w");
    if (outptr != NULL)
     {
      rewind(tmpfile);
      leof = 0;
      while (!leof)
       {
        if (fgets(lin1, 160, tmpfile) == NULL) leof = 1;  // get one line
        else fprintf(outptr, lin1);
       };
     };
   };  // ntf > 1

  fclose (outptr);
  outptr = NULL;
  fclose (tmpfile);

  remove ("satcal.tmp");
 }

int AllSatAos(WinpData* indata)
{
 /* Calculate the AOS times and times of maximum elevation for the
    various passes of all the satellites of a TLE file and write
    them to the standard output file.

    RETURN: 0 if successful, 1 if output file error, 2 if TLE file error
                            -1 if iridflare calc not meaningful
  */

  int sx, sg, j, satok, found, npass;
  double t, ts;
  double ti[3], el[3], az[3];
  char otxt[501];
  char mtxt[61];
  int svrelsat, sidflg, cklc;
  double msunsav, irkmsav;
  FILE* satfle;

  /* open the output and TLE file */
  if (indata->svfirst) outptr = fopen (indata->svfname,"w");
  else outptr = fopen (indata->svfname,"a");
  if (outptr == NULL) return 1;
  indata->svfirst = 0;

  satfle = fopen (indata->tlename,"r");

  if (satfle == NULL)
   {
    fclose(outptr);
    return 2;
   };

  cklc = 0;
  irkmsav = indata->iridkm;
  if (indata->iridflg > 0)
   {
    if ((indata->iridflg==1) && ((indata->occflg==1) || (indata->occflg==2)))
     {
      if (irkmsav <= 0)
       {
        indata->iridkm = 35.0;
        cklc = 1;
       };
     };
    if ((indata->iridflg < 3) && (indata->iridkm == 0)) return -1;
    if ((indata->iridflg == 3) && (indata->iridmag > 29.0)) return -1;
   };

  if (indata->sidxcn < 1)
   {
    indata->sidxcn = 1;
    sidflg = 1;
   }
  else sidflg = 0;

  // save current state
  sx = indata->satidx;
  sg = GBL_MXSAT;
  ShiftElm (indata, GBL_MXSAT+1, sx);
  svrelsat = indata->relsat;
  msunsav = indata->minsunel;
  if ((indata->iridflg == 1) && (indata->occflg == 1)) indata->minsunel = 90.0;
  indata->relsat = 0;

  // output header
  strcpy (otxt,"Geographic Location: ");
  strcat (otxt,indata->geoname);
  fprintf (outptr, otxt);
  fprintf(outptr,"\n");
  sprintf(otxt," Lat.: %g   Long.: %g   Height: %g \n",
                         indata->geolat, indata->geolong, indata->geoheight);
  decdot(otxt);
  fprintf (outptr, otxt);
  fprintf(outptr,"\n");
  sprintf(otxt," min.Sat.Elev.: %g   max.Sun Elev.: %g",
                         indata->minsatel, indata->minsunel);
  if (indata->visflg)
   {
    if (indata->moonflg) strcat(otxt,"   Moonshine passes only");
    else strcat(otxt,"   visible passes only");
   };
  decdot(otxt);
  fprintf (outptr, otxt);
  fprintf(outptr,"\n");
  fprintf(outptr," Year: %4i  +++ Timezone: %g +++ \n", indata->year, indata->tzone);
  fprintf(outptr,
         "=======================================================================");
  if (indata->iridflg) fprintf(outptr, "\n    Event Time (Azim.,Elev.)\n");
  else fprintf(outptr,
  "\n  Start (Azim.,Elev.)    Maximum (Azim.,MaxElev.)     End (Azim.,Elev.)\n");

  // now search
  strcpy(indata->orbelm.sname[GBL_MXSAT], "*");
  found = tleread(indata, satfle, otxt, 0);
  while (found > 0)
   {
    npass = 1;
    ShiftElm (indata, sx, sg);
    indata->iridsock = false;

    do    // double loop if Iridium solar panel flare
     {
      ts = double(indata->hour) - indata->tzone
                      + double(indata->minute)/60.0 + double(indata->second)/3600.0;
      t = mjd(indata->day, indata->month, indata->year, ts);
      ts = t + indata->duration;

      if (indata->iridflg == 3) // Iridium Satellites in operational orbits only
       {
        if (indata->iridops)
         {
          if(indata->orbelm.aflg[sx])
           {
            if ((indata->orbelm.a[sx] < 7157.0) || (indata->orbelm.a[sx] > 7157.6))
                                                                       t = ts + 1.0;
           }
          else
           {
            if ((indata->orbelm.a[sx] < 7150.0) || (indata->orbelm.a[sx] > 7159.9))
                                                                       t = ts + 1.0;
           };
         };
       };

      do    // check the various passes within the time interval for one sat
       {
        satok = nextpass(indata, sx, t, ts, ti, el, az);
        if (satok)
         {
          if (indata->iridflg)
           {
            if (satok < 2)
             {
              aosstr((ti[1]+indata->tzone/24.0), az[1], el[1], otxt, 0);
              if (indata->iridflg < 3)
               {
                if (indata->iridflg == 1)
                 {
                  if (indata->occflg == 1) strcat (otxt, "  Sun Transit");
                  if (indata->occflg == 2) strcat (otxt, "  Moon Transit");
                  if (indata->occflg == 3)
                   {
                    sprintf(mtxt,"  Occ. R.A.: %g  Dec.: %g",indata->occra,indata->occdec);
                    decdot(mtxt);
                    strcat (otxt, mtxt);
                   };
                  if ((indata->occflg == 1) || (indata->occflg == 2))
                   {
                    SunMoonTr (indata, ti[1], sx, az[2], cklc, satok, mtxt);
                   };
                 }
                else
                 {
                  if (indata->flarmod == 1) sprintf(mtxt,"  Flare  Y: %g  P: %g",indata->occra, indata->occdec);
                  else sprintf(mtxt,"  Flare  R.A.: %g  Dec.: %g",indata->occra, indata->occdec);
                  decdot(mtxt);
                  strcat (otxt, mtxt);
                 };
                sprintf(mtxt,"  dist:%7.1lf km", az[2]);
                decdot(mtxt);
                strcat(otxt, mtxt);
               }
              else
               {
                strcat(otxt, "  Iridium Flare  ");
                if (indata->iridsock) strcat(otxt, "Solar  ");
                else
                 {
                  if (indata->occra == 0) strcat(otxt, "Front  ");
                  else
                   {
                    if (indata->occra < 0) strcat(otxt, "Left   ");
                    else strcat(otxt, "Right  ");
                   };
                 };
                if (el[0] > el[2]) strcat(otxt, "->S");
                else strcat(otxt, "->N");
                sprintf(mtxt,"  mag %5.1lf", az[2]);
                decdot(mtxt);
                strcat(otxt, mtxt);
               };
             };  // satok < 2
           }
          else
           {
            strcpy (otxt, "");
            for (j=0; j<3; j++)
             {
              aosstr((ti[j]+indata->tzone/24.0), az[j], el[j], mtxt, j);
              strcat(otxt, mtxt);
             };
           }; // if iridflg
          if (satok == 1)
           {
            if (npass) fprintf(outptr,"\n");
            npass = 0;
            if (indata->iridflg == 3)
             {
              if(indata->orbelm.aflg[sx])
               {
                if((indata->orbelm.a[sx]<7157.0) || (indata->orbelm.a[sx]>7157.6))
                                                     strcat (otxt, " ? ");
                else strcat (otxt, "   ");
               }
              else
               {
                if((indata->orbelm.a[sx]<7150.0) || (indata->orbelm.a[sx]>7159.9))
                                                     strcat (otxt, " ? ");
                else strcat (otxt, "   ");
               };
             }
            else strcat (otxt, "   ");
            strcat(otxt, indata->orbelm.fsname[sx]);
            fprintf (outptr, otxt);
            fprintf(outptr,"\n");
            if ((indata->iridflg == 1) && (irkmsav == 0) &&
                                       ((indata->occflg == 1) || (indata->occflg == 2)))
             {               // try to  add satellite
              if (sidflg)
               {
                sidflg = 0;
                ShiftElm(indata, GBL_MXSAT+1, sx);
               }
              else
               {
                if (indata->sidxcn < sg)
                 {
                  ShiftElm (indata, indata->sidxcn, sx);
                  indata->sidxcn++;
                 };
               };
             };
           };  // satok == 1
          t = ti[2] + 1.0/1440.0;   // add one minute
          if (indata->orbelm.n0[sx] > 14.0) t += 1.0/24.0;  // add one hour
         };  // satok

        if (t > ts) satok = 0;
       } while (satok);  // do various passes

      if ((indata->iridflg == 3) && indata->iridsol) indata->iridsock = !indata->iridsock;
     } while (indata->iridsock);  // do double loop for Iridium solar panel flare

    if (found > 1) found = tleread(indata, satfle, otxt, 1);
    else found = 0;
   };  // while found

  if (outptr != NULL)
   {
    fclose(outptr);
    outptr = NULL;
    SortAos(indata);
   };
  if (satfle != NULL)
   {
    fclose(satfle);
   };

  // restore state
  indata->relsat = svrelsat;
  if (sidflg) indata->sidxcn = 0;
  ShiftElm (indata, sx, GBL_MXSAT+1);
  indata->minsunel = msunsav;
  indata->iridkm = irkmsav;

  return 0;
 }

/*------------------------- Iriddspy ---------------------------------*/

void Iriddspy (WinpData* indata, WoutData* outdata)
 {
  /* Display the sorted Iridium Flare data on screen.
     Add the respective satellites to the selected sat list
     as long as there is still space.
  */

  int leof, lncnt, scf, j, sg;
  char lin1[160];
  char tmfld[17], tmmin[17];
  char otxt[8];
  char ftxt[501];

  FILE* satfle;

  // check whether there is still space in the sat list
  sg = indata->sidxcn;

  satfle = fopen (indata->tlename,"r");
  if (satfle == NULL) sg = GBL_MXSAT;;
  outdata->curlines = 0;
  outptr = fopen (indata->svfname,"r");

  if (outptr == NULL) return;

  strcpy(tmmin, "XXXXXXXXXXXXXXXX");
  if (feof(outptr) == 0) leof = 0;
  else leof = 1;
  strcpy(lin1, " *** Iridium Flares *** ");
  if (indata->iridmess)
   {
    strcat (lin1, " written to ");
    strcat(lin1,indata->svfname);
   };
  indata->svfirst = 0;
  PutSpace(outdata);
  PutNxtLn (outdata, lin1);
  PutSpace(outdata);
  sprintf(lin1," Year: %4i  +++ Timezone: %g +++", indata->year, indata->tzone);
  if (indata->moonflg) strcat(lin1,"   Moonshine Flares");
  PutNxtLn (outdata, lin1);

  lncnt = 5;
  while (!leof)
   {
    if (fgets(lin1, 160, outptr) == NULL) leof = 1;  // get one line
    if (!leof && strlen(lin1) > 16)
     {
      if ((lin1[0] != 'G') && (lin1[0] != 'm') && (lin1[0] != 'Y')
                           && (lin1[0] != '=') && (lin1[0] != ' '))
       {
        strncpy(tmfld, lin1, 16);
        tmfld[16] = '\0';
        scf = strcmp(tmfld, tmmin);
        if (scf != 0)
         {
          for (j=0; j<7; j++) otxt[j] = lin1[j+28];
          otxt[7] = '\0';
          if (strcmp(otxt, "Iridium") == 0)
           {
            strcpy (tmmin, tmfld);
            if (lncnt > 4)
             {
              lncnt = 0;
              PutSpace(outdata);
             };
            for (j=28; j<76; j++)
             {
              lin1[j] = lin1[j+13];
              if (lin1[j] == '\n') lin1[j] = '\0';
              if (lin1[j] == '\r') lin1[j] = '\r';
             };
            lin1[GBL_MAXCPL-2] = '\0';
            PutNxtLnT (outdata, lin1);
            lncnt++;

	    // add sat to list if possible
            if (sg < GBL_MXSAT)
             {
              rewind(satfle);
              for (j=0; j<11; j++)indata->orbelm.sname[GBL_MXSAT][j]=lin1[j+54];
              indata->orbelm.sname[GBL_MXSAT][11] = '\0';
              if (tleread(indata, satfle, ftxt, 0))
               {
                ShiftElm (indata, sg, GBL_MXSAT);
                sg++;
                indata->sidxcn++;
               };
             };  // add sat
           };
         }; // scr != 0
       };  // != 'G' etc.
     }; //  strlen > 16
   };  // while !leof

  if (satfle != NULL) fclose(satfle);
  fclose (outptr);
  outptr = NULL;
 }

/*------------------------- Nxt4Pass ---------------------------------*/

int Nxt4Pass(WinpData* indata)
{
 /* Find the next 10 satellites (if available) on the currently
    selected TLE file to pass the station within the next 0.1 day
    (and complying with the visibility conditions). Store the
    respective orbit elements in indata->orbelm.

    RETURN: -1 if TLE file error, otherwise number of satellites found
  */
  int sx, sg, j, satok, found;
  double t, t1, ts, cfmax;
  double ti[3], el[3], az[3];
  char otxt[501];
  int svidxn, resflg;
  double tf[GBL_MXSAT];
  int cxf, cxmax;

  FILE* satfle;

  /* open the current TLE file */
  satfle = fopen (indata->tlename,"r");

  if (satfle == NULL) return -1;

  svidxn = indata->sidxcn;
  indata->relsat = 0;
  indata->sidxcn = 1;
  sg = GBL_MXSAT;

  // now search
  strcpy(indata->orbelm.sname[GBL_MXSAT], "*");
  found = tleread(indata, satfle, otxt, 0);
  t1 = RT_MJD(indata);
  cxf = 0;  // no satellite found yet
  sx = 0;

  while (found > 0)
   {
    // save old orbit elements
    ShiftElm(indata, GBL_MXSAT+1, sx);
    resflg = 1;
    ShiftElm (indata, sx, sg);
    t = t1;
    ts = t + 0.1;
    satok = nextpass(indata, sx, t, ts, ti, el, az);
    if (ti[0] <= (t1+0.0018)) satok = 0;  // pass starts in 2 minutes

    if (satok)
     {
      if (cxf < GBL_MXSAT)
       {
        tf[cxf] = ti[0];  // beginning of pass
        cxf++;
        sx++;
        if (sx >= GBL_MXSAT) sx = GBL_MXSAT-1;
        resflg = 0;
       }
      else
       {
        // find whether the new time is earlier than previous times
        cxmax = 0;
        cfmax = tf[0];
        for (j=1; j<cxf; j++)
         {
          if (tf[j] > cfmax)
           {
            cfmax = tf[j];
            cxmax = j;
           };
         };

        if (ti[0] < cfmax)
         {
          // override former elements of the latest entry
          tf[cxmax] = ti[0];
          if (cxmax == sx) resflg = 0;  // already stored at right place
          else ShiftElm (indata, cxmax, sg);
         };
       };
     };

    // restore state
    if (resflg) ShiftElm (indata, sx, GBL_MXSAT+1);
    if (found > 1) found = tleread(indata, satfle, otxt, 1);
    else found = 0;
   }  // while found > 0

  if (satfle != NULL) fclose(satfle);

  if (cxf > 0)
   {
    indata->sidxcn = cxf;
    if (cxf > 1)   // sort list of satellites according to time
     {
      for (j=0; j<cxf-1; j++)
       {
        cxmax = j;
        cfmax = tf[j];
        for (sx=j+1; sx<cxf; sx++)
         {
          if (tf[sx] < cfmax)
           {
            cxmax = sx;
            cfmax = tf[sx];
           };
         };
        if (cxmax != j)  // interchange satellites
         {
          cfmax = tf[cxmax];
          tf[cxmax] = tf[j];
          tf[j] = cfmax;
          ShiftElm(indata, sg, cxmax);
          ShiftElm(indata, cxmax, j);
          ShiftElm(indata, j, sg);
         };
       };
     };
   }
  else indata->sidxcn = svidxn;
  indata->satidx = 0;

  return cxf;
}

/*------------------------- GetPassDTxt ---------------------------------*/
void prpang (char* ntx, int ra4flg, int dmsflg, double lat)
 {
  /* prepare the latitude/longitude angle for output in ntx
     ra4flg = 1 if 3 or 4 digits, otherwise just 2 decimal digits
     dmsflg = 1 if output in deg / min / sec, otherwise decimal degrees
     if dmsflg = -1 output in deg/min/sec but already converted
     if dmsflg = -2 output in hours/min/sec but already converted
     lat = angle to be prepared
  */
  double lt2;
  int j;

  if (dmsflg != 0)
   {
    if (dmsflg > 0) lt2 = DegFDms(lat);
    else lt2 = lat;
    if (ra4flg) sprintf(ntx, "%9.4lf", lt2);
    else
     {
      if (lt2 < 0) lt2 -= 0.002;
      else lt2 += 0.002;
      sprintf(ntx, "%7.2lf", lt2);
     };

    j = 0;
    do
     {
      if (ntx[j] == '.')
       {
        if (dmsflg > -2) ntx[j] = cdegs;
        else ntx[j] = 'h';
        j = 7;
       };
      j++;
     } while (j < 7);
   }
  else
   {
    if (ra4flg) sprintf(ntx, "%9.3lf", lat);
    else sprintf(ntx, "%7.2lf", lat);
   };
  decdot (ntx);
 }

int GetPassDTxt(WinpData* indata, double& t, char* jtxt, int flsav)
 {
  /* Get text for the pass details output
     t = last time considered by GetPassDTxt (0 if first call in a pass)
     jtxt : corresponding output text
     flsav : 1 if text, 2 if groundtrack to be saved on file, otherwise 0

     RETURN: 1 if more output pending, 0 otherwise
   */
  int sx, dd, mm, yy, deg, mnt, h1, m1;
  int rlsflg, sx2;
  static double epmax;  // max elevation for pass
  double hh, sec, msunsav, spsel;
  char ltxt[81];

  sx = indata->satidx;

  msunsav = indata->minsunel;
  if (indata->relsat && (indata->sidxcn > 1) && (indata->occflg == 0)
                                             && (indata->flarflg == 0))
   {
    rlsflg = 1;
    sx2 = sx + 1;
    if (sx2 >= indata->sidxcn) sx2 = 0;
   }
  else rlsflg = 0;
  if (indata->occflg == 1) indata->minsunel = 90.0;
  if (t == 0)  // first call, write headline and get start time
   {
    t = indata->tpass[indata->npsel+indata->npoffs];  // start of pass
    epmax = -100.0;
    indata->tpole = 0;
    if (indata->tstep == (1.0/1440.0))   // if timestep exactly one min
     {
      caldat(t, dd, mm, yy, hh);
      dms (hh,deg,mnt,sec);
      if (sec > 0) mnt ++;
      hh = ddd (deg, mnt, 0);
      t = mjd (dd, mm, yy, hh);  // set t to the next min
     }
    else
     {
      if (indata->tstep >= (1.0/86400.0))  // set t to full sec
       {
        caldat(t, dd, mm, yy, hh);
        dms (hh,deg,mnt,sec);
        m1 = int(sec);
        m1++;
        sec = m1;
        hh = ddd (deg, mnt, sec);
        t = mjd (dd, mm, yy, hh);
       };
     };
    caldat((t+indata->tzone/24.0), dd, mm, yy, hh);

    if (flsav)
     {
      strcpy (jtxt, "\n\n\n");
      strcat(jtxt,indata->orbelm.fsname[sx]);
     }
    else strcpy(jtxt,indata->orbelm.fsname[sx]);
    if (rlsflg)
     {
      strcat (jtxt, "\n");
      strcat(jtxt,indata->orbelm.fsname[sx2]);
     };
    strcat (jtxt,"\n______________________________________________________\n");
    sprintf(ltxt,"Year: %4i  +++> Timezone %g <+++" , yy, indata->tzone);
    decdot(ltxt);
    strcat (jtxt, ltxt);
    if (indata->visflg)
     {
      if (indata->moonflg) strcat (jtxt, "  Moonshine");
      else strcat(jtxt, "  visible");
     };
    strcat(jtxt, "\n");
    if (flsav < 2)
     {
      strcat(jtxt, "\n                             ");
      if ((indata->occflg > 0) || ((indata->flarflg > 0) && (indata->flarmod > 0)))
                                       strcat (jtxt, " Ctr.Line  long.    lat.\n");
      else
       {
        if (rlsflg) strcat (jtxt, "       Azim.  Elev.              lat.   long\n");
        else strcat (jtxt, "         Azim.  Elev.                 R.A.   Decl\n");
       };
     };
   }
  else strcpy (jtxt,"\n");

  GetSatPos (t, indata, satpos, sx, 0);

  if (cksatok (indata, satpos, sx))
   {
    aosstr((t+indata->tzone/24.0), 0, 0, ltxt, 0);
    if (indata->tstep == (1.0/1440.0))   // if timestep exactly one min
     {
      ltxt[13] = '\0';
     }
    else
     {
      if (indata->tstep >= 1.157407407e-05) ltxt[16] = '\0'; // if >= 1 sec
      else
       {
        ltxt[7] = '\0';
        strcat (jtxt, ltxt);
        caldat((t+indata->tzone/24.0), dd, m1, yy, hh);
        dms (hh, h1, m1, sec);
        if (sec >= 59.95)
         {
          if (m1 < 59)
           {
            m1++;
            sec = 0;
           }
          else
           {
            if (h1 < 24)
             {
              h1++;
              m1 = 0;
              sec = 0;
             };
           };
         };
        sprintf(ltxt," %2i:%02i:%4.1lf", h1, m1, sec);
        ltxt[11]='\0';
        decdot(ltxt);
       };
     };
    strcat (jtxt, ltxt);
    if (((indata->occflg > 0) || ((indata->flarflg > 0) && (indata->flarmod > 0)))
                              && (flsav < 2))
     {
      if (satpos->occlat[sx] < 100.0)
       {
        if (flsav)
         {
          sprintf(ltxt,"   %10.5lf  %10.5lf ", satpos->occlng[sx], satpos->occlat[sx]);
          decdot(ltxt);
          strcat (jtxt, ltxt);
         }
        else
         {
          strcat(jtxt, "      ");
          prpang (ltxt, indata->ra4dflg, indata->dmsflg, satpos->occlng[sx]);
          strcat (jtxt, ltxt);
          strcat(jtxt, "  ");
          prpang (ltxt, indata->ra4dflg, indata->dmsflg, satpos->occlat[sx]);
          strcat (jtxt, ltxt);
         };
       }
      else strcat(jtxt, "   No Intersection");
     }
    else
     {
      spsel = satpos->elev[sx];
      spsel = ElvRefr(indata, spsel);
      if (flsav)
       {
        if (flsav == 1)sprintf(ltxt, "    %8.3lf  %8.3lf",satpos->azim[sx],spsel);
        else sprintf(ltxt, "    %10.5lf  %10.5lf  %12.3lf",
                        satpos->lng[sx], satpos->lat[sx], satpos->height[sx]);
       }
      else
       {
        if ((rlsflg == 0) && (indata->ra4dflg == 1))
             sprintf(ltxt, "    %8.3lf  %8.3lf", satpos->azim[sx], spsel);
        else sprintf(ltxt, "     %7.2lf  %7.2lf", satpos->azim[sx], spsel);
       };
      decdot(ltxt);
      strcat (jtxt, ltxt);
      if (flsav < 2)
       {
        if ((rlsflg == 0) && (indata->ra4dflg == 1))
               sprintf(ltxt, "     %9.4lf  %9.4lf", satpos->RA[sx], satpos->Dec[sx]);
        else sprintf(ltxt, "        %7.2lf  %7.2lf", satpos->RA[sx], satpos->Dec[sx]);
        decdot(ltxt);
        strcat (jtxt, ltxt);
       };
     };
    if (satpos->elev[sx] > epmax)
     {
      epmax = satpos->elev[sx];
      indata->tpole = t;
     };
    mm = 1;
   }
  else mm = 0;

  t += indata->tstep;
  if (t == 0) t = 1.0;  // just in case to avoid an endless loop
  indata->minsunel = msunsav;

  return mm;
 }

/*------------------------- GetPoleTxt ---------------------------------*/

void GetPoleTxt(WinpData* indata, char* jtxt)
 {
  /* Get text for the pole of the pass (end of pass details output)
     jtxt : corresponding output text
   */
  int sx;
  double t, dt, az, el, tut1;
  char ltxt[41];
  Vec3 r1, r2;

  t = indata->tpole;
  tut1 = t;
  if (indata->IERSflg) tut1 += indata->Ut1Utc/86400.0;
  sx = indata->satidx;

  strcpy (jtxt, "\n\n");
  if (indata->relsat && (indata->sidxcn > 1) && (indata->occflg == 0)
                                    && (indata->flarflg == 0)) return;

  // calculate the pole of the pass +/- 15 sec from maximum
  dt = 15.0 / 86400;  // 15 sec
  GetSatPos ((t-dt), indata, satpos, sx, 0);
  r1[0] = 1.0;
  r1[1] = 15.0*DmsDegF(satpos->RA[sx])*degrad;
  r1[2] = DmsDegF(satpos->Dec[sx])*degrad;
  r1 = polcar(r1);

  GetSatPos ((t+dt), indata, satpos, sx, 0);

  r2[0] = 1.0;
  r2[1] = 15.0*DmsDegF(satpos->RA[sx])*degrad;
  r2[2] = DmsDegF(satpos->Dec[sx])*degrad;
  r2 = polcar(r2);
  r2 = r1 * r2;
  if (abs(r2) != 0)
   {
    AppPos (tut1, 0, indata->geolat*degrad, indata->geolong*degrad,
                     indata->geoheight, 0, r2, az, el, dt);
    if (el < 0)
     {
      r2[0] = -r2[0];  // opposite direction points upwards
      r2[1] = -r2[1];
      r2[2] = -r2[2];
      AppPos (tut1, 0, indata->geolat*degrad, indata->geolong*degrad,
                       indata->geoheight, 0, r2, az, el, dt);
     };
    r2 = carpol (r2);
    r2[1] = DegFDms (r2[1]/(15.0*degrad));
    r2[2] = DegFDms (r2[2] / degrad);
    az /= degrad;
    if (az < 0) az += 360.0;
    el /= degrad;

    // prepare the output
    strcat(jtxt, "Apparent Pole of orbit (Azim   Elev   R.A.  Decl)\n");
    sprintf(ltxt, "           %7.2lf %7.2lf", az, el);
    decdot(ltxt);
    strcat (jtxt, ltxt);
    if(indata->ra4dflg == 1) sprintf(ltxt, "    %9.4lf  %9.4lf", r2[1], r2[2]);
    else sprintf(ltxt, "    %7.2lf  %7.2lf", r2[1], r2[2]);
    decdot(ltxt);
    strcat (jtxt, ltxt);
   };  // abs(r3)
 }

// ------------------------ Save Orbit Track -------------------------

int saveOrbitTrack(WinpData* indata)
{
  // save track on file

  int sx;
  double t, ts;
  char ltxt[81];
  char jtxt[501];


  // open the save file
  outptr = fopen (indata->svfname,"w");
  if (outptr == NULL) return 1;

  ts = double(indata->hour) - indata->tzone
                  + double(indata->minute)/60.0 + double(indata->second)/3600.0;
  t = mjd(indata->day, indata->month, indata->year, ts);
  ts = t + indata->duration;

  sx = indata->satidx;

  sprintf(jtxt,"\n Year: %4i  +++> Timezone %g <+++" , indata->year, indata->tzone);
  strcat(jtxt, "\n                     ");
  if ((indata->occflg > 0) || ((indata->flarflg > 0) && (indata->flarmod > 0)))
                                    strcat (jtxt, " Ctr.Line  long.    lat.\n");
  else strcat (jtxt,"\n");
  decdot(jtxt);

  fprintf (outptr, jtxt);  // write header
  fprintf(outptr,"\n");

  while (t < ts)
   {
    GetSatPos (t, indata, satpos, sx, 0);

    aosstr((t+indata->tzone/24.0), 0, 0, ltxt, 0);

    ltxt[16]='\0';

    strcpy (jtxt, ltxt);
    if (((indata->occflg > 0) || ((indata->flarflg > 0) && (indata->flarmod > 0))))
     {
      sprintf(ltxt,"   %10.5lf  %10.5lf ", satpos->occlng[sx], satpos->occlat[sx]);
      decdot(ltxt);
      strcat (jtxt, ltxt);
      if (satpos->occlat[sx] >= 100.0) strcat(jtxt, "  No Intersection");
     }
    else
     {
      sprintf(ltxt, "    %10.5lf  %10.5lf  %12.3lf",
                       satpos->lng[sx], satpos->lat[sx], satpos->height[sx]);
      decdot(ltxt);
      strcat (jtxt, ltxt);
     }

    fprintf (outptr, jtxt);
    fprintf(outptr,"\n");

    t += indata->tstep;
   };

   if (outptr != NULL)
    {
     fclose(outptr);
     outptr = NULL;
    };

   return 0;
}

/*------------------------- SkyAzim ---------------------------------*/

void SkyAzim(WinpData* indata, double t)
 {
  /* Get azimuth for automatic Sky Map at time t (which has been found
     by NextPass or LastPass)
     The azimuth will be placed in indata->skazimuth (in deg)
   */
  int sx;

  sx = indata->satidx;
  GetSatPos (t, indata, satpos, sx, 0);
  indata->skazimuth = satpos->azim[sx];
 }

/*------------------------- LocalInf -------------------------------*/

void LocalInf (WinpData* indata, char* otxt)
 {
  // get text about the position of all satellites selected from
  //  the current local observer
  int svsx, svrls, fcall, j;
  double t, a1, e1, ep2, a2, e2, epochsv, azm;
  char ttxt[61];
  Vec3 rr, vv;
  Mat3 m1;

  azm = -1.0;
  if (indata->skymap)
   {
    if (indata->el1 < - 90.0) strcpy(otxt, "\n\n   *** Cursor outside Map ***");
    else
     {
      sprintf(otxt,"\n\nAzimuth (deg.):  %6.1lf,    Elevation (deg):  %6.1lf",
                            indata->az1, indata->el1);
      decdot(otxt);
      sprintf(ttxt,"\n\n R.A.(HH.MM):  %6.2lf,    Decl.(DD.MM):  %6.2lf",
                            indata->az2, indata->el2);
      decdot(ttxt);
      strcat(otxt,ttxt);
      if (indata->sidxcn < 1)
                  strcat(otxt, "\n\n\n *** No satellite has been selected ***");
      else
       {
        svrls = indata->relsat;
        epochsv = indata->yrepoch;
        indata->relsat = 0;
        t = indata->lastmjd;
        j = indata->satidx;
        a1 = indata->az1;
        e1 = indata->el1;
        GetSatPos (t, indata, satpos, j, 0);

        // angular distance
        t = sin(e1*degrad)*sin(satpos->elev[j]*degrad)
                          + cos(e1*degrad)*cos(satpos->elev[j]*degrad)
                          * cos(a1*degrad - satpos->azim[j]*degrad);
        if (fabs(t) <= 1.0)
         {
          t = acos(t) / degrad;
          strcat(otxt,"\n\n");
          strncat(otxt,indata->orbelm.fsname[j], 21);
          sprintf(ttxt, "\n\tAngular distance:%7.2lf", t);
          decdot(ttxt);
          strcat(otxt,ttxt);
         };

        // direction across sky
        t = indata->lastmjd;
        indata->yrepoch = 0;
        GetSatPos (t, indata, satpos, j, 0);
        a1 = satpos->RA[j];
        e1 = satpos->Dec[j];
        a1 = DmsDegF(a1)*15.0*degrad;
        e1 = DmsDegF(e1)*degrad;
        t = indata->lastmjd;
        t = t + 2.0/86400.0; // 2 sec later
        GetSatPos (t, indata, satpos, j, 0);
        a2 = satpos->RA[j];
        e2 = satpos->Dec[j];
        a2 = DmsDegF(a2)*15.0*degrad;
        e2 = DmsDegF(e2)*degrad;

        a1 = a2 - a1;
        t = sin(e1)*sin(e2) + cos(e1)*cos(e2)*cos(a1);
        if(fabs(t) <= 1.0) t = acos(t);
        else t = 0;
        a2 = cos(e1)*sin(t);
        if(fabs(a2) > 1.0e-50)
         {
          t = (sin(e2) - sin(e1)*cos(t))/a2; 
          if (fabs(t) <= 1.0)
           {
            t = acos(t) / degrad;
            if (sin(a1) > 0) t = -t;
            if (t < 0) t += 360.0;
            sprintf(ttxt, "\n\n\tSky Course:%7.2lf", t);
            decdot(ttxt);
            strcat(otxt,ttxt);
           }; 
         };

        // restore current state            
        indata->relsat = svrls;
        indata->yrepoch = epochsv;
        t = indata->lastmjd;
        GetSatPos (t, indata, satpos, j, 0);
       };
     };

    return;
   };

  if ((indata->geolat < - 90.0) || (indata->geolong > 180.0))
   {
    strcpy(otxt, "\n\n   *** Cursor outside Map ***");
    return;
   };

  // save current satellite index and relsat flag
  svsx = indata->satidx;
  svrls = indata->relsat;
  indata->relsat = 0;
  t = indata->lastmjd;

  if (indata->sidxcn < 1) strcpy(otxt, "*** No satellite has been selected ***");
  else
   {
    fcall = 1;
    for (j=0; j<indata->sidxcn; j++)
     {
      indata->satidx = j;
      GetSatPos (t, indata, satpos, j, 0);
      if (fcall)
       {
        sprintf(otxt,"Lat: %g,  Long: %g,   SunElev:%6.1lf",
                               indata->geolat, indata->geolong, satpos->obssun);
        decdot(otxt);
        fcall = 0;
       };
      strcat(otxt,"\n\n");
      strncat(otxt,indata->orbelm.fsname[j], 21);
      sprintf(ttxt, "\n\tAzim:%7.2lf,\tElev:%7.2lf,\tSun:%6.1lf",
                             satpos->azim[j], ElvRefr(indata, satpos->elev[j]),
                             satpos->sunang[j]);
      decdot(ttxt);
      strcat(otxt,ttxt);
     };
   };

  if (indata->sidxcn == 1)  // as a bonus get the along-the-track distance
   {
    if (indata->orbelm.numflg[0] && (indata->lndist > 0.0))
     { // distance to surface of Moon if close enough
      t = indata->lndist*0.001 - 1738.0;
      sprintf(ttxt, "\n\n\tDistance to surface of Moon:%9.1lf km", t);
      decdot(ttxt);
      strcat(otxt,ttxt);
     }
    else // check angular distance
     {
      t = sin(indata->geolat*degrad)*sin(satpos->lat[0]*degrad)
                            + cos(indata->geolat*degrad)*cos(satpos->lat[0]*degrad)
                            * cos(indata->geolong*degrad - satpos->lng[0]*degrad);
      if (fabs(t) <= 1.0)
       {
        azm = acos(t);
        t = azm * 6378.137;
        sprintf(ttxt, "\n\n\tAlong-the-Surface-Distance:%8.1lf km", t);
        decdot(ttxt);
        strcat(otxt,ttxt);
       };
     };

    // state vector output
    strcat(otxt, "\n\n\tState Vector in km and km/s");
    sprintf(ttxt, "\nx\t%16.8le ", (indata->orbelm.x[0]/1000.0));
    decdot(ttxt);
    strcat(otxt,ttxt);
    sprintf(ttxt, "\ny\t%16.8le ", (indata->orbelm.y[0]/1000.0));
    decdot(ttxt);
    strcat(otxt,ttxt);
    sprintf(ttxt, "\nz\t%16.8le ", (indata->orbelm.z[0]/1000.0));
    decdot(ttxt);
    strcat(otxt,ttxt);
    sprintf(ttxt, "\nvx\t%16.8le ", (indata->orbelm.vx[0]/1000.0));
    decdot(ttxt);
    strcat(otxt,ttxt);
    sprintf(ttxt, "\nvy\t%16.8le ", (indata->orbelm.vy[0]/1000.0));
    decdot(ttxt);
    strcat(otxt,ttxt);
    sprintf(ttxt, "\nvz\t%16.8le ", (indata->orbelm.vz[0]/1000.0));
    decdot(ttxt);
    strcat(otxt,ttxt);

    // Beta Angle
    sprintf(ttxt, "\n\n\tBeta Angle: %7.2lf", satpos->beta[0]);
    decdot(ttxt);
    strcat(otxt,ttxt);
    // Sun Azimuth from Satellite
    e1 = satpos->sunazm[0]*180.0/M_PI;
    if (abs(indata->flarmod) == 2)
     {
      if (e1 < 0) e1 += 360.0;
      sprintf(ttxt, "\n\n\tSun Azimuth (from North): %7.2lf", e1);
      decdot(ttxt);
      strcat(otxt,ttxt);
     }
    else
     {
      rr[0] = indata->orbelm.x[0];
      rr[1] = indata->orbelm.y[0];
      rr[2] = indata->orbelm.z[0];
      vv[0] = indata->orbelm.vx[0];
      vv[1] = indata->orbelm.vy[0];
      vv[2] = indata->orbelm.vz[0];

      t = indata->lastmjd;
      t = julcent (t);
      m1 = nutmat(t, ep2, indata->IERSflg);   // nutation
      rr = mxvct(m1, rr);    // true of date
      vv = mxvct(m1, vv);
      if (indata->flaryst) m1 = GetYSMat (indata, rr, vv, 0, 0);
      else m1 = GetRVMat (rr, vv, 0, 0, 1);

      vv[0] = 0;
      vv[1] = 0;
      vv[2] = 1.0;
      vv = mxvct(m1, vv);
      vv = vnorm(vv);
      vv = carpol(vv);
      a1 = 360.0 - vv[1] / degrad;
      a1 = e1 + a1;
      if (a1 < 0) a1 += 360.0;
      if (a1 > 360.0) a1 -= 360.0;
      if (indata->flaryst)
                   sprintf(ttxt, "\n\n\tSun Azimuth (+V YSM): %7.2lf", a1);
      else sprintf(ttxt, "\n\n\tSun Azimuth (from +V): %7.2lf", a1);
      decdot(ttxt);
      strcat(otxt,ttxt);
      a1 = abs(rr)/1000.0;
      if (a1 > 6378.137)
       {
        a1 = asin(6378.137 / a1)*180.0/M_PI;
        a1 = 90.0 - a1;
        a1 = satpos->sunang[0] - a1;
        sprintf(ttxt, "\n\tSun Elevation: %7.2lf", a1);
        decdot(ttxt);
        strcat(otxt,ttxt);
       };

      // position of observer as viewed from satellite

      vv = GeoPos (indata->lastmjd, 0.0, indata->geolat*degrad, indata->geolong*degrad,
                                         indata->geoheight, 0.0, 0.0) * 6378.137;
      vv = vv - rr/1000.0;
      vv = mxvct(m1, vv);
      vv = vnorm(vv);
      vv = carpol(vv);
      a1 = 360.0 - vv[1] / degrad;
      if (a1 < 0) a1 += 360.0;
      if (a1 > 360.0) a1 -= 360.0;
      t = vv[2] / degrad;
      if (azm > 0) // calculate azimuth of observer as viewed from S/C
       {
        a2 = cos(satpos->lat[0]*degrad) * sin(azm);
        if(fabs(a2) > 1.0e-50)
         {
          azm = (sin(indata->geolat*degrad) - sin(satpos->lat[0]*degrad)*cos(azm)) / a2;
          azm = acos(azm) / degrad;
          if (sin((indata->geolong - satpos->lng[0])*degrad) < 0) azm = -azm; 
          if (azm < 0) azm += 360.0;
          if (azm > 360.0) azm -= 360.0;
         }
        else azm = -1.0;
       };
      if (azm > 0) 
        sprintf(ttxt, "\n\n\tObserver (Yaw, Pitch, Azimuth): %7.2lf  %7.2lf  %7.2lf", a1, t, azm);
      else sprintf(ttxt, "\n\n\tObserver (Yaw, Pitch): %7.2lf  %7.2lf", a1, t);
      decdot0(ttxt);
      strcat(otxt,ttxt);
     };
   };

  // restore current satellite index and relsat flag
  indata->satidx = svsx;
  indata->relsat = svrls;
 }

/*------------------------- chcpy ---------------------------------*/

void chcpy (char* s1, char* s2, int pos, int len)
 {
  /* copy characters from string s2 to s1 starting at pos (in s1)
     for a length of len
  */
  int j;

  for (j=0; j<len; j++) s1[pos+j] = s2[j];
 }

/*------------------------- SaveTle ---------------------------------*/

void cvtlef (char* tmpbuf, double f)
 {
  // convert a double f into the TLE special double format +XXXXX-X
  int lix;
  long int lgg;
  double lgi, d, g;

  if (f == 0) strcpy (tmpbuf, " 00000-0");
  else
   {
    f *= 10.0;
    g = f;
    if (f < 0) g = -g;
    lgi = log10(g);
    lix = int(lgi);
    lix--;
    if (lix > 9) lix = 9;
    if (lix < -9) lix = -9;
    d = pow(10.0,-lix);
    g = f * d;
    g *= 1.0E4;
    lgg = long(float(g));
    if (lgg < 0) lgg = -lgg;
    if (lgg > 99999) lgg = 99999;
    if (lix < 0) lix = -lix;
    sprintf (tmpbuf," %5li %1i", lgg, lix);
    if (f < 0) tmpbuf[0] = '-';
    if (lgi < 0) tmpbuf[6] = '-';
   };
 }

char tcksum (char* tmpbuf)
 {
  // get the checksum of the TLE line
  int j, k;
  char res;
  char rstxt[7];

  k = 0;
  for (j=0; j<68; j++)
   {
    if (tmpbuf[j] == '1') k += 1;
    if (tmpbuf[j] == '2') k += 2;
    if (tmpbuf[j] == '3') k += 3;
    if (tmpbuf[j] == '4') k += 4;
    if (tmpbuf[j] == '5') k += 5;
    if (tmpbuf[j] == '6') k += 6;
    if (tmpbuf[j] == '7') k += 7;
    if (tmpbuf[j] == '8') k += 8;
    if (tmpbuf[j] == '9') k += 9;
    if (tmpbuf[j] == '-') k += 1;
   };
  k = k % 10;
  sprintf (rstxt,"%5i", k);

  res = rstxt[4];
  return res;
 }

int SaveTle (WinpData* indata)
 {
  /* Save the orbit elements of the currently selected satellites
     on TLE Save File.

     RETURN: 0 if save was successful
             1 if no satellites selected
             2 file error
   */

  FILE *outtle;  /* TLE Output File */
  int sx, j, res, day, month, year;
  double hour, ecc;
  char tlebuf[70];
  char tmpbuf[70];

  if (indata->sidxcn > 0)   // write only if satellites have been selected
   {
    // open the TLE output file
    if (indata->svtlefst) outtle = fopen (indata->tlesave,"w");
    else outtle = fopen (indata->tlesave,"a");
    indata->svtlefst = 0;

    // write the elements on file
    if (outtle != NULL)
     {
      // now output the TLE's
      for (sx=0; sx<indata->sidxcn; sx++)
       {
        // satellite name
        strcpy(tlebuf, indata->orbelm.fsname[sx]);
        res = strlen(tlebuf);
        for (j=0; j<res; j++) if (tlebuf[j] == '\r') tlebuf[j] = ' ';
        for (j=res; j<40; j++) tlebuf[j] = ' ';
        tlebuf[40] = '\0';
        fputs(tlebuf, outtle);
        fprintf(outtle,"\n");

        // line 1
        sprintf(tlebuf, "1 %5i", indata->orbelm.norad[sx]);
        strcpy(indata->ldsgn, indata->orbelm.ldsgn[sx]);
        convldsn(indata, false);
        for (j=7; j<70; j++) tlebuf[j] = ' ';
        for (j=0; j<8 ;j++) tlebuf[j+9] = indata->idsgn[j];
        caldat (indata->orbelm.mjdep[sx], day, month, year, hour);
        hour = indata->orbelm.mjdep[sx] - mjd(0,1,year,0);
        sprintf (tmpbuf,"%4i", year);
        tlebuf[18] = tmpbuf[2];
        tlebuf[19] = tmpbuf[3];
        sprintf (tmpbuf,"%12.8lf", hour);
        decdot(tmpbuf);
        chcpy(tlebuf, tmpbuf, 20, 12);

        if (indata->orbelm.numflg[sx])  // do all the 4 lines of the numeric case
         {
          // dummy revolution number (I don't care about those!)
          tlebuf[37] = '0';

          strcpy(tmpbuf,"+++++STATEVCT1+++++");
          chcpy(tlebuf, tmpbuf, 39, 19);
          tlebuf[68] = '1';
          tlebuf[69] = '\0';
          fputs(tlebuf, outtle);
          fprintf(outtle,"\n");

          // Numeric Case Line 2
          for (j=0; j<70; j++) tlebuf[j] = ' ';
          sprintf (tmpbuf,"%20.12le %20.12le %20.12le", indata->orbelm.x0[sx],
                                indata->orbelm.y0[sx], indata->orbelm.z0[sx]);
          decdot(tmpbuf);
          chcpy(tlebuf, tmpbuf, 0, 62);
          tlebuf[68] = '2';
          tlebuf[69] = '\0';
          fputs(tlebuf, outtle);
          fprintf(outtle,"\n");

          // Numeric Case Line 3
          for (j=0; j<70; j++) tlebuf[j] = ' ';
          sprintf (tmpbuf,"%20.12le %20.12le %20.12le", indata->orbelm.vx0[sx],
                               indata->orbelm.vy0[sx], indata->orbelm.vz0[sx]);
          decdot(tmpbuf);
          chcpy(tlebuf, tmpbuf, 0, 62);
          tlebuf[68] = '3';
          tlebuf[69] = '\0';
          fputs(tlebuf, outtle);
          fprintf(outtle,"\n");

          // Numeric Case Line 4
          for (j=0; j<70; j++) tlebuf[j] = ' ';
          sprintf (tmpbuf,"%7.1lf %7.1lf %7.1lf %13.5le %13.5le %13.5le",
                indata->orbelm.F107[sx], indata->orbelm.FB107[sx],
                indata->orbelm.Kp[sx], indata->orbelm.Cd[sx],
                indata->orbelm.Cr[sx], indata->orbelm.Am[sx] );
          decdot(tmpbuf);
          chcpy(tlebuf, tmpbuf, 0, 65);
          tlebuf[68] = '4';
          tlebuf[69] = '\0';
          fputs(tlebuf, outtle);
          fprintf(outtle,"\n");
         }

        else // continue with ordinary TLE's
         {
          if (indata->orbelm.ndot[sx] >= 0)
                   sprintf (tmpbuf,"%10.8lf",indata->orbelm.ndot[sx]);
          else
           {
            sprintf (tmpbuf,"%11.8lf",indata->orbelm.ndot[sx]);
            for (res=0; res<10; res++) tmpbuf[res] = tmpbuf[res+1];
            tmpbuf[0] = '-';
           };
          decdot(tmpbuf);
          chcpy(tlebuf, tmpbuf, 33, 10);
          cvtlef (tmpbuf, indata->orbelm.n6dot[sx]);
          chcpy(tlebuf, tmpbuf, 44, 8);
          cvtlef (tmpbuf, indata->orbelm.bstar[sx]);
          chcpy(tlebuf, tmpbuf, 53, 8);
          if (indata->orbelm.aflg[sx]) tlebuf[62] = '0';
          else tlebuf[62] = '9';
          tlebuf[68] = tcksum(tlebuf);
          tlebuf[69] = '\0';
          fputs(tlebuf, outtle);
          fprintf(outtle,"\n");

          // line 2
          tlebuf[0] = '2';
          for (j=7; j<70; j++) tlebuf[j] = ' ';
          sprintf (tmpbuf,"%8.4lf",indata->orbelm.inc[sx]);
          decdot(tmpbuf);
          chcpy(tlebuf, tmpbuf, 8, 8);
          sprintf (tmpbuf,"%8.4lf",indata->orbelm.ra[sx]);
          decdot(tmpbuf);
          chcpy(tlebuf, tmpbuf, 17, 8);
          ecc = indata->orbelm.ecc[sx];
          if (ecc < 1.0)
           {
            sprintf (tmpbuf,"%9.7lf",indata->orbelm.ecc[sx]);
            decdot(tmpbuf);
            for (res=0; res<7; res++) tmpbuf[res] = tmpbuf[res+2];
            chcpy(tlebuf, tmpbuf, 26, 7);
           }
          else
           {
            sprintf (tmpbuf,"%8.5lf",indata->orbelm.ecc[sx]);
            decdot(tmpbuf);
            chcpy(tlebuf, tmpbuf, 25, 8);
           };
          sprintf (tmpbuf,"%8.4lf",indata->orbelm.aper[sx]);
          decdot(tmpbuf);
          chcpy(tlebuf, tmpbuf, 34, 8);
          sprintf (tmpbuf,"%8.4lf",indata->orbelm.m0[sx]);
          decdot(tmpbuf);
          chcpy(tlebuf, tmpbuf, 43, 8);
          if (ecc < 1.0) sprintf (tmpbuf,"%11.8lf",indata->orbelm.n0[sx]);
          else
           {
            if (indata->orbelm.n0[sx] < 1.0e6)
                 sprintf (tmpbuf,"%11.4lf",indata->orbelm.n0[sx]);
            else sprintf (tmpbuf,"%11.2lf",indata->orbelm.n0[sx]);
           };
          decdot(tmpbuf);
          chcpy(tlebuf, tmpbuf, 52, 11);
          tlebuf[68] = tcksum(tlebuf);
          tlebuf[69] = '\0';
          fputs(tlebuf, outtle);
          fprintf(outtle,"\n");
         };  // numflg
       };

      // close the file
      fclose(outtle);
      res = 0;
     }
    else res = 2;
   }
  else res = 1;

  return res;
 }

/*------------------------- ExDeltaV ---------------------------------*/

int ExDeltaV (WinpData* indata, double delv, double yaw, double pitch)
 {
  /* Execute a delta-v maneuver with the currently selected satellite

     delv : delta-v in m/s
     yaw, pitch : yaw and pitch angle of direction of thrust in degrees

     RETURN: 0 if delta-v resulted in an elliptic orbit
             1 if the resulting orbit is non-elliptic
   */
  const double gm = 2.9755364003E15;  // Gravitation Constant G*Mearth

  int sx, res;
  double t1, t2;
  Vec3 r1, v1, u2, u3;
  Mat3 mx, m2;

  res = 0;
  sx = indata->satidx;

  t1 = indata->lastmjd;
  GetSatPos (t1, indata, satpos, sx, 0);

  r1[0] = indata->orbelm.x[sx];
  r1[1] = indata->orbelm.y[sx];
  r1[2] = indata->orbelm.z[sx];
  v1[0] = indata->orbelm.vx[sx];
  v1[1] = indata->orbelm.vy[sx];
  v1[2] = indata->orbelm.vz[sx];

  if((yaw != 0) || (pitch != 0))
   {
    /* Get matrix mx to transform from the original reference (equatorial)
       into a coordinate system which is defined by the r-v orbit plane
       with x pointing into the direction which results when, starting
       from +v one first yaws then pitches by the respective yaw-pitch
       angles (assuming being in an airplane with the bottom pointing
       toward the center of the Earth).
     */

    m2 = GetRVMat (r1, v1, yaw, pitch, 1);
    mx = mxtrn(m2);  // transforms back into original reference

    // delta-v vector
    u3[0] = delv;
    u3[1] = 0;
    u3[2] = 0;
    u2 = mxvct(mx, u3);
   }

  else   // delta-v along present velocity vector
   {
    u3 = vnorm(v1);
    u2 = delv * u3;
   };

  v1 += u2;
  indata->orbelm.vx[sx] = v1[0];
  indata->orbelm.vy[sx] = v1[1];
  indata->orbelm.vz[sx] = v1[2];
  r1 /= 1000.0;  // convert into km for later
  v1 /= 1000.0;

  indata->orbelm.mjdcur[sx] = t1;
  indata->orbelm.x0[sx] = indata->orbelm.x[sx];
  indata->orbelm.y0[sx] = indata->orbelm.y[sx];
  indata->orbelm.z0[sx] = indata->orbelm.z[sx];
  indata->orbelm.vx0[sx] = indata->orbelm.vx[sx];
  indata->orbelm.vy0[sx] = indata->orbelm.vy[sx];
  indata->orbelm.vz0[sx] = indata->orbelm.vz[sx];

  // get updated orbit elements from state vector
  v1 *= 86400.0;  // velocity in km/d
  indata->orbelm.mjdep[sx] = t1;
  oscelm (gm, t1, r1, v1, t2, indata->orbelm.m0[sx], indata->orbelm.a[sx],
                              indata->orbelm.ecc[sx], indata->orbelm.ra[sx],
                              indata->orbelm.aper[sx], indata->orbelm.inc[sx]);

  if (indata->orbelm.ecc[sx] < 1.0)
   {
    if(indata->orbelm.aflg[sx])
     {
      r1 *= 1000.0;
      v1 *= 1000.0;
      v1 /= 86400.0;
      rv2el(r1, v1, indata->orbelm.inc[sx], indata->orbelm.ra[sx], indata->orbelm.ecc[sx],
                indata->orbelm.aper[sx], indata->orbelm.m0[sx], indata->orbelm.n0[sx]);
      indata->orbelm.a[sx] = getsax (1, indata->orbelm.n0[sx], indata->orbelm.ecc[sx],
                                        indata->orbelm.inc[sx]);
     }
    else indata->orbelm.n0[sx] = getn0 (indata->orbelm.a[sx],
                                        indata->orbelm.ecc[sx], indata->orbelm.inc[sx]);
   }
  else
   {
    indata->orbelm.aflg[sx] = 0;   // don't use SPG4 after maneuver.
    if (indata->orbelm.numflg[sx] == 0) indata->orbelm.mjdep[sx] = t2;
    indata->orbelm.n0[sx] = indata->orbelm.a[sx];
    if (indata->orbelm.ecc[sx] != 1.0)
            indata->orbelm.n0[sx] *= (1.0 - indata->orbelm.ecc[sx]);
    res = 1;
   };

  indata->lastsx = 9999;  // reset sx marker for SGP4 initialization

  return res;
 }

/*------------------------- LaunchOrb ---------------------------------*/

void LaunchOrb (WinpData* indata)
 {
  /* calculate an approximate launch orbit  */
  const double re = 6378.137;  // Earth equatorial radius
  const double j2 = 1.082628E-3;   // J2-Term
  const double mp2 = 2.0*M_PI;
  double c, dlam, azim;
  double razim, ecc, a, n0, aper, ran, dt;
  int sx, sx2, nmfsav;
  bool nw;
  Vec3 r1, v1;

  sx = indata->satidx;

  indata->orbelm.mjdep[sx] = indata->lastmjd;
  nw = false;
  nmfsav = indata->orbelm.numflg[sx];
  indata->orbelm.numflg[sx] = 0;

  dlam = indata->lnchlat*degrad;
  if (fabs(dlam) < 1.396) dlam = atan (tan(dlam) / 1.00674); // geocentric!
  c = sin(indata->orbelm.inc[sx]*degrad);
  if (c == 0) c = 1e-5;
  c = sin(dlam) / c;
  if (c > 1.0) c = 1.0;   // just in case
  if (c < -1.0) c = -1.0;
  c = asin(c);
  if (indata->lsouthflg) c = M_PI - c;

  azim = cos(dlam);
  if (azim == 0) azim = 1e-5;
  dlam = cos(c) / azim;
  if (dlam > 1.0) dlam = 1.0;
  if (dlam < -1.0) dlam = -1.0;
  azim = cos(indata->orbelm.inc[sx]*degrad) / azim;
  indata->lnchazm = atan20(azim,sin(indata->orbelm.inc[sx]*degrad)*dlam);
  indata->lnchazm /= degrad;
  dlam = fabs(acos(dlam));
  if ((azim*indata->lnchlat) < 0) dlam = -dlam;
  azim = indata->lnchlng - dlam/degrad;
  razim = azim;

  dlam = indata->tmeco;
  if (dlam > 12.0) dlam = 12.0;
  dlam = 0.45*dlam/1440.0;
  indata->orbelm.ra[sx] = 15.0*lsidtim((indata->orbelm.mjdep[sx]+dlam),azim,0);
  indata->orbelm.m0[sx] = -360.0*indata->orbelm.n0[sx] * indata->tmeco/1440.0;
  dlam = -0.45 * indata->orbelm.m0[sx] * degrad; // series expansion of anomaly
  azim = indata->orbelm.ecc[sx];
  dlam = dlam + (2.0 - 0.25*azim*azim) * azim * sin (dlam)
                            + 1.25*azim*azim*sin(2.0*dlam)
                            + 1.08333*azim*azim*azim*sin(3.0*dlam);

  indata->orbelm.aper[sx] = (c + dlam) / degrad;
  if (indata->orbelm.ra[sx] < 0) indata->orbelm.ra[sx] += 360.0;
  if (indata->orbelm.ra[sx] > 360) indata->orbelm.ra[sx] -= 360.0;
  if (indata->tmeco > 12.0)
        indata->orbelm.m0[sx] = -360.0*indata->orbelm.n0[sx] * 12.0/1440.0;
  if (indata->orbelm.m0[sx] < 0) indata->orbelm.m0[sx] += 360.0;
  if (indata->orbelm.m0[sx] > 360) indata->orbelm.m0[sx] -= 360.0;
  if (indata->orbelm.aper[sx] < 0) indata->orbelm.aper[sx] += 360.0;
  if (indata->orbelm.aper[sx] > 360) indata->orbelm.aper[sx] -= 360.0;

  // adjust launch time and Right Ascension for rendezvous launch
  if (indata->rendzs >= 0)
   {
    nw = true;
    sx2 = indata->rendzt;
    ecc = indata->orbelm.ecc[sx2];
    a = indata->orbelm.a[sx2];
    if (ecc >= 1.0) ecc = 0.9999;
    n0 = mp2 * indata->orbelm.n0[sx2];
    if (a < 1.0) a = 1.0;  // avoid possible crashes later on
    aper = (re / a) / (1.0 - ecc*ecc);
    aper = 1.5 * j2 * aper*aper * n0;
    n0 = indata->orbelm.inc[sx2]*degrad;
    ran = -aper * cos(n0);
    dt = indata->orbelm.mjdep[sx] - indata->orbelm.mjdep[sx2];
    aper = (indata->orbelm.ra[sx] - indata->orbelm.ra[sx2]) * degrad;
    n0 = 6.3003888;  // radians per sidereal day
    aper = aper - ran*dt;
    c = atan20 (sin(aper), cos(aper));  // normalize
    dt = c / n0;
    // correct for difference in orbit heights
    ecc = indata->orbelm.ecc[sx];
    a = indata->orbelm.a[sx];
    if (ecc >= 1.0) ecc = 0.9999;

    n0 = mp2 * indata->orbelm.n0[sx];
    if (a < 1.0) a = 1.0;  // avoid possible crashes later on
    aper = (re / a) / (1.0 - ecc*ecc);
    aper = 1.5 * j2 * aper*aper * n0;
    n0 = indata->orbelm.inc[sx]*degrad;
    dlam = -aper * cos(n0);
    aper = indata->tmeco;
    if (indata->rendtim != 0.0)
     {
      if (aper > 12.0) aper = 12.0;
      dt = dt - (dlam - ran) * indata->rendtim / 6.3003888 - 0.45*aper/1440.0;
     };
    indata->orbelm.mjdep[sx] -= dt;
    aper = indata->tmeco;
    if (aper > 12.0) aper = 12.0;
    indata->orbelm.ra[sx] = 15.0*lsidtim((indata->orbelm.mjdep[sx]+0.45*aper/1440.0),razim,0);
    if (indata->orbelm.ra[sx] < 0) indata->orbelm.ra[sx] += 360.0;
    if (indata->orbelm.ra[sx] > 360) indata->orbelm.ra[sx] -= 360.0;
    indata->lastmjd = indata->orbelm.mjdep[sx];
   };

  // copy to num-set and change to mean elements if needed
  if ((indata->orbelm.ecc[sx] < 1.0) && (indata->orbelm.aflg[sx]))
   {
    indata->orbelm.aflg[sx] = 0;
    EpochState (indata, sx, nw);
    r1[0] = indata->orbelm.x[sx];
    r1[1] = indata->orbelm.y[sx];
    r1[2] = indata->orbelm.z[sx];
    v1[0] = indata->orbelm.vx[sx];
    v1[1] = indata->orbelm.vy[sx];
    v1[2] = indata->orbelm.vz[sx];
    rv2el(r1, v1, indata->orbelm.inc[sx], indata->orbelm.ra[sx], indata->orbelm.ecc[sx],
                  indata->orbelm.aper[sx], indata->orbelm.m0[sx], indata->orbelm.n0[sx]);
    indata->orbelm.a[sx] = getsax (1, indata->orbelm.n0[sx], indata->orbelm.ecc[sx],
                                      indata->orbelm.inc[sx]);
    indata->orbelm.aflg[sx] = 1;
   }
  indata->lastsx = 9999;  // reset sx marker for SGP4 initialization
  EpochState (indata, sx, nw);
  indata->orbelm.numflg[sx] = nmfsav;
 }

/*------------------------- OrbitDet ---------------------------------*/

int PhiDiff (double g1, double g2, double psi1, double psi2, double grav,
             Vec3& r1, Vec3& r2, Vec3& p1, Vec3& p2, Vec3& ru1, Vec3& ru2,
             double rabs)
 {
  /* Auxilliary function for OrbitDet.
     Calculates the difference of central angle between the
     two radius vectors r1 and r2 according to Kepler's law
     and according to the "forced" geometry.

     RETURN: -1 if negative, 1 if positive
  */
  int res;
  double rho1, rho2;
  Vec3 c1;

  rho1 = g1 + rabs*rabs;
  if (rho1 > 0) rho1 = psi1 + sqrt(rho1);
  else rho1 = psi1;
  r1 = p1 + rho1*ru1;
  rho2 = g2 + rabs*rabs;
  if (rho2 > 0) rho2 = psi2 + sqrt(rho2);
  else rho2 = psi2;
  r2 = p2 + rho2*ru2;

  // get inclination
  c1 = r1 * r2;
  c1 = vnorm (ru1);
  if (fabs(c1[2]) > 1.0) c1[2] = 0;
  c1[2] = acos(c1[2]);  // inclination

  // compare via Kepler
  rho1 = dot(r1, r2) / (rabs*rabs);
  if (rho1 < -1.0) rho1 = -1.0;
  if (rho1 > 1.0) rho1 = 1.0;
  rho1 = acos(rho1);     // geometric value
  rho2 = getn0(rabs, 0, c1[2]) * 2.0*M_PI * grav;
  if (rho2 > rho1) res = 1;
  else res = -1;

  return res;
 }


void OrbitDet (WinpData* indata)
 {
  // circular orbit determination from two observations
  const double re = 6378.137;  // Earth equatorial radius

  int sx, nit, nold;
  double az1, el1, az2, el2;
  double g1, g2, psi1, psi2;
  double grav, rabs, rlow, rup, dt, tut1;
  Vec3 p1, p2, ru1, ru2, r1, r2;
  Mat3 mx, pmx;

  rabs = 0;

  tut1 = indata->lastmjd;
  if (indata->IERSflg) tut1 += indata->Ut1Utc/86400.0;

  // precession matrix epoch -> true of date
  if (indata->yrepoch != 0)
   {
    rup = (indata->yrepoch - 2000.0) / 100.0;
    dt = julcent (indata->lastmjd);
    pmx = pmatequ (rup, dt);
    mx = nutmat(dt, rup, indata->IERSflg);  // nutation
    pmx = mx * pmx;
   };

  if (indata->sidxcn < GBL_MXSAT) indata->sidxcn++;
  if (indata->sidxcn > 0) indata->satidx = indata->sidxcn - 1;
  else indata->satidx = 0;
  sx = indata->satidx;
  indata->orbelm.mjdep[sx] = indata->lastmjd;
  indata->orbelm.numflg[sx] = 0;

  if (indata->azra1flg)   // get the unit vectors along the slant range
   {
    az1 = 15.0*DmsDegF(indata->az1)*degrad;
    el1 = DmsDegF(indata->el1)*degrad;
    nit = 1;
   }
  else
   {
    AppRADec (tut1, 0, indata->geolat*degrad,
                       indata->geolong*degrad, indata->az1*degrad,
                       indata->el1*degrad, az1, el1);
    nit = 0;
   };
  ru1[0] = 1.0;
  ru1[1] = az1;
  ru1[2] = el1;
  ru1 = polcar(ru1);

  if (nit && (indata->yrepoch != 0)) ru1 = mxvct (pmx, ru1);
  if (indata->azra2flg)
   {
    az2 = 15.0*DmsDegF(indata->az2)*degrad;
    el2 = DmsDegF(indata->el2)*degrad;
    nit = 1;
   }
  else
   {
    AppRADec ((tut1+indata->deltm/86400.0), 0, indata->geolat*degrad,
                            indata->geolong*degrad, indata->az2*degrad,
                            indata->el2*degrad, az2, el2);
    nit = 0;
   };
  ru2[0] = 1.0;
  ru2[1] = az2;
  ru2[2] = el2;
  ru2 = polcar(ru2);
  if (nit && (indata->yrepoch != 0)) ru2 = mxvct (pmx, ru2);

  // get geographic position vectors and elongation
  p1 = GeoPos(tut1, 0, indata->geolat*degrad,indata->geolong*degrad, indata->geoheight)*re;
  p2 = GeoPos((tut1+indata->deltm/86400.0), 0, indata->geolat*degrad,
                                            indata->geolong*degrad, indata->geoheight)*re;
  g1 = abs(p1);
  psi1 = -dot(ru1, p1);
  g1 = psi1*psi1 - g1*g1;
  g2 = abs(p2);
  psi2 = -dot(ru2, p2);
  g2 = psi2*psi2 - g2*g2;
  grav = indata->deltm/86400.0;

  // find bracket of solution to the root of PhiDiff
  rup = 6478.14;
  nold = PhiDiff (g1, g2, psi1, psi2, grav, r1, r2, p1, p2, ru1, ru2, rup);
  do
   {
    rup += 100.0;
   } while ((nold == PhiDiff(g1,g2,psi1,psi2,grav,r1,r2,p1,p2,ru1,ru2,rup))
                         && (rup < 3e5));
  rlow = rup - 100.0;

  nit = 0;
  do    // find the root with the half intervall method
   {
    if (fabs(rup - rlow) < 1e-3) nit = 999;
    else
     {
      rabs =  (rlow + rup) / 2.0;
      if (nold == PhiDiff(g1,g2,psi1,psi2,grav,r1,r2,p1,p2,ru1,ru2,rabs))
                     rlow = rabs;
      else rup = rabs;
      nit++;
     };
   } while (nit < 300);
  if (nit != 999) strcpy(indata->orbelm.fsname[sx],"No Convergence");
  else strcpy(indata->orbelm.fsname[sx],"New");

  // convert vectors into Kepler elements
  indata->orbelm.m0[sx] = 0;   // by definition
  indata->orbelm.ecc[sx] = 0;  // by definition (circular orbit)
  indata->orbelm.ndot[sx] = 0;
  indata->orbelm.adot[sx] = 0;  // we don't know any better
  indata->orbelm.n6dot[sx] = 0;
  indata->orbelm.bstar[sx] = 0;
  indata->orbelm.aflg[sx] = 0;
  indata->orbelm.a[sx] = rabs;

  // convert into Mean-of-Date
  mx = mxtrn(mx);
  r1 = mxvct(mx, r1);
  r2 = mxvct(mx, r2);
  ru1 = r1 * r2;
  ru1 = vnorm (ru1);

  if (fabs(ru1[2]) > 1.0) grav = 0;
  else grav = acos(ru1[2]);  // inclination
  indata->orbelm.inc[sx] = grav/degrad;
  indata->orbelm.n0[sx]=getn0(indata->orbelm.a[sx],0,indata->orbelm.inc[sx]);
  g1 = atan20(ru1[0],-ru1[1]);  // R.A.
  indata->orbelm.ra[sx] = g1 / degrad;
  ru1 = vnorm (r1);    // now get the Argument of Perigee
  g2 = ru1[0]*cos(g1) + ru1[1]*sin(g1);
  grav = sin(grav);
  if (grav != 0) g1 = ru1[2]/grav;
  else g1 = 0;
  indata->orbelm.aper[sx] = atan20(g1, g2) / degrad;

  // copy to num-set
  indata->lastsx = 9999;  // reset sx marker for SGP4 initialization

  EpochState (indata, sx, true);
 }

/*------------------------- SatMatch ---------------------------------*/

int mtchok(double dst, double azt, double elt, double az1, double el1,
           double cntdir, int drctflg, double az2, double el2)
 {
  /* Auxilliary routine to check whether the conditions for the
     satellite match are met (return 1) or not (return 0);
     dst = cosine of angular distance,
     azt, elt = target azimuth and elevation
     cntdir = target direction
     az1, el1 = satellite azimuth and elevation
     drctflg = 1 if direction also to be considered
     az2, el2 = satellite azimuth and elevation dt after (az1, el1)
     all angles are in radians
   */
  int mtok;
  double d, s;
  Vec3 v1, v2;
  Mat3 mx;

  // check angular distance
  d = sin(elt)*sin(el1) + cos(elt)*cos(el1)*cos(azt - az1);
  if (d > dst) mtok = 1;
  else mtok = 0;

  // now check for direction
  if (mtok && drctflg)
   {
    d = sin(el2)*sin(el1) + cos(el2)*cos(el1)*cos(az2 - az1);
    s = 1.0 - d*d;
    if (s > 0) s = sqrt(s);
    else s = 0;
    s = s * cos(el1);
    if (fabs(s) > 1e-50)
     {
      d = (sin(el2) - sin(el1)*d) / s;
      if (fabs(d) <= 1.0)
       {
        d = acos(d);
        if (sin(az2 - az1) < 0) d = -d;
        v1.assign(0, 0, 1.0);
        v2 = v1;
        mx = xrot (d);
        v1 = mxvct(mx, v1);
        mx = xrot (cntdir);
        v2 = mxvct(mx, v2);
        if (dot (v1, v2) < cos(M_PI/4.0)) mtok = 0;
       };
     };
   };

  return mtok;
 }

void convRDAE(WinpData* indata, double t, double ep2, 
              double& cntaz, double& cntel)
 {
  double dumdst, tut1;
  Vec3 r1;
  Mat3 mx;

  if (indata->sdra1flg)  // convert from R.A./Decl into Azimuth/Elevation
   {
    cntaz = 15.0*DmsDegF(indata->sdaz1)*degrad;
    cntel = DmsDegF(indata->sdel1)*degrad;
    r1[0] = 1.0;
    r1[1] = cntaz;
    r1[2] = cntel;
    r1= polcar(r1);
    tut1 = t;
    if (indata->IERSflg) tut1 += indata->Ut1Utc/86400.0;
        AppPos (tut1, ep2, indata->geolat*degrad, indata->geolong*degrad,
                           indata->geoheight, 0, r1, cntaz, cntel, dumdst);
   }
  else
   {
    cntaz = indata->sdaz1 * degrad;
    cntel = indata->sdel1 * degrad;
   };
 }

int SatMatch(WinpData* indata, WoutData* outdata)
 {
  // Try to match satellites from present TLE file with observations.
  const double re = 6378.137;  // Earth equatorial radius

  int sx, sg, satok, found, npass, maxpass, scount, odcnt, mok;
  int svrelsat, dflg, sidflg, svsidx, updflg, svsdra1flg;
  long int nsats;
  double t, tb, ts, ep2, dt, tut1;
  double ti[3], el[3], az[3];
  double cntaz, cntel, cntdst, cntdir, dumdst;
  double svsdaz1, svsdel1, svsddeltm;
  char ckld;
  char otxt[GBL_MAXCPL];
  char outbuf[81];
  char mtxt[501];
  Vec3 r1;
  Mat3 mx;

  FILE* satfle;

  // save launch dialog state
  svsdaz1 = indata->sdaz1; 
  svsdel1 = indata->sdel1;
  svsddeltm = indata->sddeltm; 
  svsdra1flg = indata->sdra1flg; 

  // set up search
  indata->lorbnorad = 0;
  indata->lorbmjdep = 0;
  indata->lorbrelsat = indata->relsat;
  outdata->curlines = 0;
  maxpass = GBL_MAXLIN;
  maxpass -= 10;
  maxpass /= 3;  // to avoid overflow of the windows buffer
  maxpass--;
  odcnt = 0;

  // open the TLE file
  satfle = fopen (indata->tlename,"r");
  if (satfle == NULL)
   {
    PutSpace(outdata);
    sprintf (outbuf, " *** Unable to open TLE file ***");
    PutNxtLn (outdata, outbuf);
    PutNxtLn (outdata, indata->tlename);
    PutSpace(outdata);
    sprintf (outbuf, " Select a valid TLE file first.");
    PutNxtLn (outdata, outbuf);
    PutSpace(outdata);
    return 1;
   }
  else
   {
    PutSpace(outdata);
    sprintf (outbuf, " Possible matches on TLE-file:");
    PutNxtLn (outdata, outbuf);
    sprintf (outbuf, " ");
    strcat (outbuf, indata->tlename);
    PutNxtLn (outdata, outbuf);
    PutSpace(outdata);
   };

  SatPosData *stpos = new SatPosData;

  // save current state
  sx = indata->sidxcn - 1;
  sg = GBL_MXSAT;
  svrelsat = indata->relsat;
  svsidx = indata->sidxcn;
  sidflg = 0;
  if (indata->sidxcn < 1)
   {
    indata->sidxcn = 1;
    sx = 0;
   }
  else
   {
    if (indata->sidxcn < GBL_MXSAT)
     {
      sx++;
      indata->sidxcn++;
     }
    else
     {
      ShiftElm (indata, GBL_MXSAT+1, sx);
      sidflg = 1;
     };
   };

  if(indata->odtmatch)  // set up special case for orb det match
   {
    indata->sdaz1 = indata->az1; 
    indata->sdel1 = indata->el1;
    dumdst = indata->deltm / 60.0;
    if (dumdst > indata->sddeltm) indata->sddeltm = dumdst;
    indata->sdra1flg = indata->azra1flg; 
   };

  indata->relsat = 0;
  // set the search area and time interval
  t = indata->lastmjd;
  ts = julcent (t);
  mx = nutmat(ts,ep2);   // nutation to get ep2
  ts = indata->sddeltm / 1440.0;  // convert into days
  tb = t - ts;  // start time
  ts += t;      // stop time

  convRDAE(indata, t, ep2, cntaz, cntel);

  cntdst = indata->sdmxang * degrad;
  if (indata->sdra1flg && (indata->sddeltm > 2.0)) updflg = 1;
  else updflg = 0;
  if (((cntel + cntdst) / degrad) > 89.1) dflg = 0;
  else dflg = indata->sddrctflg;  // no direction check if elevation too high
  if (indata->odtmatch) dflg = 0;  // no direction check if orbit det call
  cntdst = cos (cntdst);  // that's all we need to check the distance
  cntdir = indata->sddirt *degrad;
  if (cntdir > M_PI) cntdir -= 2.0*M_PI;

  // now search
  strcpy(indata->orbelm.sname[GBL_MXSAT], "*");
  found = tleread(indata, satfle, mtxt, 0);
  npass = 0;
  nsats = 0;  // number of satellites checked
  scount = 60;  // count of seconds for azimuth/elevation update

  while (found > 0)
   {
    ShiftElm (indata, sx, sg);
    nsats++;
    if ((indata->orbelm.n0[sx] < 2.0)
            && (indata->orbelm.ecc[sx] < 0.1)) dt = 1.0 / 1440.0;  // 1 min
    else dt = 1.0 / 86400.0;  // 1 sec time interval if lower orbit

    if(indata->odtmatch)
     {
      indata->sdaz1 = indata->az1; 
      indata->sdel1 = indata->el1;
      indata->sdra1flg = indata->azra1flg; 
      convRDAE(indata, t, ep2, cntaz, cntel);
      odcnt = 0;
     };

    t = tb;
    do            // check individual satellite
     {
      satok = nextpass(indata, sx, t, ts, ti, el, az);
      if (satok)
       {
        // reject small objects if wanted
        if (indata->sdmvis)
         {
          if (indata->orbelm.n0[sx] < 2.5) satok = 0;
          if (indata->orbelm.ldsgn[sx][9] != ' ') satok =  0;
          ckld = indata->orbelm.ldsgn[sx][8];
          if ((ckld != 'A') && (ckld != 'B') && (ckld != 'D') 
            && (ckld != 'D') && (ckld != 'E')) satok =  0;
         };
       };
      if (satok)
       {
        t = ti[0];
        do
         {
          if (updflg && (scount >= 60))  // refresh Azimuth/Elevation
           {
            scount = 0;
            cntaz = 15.0*DmsDegF(indata->sdaz1)*degrad;
            cntel = DmsDegF(indata->sdel1)*degrad;
            r1[0] = 1.0;
            r1[1] = cntaz;
            r1[2] = cntel;
            r1= polcar(r1);

	    tut1 = t;
            if (indata->IERSflg) tut1 += indata->Ut1Utc/86400.0;
                   AppPos (tut1, ep2, indata->geolat*degrad, indata->geolong*degrad,
                                      indata->geoheight, 0, r1, cntaz, cntel, dumdst);
            if (indata->sddrctflg)  // no direction check near zenith
             {
              if (((cntel + cntdst) / degrad) > 89.1) dflg = 0;
              else dflg = 1;
             };
           };

          GetSatPos (t, indata, stpos, sx, 0);
          az[1] = stpos->azim[sx] * degrad;
          el[1] = stpos->elev[sx] * degrad;
          if (indata->sddrctflg) GetSatPos ((t+dt), indata, stpos, sx, 0);
          mok = mtchok(cntdst, cntaz, cntel, az[1], el[1], cntdir, dflg,
                             stpos->azim[sx]*degrad, stpos->elev[sx]*degrad);
          if(indata->odtmatch && (mok != 0))
           {
            if (odcnt == 0)
             {
              indata->sdaz1 = indata->az2; 
              indata->sdel1 = indata->el2;
              indata->sdra1flg = indata->azra2flg; 
              convRDAE(indata, t, ep2, cntaz, cntel);
              odcnt = 1;
              mok = 0;
             }; 
           };
          if(mok)
           {
            satok = 0;   // output satellite match
            npass++;
            sprintf(otxt," NORAD#:%6i  Int.Launch#: ", indata->orbelm.norad[sx]);
            strcat(otxt, indata->idsgn);
            strcat(otxt, "  ");
            strcat(otxt, indata->orbelm.fsname[sx]);
            PutSpace(outdata);
            PutNxtLnT (outdata, otxt);

            az[0] = indata->orbelm.a[sx] * (1.0 + indata->orbelm.ecc[sx]) - re;
            el[0] = indata->orbelm.a[sx] * (1.0 - indata->orbelm.ecc[sx]) - re;
            sprintf(otxt," Incl.: %8.4lf   Perigee: %10.3lf km  Apogee: %10.3lf km",
                                   indata->orbelm.inc[sx], el[0], az[0]);
            PutNxtLnT (outdata, otxt);

	    if (!sidflg)  // store found sat in case if there is enough room
             {
              ShiftElm(indata, GBL_MXSAT+1, sx);
              svsidx = indata->sidxcn;
              if (indata->sidxcn < GBL_MXSAT)
               {
                sx++;
                indata->sidxcn++;
               }
              else sidflg = 1;
             };
           }
          else
           {
            t += dt;
            if (updflg)
             {
              if (dt > 1.5e-5) scount = 60; // update every minute
              else scount++;
             };
            if (t > ti[2]) satok = 0;
           };
         } while (satok);

        if (satok)  // try the next pass
         {
          strcat(otxt, indata->orbelm.fsname[sx]);
          t = ti[2] + 1.0/1440.0;   // add one minute
          if (indata->orbelm.n0[sx] > 14.0) t += 1.0/24.0;  // add one hour
         };
       };

      if (t > ts) satok = 0;
     } while (satok);  // individual sat check

    if (npass > maxpass) found = 0;  // output buffer full
    if (found > 1) found = tleread(indata, satfle, mtxt, 1);
    else found = 0;
   };  // while found

  if (satfle != NULL) fclose(satfle);

  if (npass < 1)
   {
    outdata->curlines = 0;
    PutSpace(outdata);
    sprintf (otxt, " *** No matches found on TLE file ***");
    PutNxtLn (outdata, otxt);
    PutNxtLn (outdata, indata->tlename);
    PutSpace(outdata);
   };

  PutSpace(outdata);
  PutSpace(outdata);
  sprintf(otxt, " Number of satellites checked: %ld", nsats);
  PutNxtLn (outdata, otxt);
  sprintf(otxt, " Number of possible matches  : %i", npass);
  PutNxtLn (outdata, otxt);
  if (npass > maxpass)
   {
    sprintf (otxt, " *** Output buffer full. More matches possible ***");
    PutNxtLn (outdata, otxt);
   };

  delete stpos;

  // restore state
  indata->relsat = svrelsat;
  if (sidflg) ShiftElm (indata, sx, GBL_MXSAT+1);
  indata->sidxcn = svsidx;
  // restore launch dialog state
  indata->sdaz1 = svsdaz1; 
  indata->sdel1 = svsdel1;
  indata->sddeltm = svsddeltm; 
  indata->sdra1flg = svsdra1flg; 

  return 0;
 }

/*------------------------- Sgp4  ---------------------------------*/

void Sgp4 (WinpData* indata, double dt, int sx, Vec3& r1, Vec3& v1)
 {
  /* NORAD Orbit Model SGP4 for time dt (in days from epoch) for
     satellite sx and orbit elements indata->orbelm.
     The result will be the osculating state vector r1, v1
  */

  int res;
  double epoch, xn0, xincl, e0, bstar, omega0, xm0, xnode0;
  double tsince;
  double rspxv[3], vspxv[3];
  bool usesdp4;
  gravconsttype whichconst;

  // set orbit elements
  tsince = dt * 1440.0;
  if (indata->wgs84 == 0) whichconst = wgs72;
  else whichconst = wgs84;

  if (sx != indata->lastsx)
   {
    xn0 = 2.0*M_PI*indata->orbelm.n0[sx] / 1440.0;
    if (xn0 == 0) xn0 = 1-0e-8;
    xincl = indata->orbelm.inc[sx]*degrad;
    e0 = indata->orbelm.ecc[sx];
    if (e0 < 1.0e-8) e0 = 1.0e-8;
    bstar = indata->orbelm.bstar[sx];
    omega0 = indata->orbelm.aper[sx]*degrad;
    xm0 = indata->orbelm.m0[sx]*degrad;
    xnode0 = indata->orbelm.ra[sx]*degrad;
    epoch = indata->orbelm.mjdep[sx] - 33281.0;
    usesdp4 = indata->usesdp4;

    res = sgp4init ( whichconst, usesdp4, epoch, bstar, e0, omega0,
                     xincl, xm0, xn0, xnode0, satrec );

    if (res == 0) indata->lastsx = sx;
   };

  res = sgp4 (whichconst, satrec, tsince, rspxv, vspxv);

  if (res == 0)   // no errors encountered
   {
    r1[0] = rspxv[0];
    r1[1] = rspxv[1];
    r1[2] = rspxv[2];
    v1[0] = vspxv[0];
    v1[1] = vspxv[1];
    v1[2] = vspxv[2];
   }
  else  // put in some values to avoid a crash
   {
    r1[0] = 100.0;
    r1[1] = 0.0;
    r1[2] = 0.0;
    v1[0] = 0.0;
    v1[1] = 10.0;
    v1[2] = 0.0;
   };

  return;
 };

void rv2el(Vec3& r1, Vec3& v1, double& xincl, double& xnodeo,
                     double& eo, double& omegao, double& xmo, double& xno)
{
 /* get mean SGP4 elements from the state vector r1, v1 (in m and m/s)

    The first section calculates the six classical osculating orbital elements:
    ik  inclination
    ok  node
    ek  eccentricity
    wk  perigee
    mk  anomaly
    xn  motion

    In addition, some other osculating quantities are found:
    rk       topocentric distance
    uk       longitude
    rdotk    vv2 component parallel to rr2
    rfdotk   vv2 component perpendicular to rr2
    aodp     semi major axis
    pl       semi parameter

  adapted from a program by Scott Campbell

 */

 const double xj3 = -2.53881E-6;
 const double ck2 = 5.413079E-4;
 const double xke = 0.074366916133173408;
 const double twopi = 2.0 * M_PI;
 const double re = 6378135.0;  // Earth radius
 const double de2ra = 180.0 / M_PI;

 double xinck, xnodek, ek, wk, xn, rk, uk;
 double aodp, pl, rdotk, rfdotk, t;
 double cosio, sinio, sin2u, cos2u, temp1, temp2;
 double rdot, rfdot, theta2, betal, x3thm1, x1mth2, x7thm1;
 double esine, ecose, elsq, cosepw, sinepw, axn, ayn;
 double cosu, sinu, capu, a3ovk2, xlcof, aycof, aynl, xll;
 double xl, a0, a1, a2, d0, d1, beta, beta2, r, u;
 int i;
 Vec3 rr2, vv2, h, n, vec, vk, vz, vy;

 rr2 = r1 / re;  // change to Earth radii and Earth radii per minute
 vv2 = v1 * 60.0 / re;
 vk = vv2 / xke;
 h = rr2 * vk;
 pl = dot(h, h);
 if (pl < 1e-20) pl = 1e-20; // just in case
 vz.assign(0.0, 0.0, 1.0);
 n = vz * h;
 rk = abs(rr2);
 if (rk < 1e-20) rk = 1e-20;
 rdotk = dot(rr2, vv2) / rk;
 rfdotk = abs(h) * xke / rk;
 rr2 = vnorm(rr2);
 n = vnorm(n);
 uk = dot(rr2,n);
 if (fabs(uk) > 1.0) uk = 1.0;  // just in case
 uk = acos(uk);
 if(rr2[2] < 0) uk = twopi - uk;

 vz = vk * h;
 vy = -1.0 * rr2;
 vec = vz + vy;
 ek = abs(vec);
 t = abs(h);
 if (t < 1e-20) t = 1e-20;

 xinck = h[2] / t;
 if (fabs(xinck) > 1.0) xinck = 1.0;  // just in case
 xinck = acos(xinck);
 t = abs(n);
 if (t < 1e-20) t = 1e-20;
 xnodek = n[0] / t;
 if (fabs(xnodek) > 1.0) xnodek = 1.0; // just in case

 xnodek = acos(xnodek);
 if(n[1] < 0)
  {
   wk = twopi - xnodek;
   xnodek = fmod(wk, twopi);
   if (wk < 0) xnodek = twopi + xnodek;
  };
 vec = vnorm(vec);
 wk = dot(vec, n);
 if (fabs(wk) > 1.0) wk = 1.0;  // just in case
 wk = acos(wk);
 if(vec[2] < 0)
  {
   xn = twopi - wk;
   wk = fmod(xn, twopi);
   if (xn < 0) wk = twopi + wk;
  };
 aodp = 1.0 - ek*ek;
 if (aodp < 1e-20) aodp = 1e-20; // just in case
 aodp = pl / aodp;
 xn = xke * pow(aodp, -1.5);

/*
In the first loop the osculating elements rk, uk, xnodek, xinck,
rdotk, and rfdotk are used as anchors to find the corresponding final
SGP4 mean elements r, u, xnodeo, xincl, rdot, and rfdot.
Several other final mean values based on these are also found:
betal, cosio, sinio, theta2, cos2u, sin2u, x3thm1, x7thm1, x1mth2.
In addition, the osculating values initially held by aodp, pl, and xn
are replaced by intermediate (not osculating and not mean) values used by SGP4.
The loop converges on the value of pl in about four iterations.
*/

 //  seed value for first loop
 xincl = xinck;
 u = uk;

 for (i = 0; i < 40; ++i)
  {
   a2 = pl;
   betal = sqrt(pl / aodp);
   temp1 = ck2  / pl;
   temp2 = temp1 / pl;
   cosio = cos(xincl);
   sinio = sin(xincl);
   sin2u = sin(2.0*u);
   cos2u = cos(2.0*u);
   theta2 = cosio * cosio;
   x3thm1 = 3.0 * theta2 - 1.0;
   x1mth2 = 1.0 - theta2;
   x7thm1 = 7.0 * theta2 - 1.0;
   r = 1.0 - 1.5 * temp2 * betal * x3thm1;
   if (fabs(r) < 1e-20) r = 1e-20;
   r = (rk - 0.5 * temp1 * x1mth2 * cos2u) / r;
   if (fabs(r) < 1e-20) r = 1e-20;
   u = uk + 0.25 * temp2 * x7thm1 * sin2u;
   xnodeo = xnodek - 1.5 * temp2 * cosio * sin2u;
   xincl = xinck - 1.5 * temp2 * cosio * sinio * cos2u;
   rdot = rdotk + xn * temp1 * x1mth2 * sin2u;
   rfdot = rfdotk - xn * temp1 * (x1mth2 * cos2u + 1.5 * x3thm1);
   t = r * rfdot / xke;
   pl = t * t;
   if (pl < 1e-20) pl = 1e-20; // just in case

   // vis-viva equation
   aodp = 2.0/r - (rdot*rdot + rfdot*rfdot)/(xke*xke);
   if (aodp < 1e-20) aodp = 1e-20;
   aodp = 1.0 / aodp;
   xn = xke * pow(aodp, -1.5);
   if(fabs(a2 - pl) < 1.e-13) break;
  };

/*
The next values are calculated from constants and a combination
of mean and intermediate quantities from the first loop.
These values all remain fixed and are used in the second loop.
*/
 // preliminary values for the second loop
 ecose = 1.0 - r / aodp;
 esine = r * rdot / (xke * sqrt(aodp));   // needed for Kepler's eqn.
 elsq = 1. - pl / aodp;                   // intermediate eccentricity squared
 if (elsq < 0) elsq = 0.0;

 a3ovk2 = -xj3 / ck2;
 xlcof = 1.0 + cosio;
 if (fabs(xlcof) < 1e-20) xlcof = 1e-20; // just in case
 xlcof = 0.125 * a3ovk2 * sinio * (3. + 5. * cosio) / xlcof;
 aycof = 0.25 * a3ovk2 * sinio;
 temp1 = 1.0 - elsq;
 if (temp1 >= 0) temp1 = 1.0 + sqrt(temp1);
 else temp1 = 1.0;
 temp1 = esine / temp1;
 cosu = cos(u);
 sinu = sin(u);

/*
The second loop normally converges in about six iterations to the
final mean value for the eccentricity, eo.  The mean perigee, omegao,
is also determined.  Cosepw and sinepw are found to twelve decimal places
and are used to calculate an intermediate value for the eccentric anomaly,
temp2.  Temp2 is then used in Kepler's equation to find an intermediate
value for the true longitude, capu.
*/
 //  seed values for loop
 eo = sqrt(elsq);
 omegao = wk;
 axn = eo * cos(omegao);

 for (i = 0; i < 40; ++i)
  {
   a2 = eo;
   beta = 1.0 - eo*eo;
   if (beta < 1e-20) beta = 1e-20;
   t = 1.0 / (aodp * beta);
   aynl = t * aycof;
   ayn = eo * sin(omegao) + aynl;
   cosepw = r * cosu / aodp + axn - ayn * temp1;
   sinepw = r * sinu / aodp + ayn + axn * temp1;

   axn = cosepw * ecose + sinepw * esine;
   ayn = sinepw * ecose - cosepw * esine;
   xll = atan2((ayn - aynl), axn);
   omegao = fmod(xll, twopi);
   if (omegao < 0) omegao += twopi;
   xll = cos(omegao);
   if (fabs(xll) < 1e-20) xll = 1e-20;
   eo = axn / xll;
   if(fabs(a2 - eo) < 1.e-13) break;
  };

 temp2 = atan2(sinepw, cosepw);
 capu = temp2 - esine;      // Kepler's equation
 xll = t * xlcof * axn;

 // xll adjusts the intermediate true longitude,
 // capu, to the mean true longitude, xl
 xl = capu - xll;
 xmo = fmod((xl - omegao),twopi);
 if (xmo < 0) xmo += twopi;  // mean anomaly

/*
The third loop usually converges after three iterations to the mean
semi-major axis, a1, which is then used to find the mean motion, xno.
*/

 a0 = aodp;
 a1 = a0;
 beta2 = sqrt(beta);
 t = beta * beta2;
 if (t < 1e-20) t = 1e-20;  // just in case
 t = 1.5 * ck2 * x3thm1 / t;

 for (i = 0; i < 40; ++i)
  {
   a2 = a1;
   d0 = t / (a0*a0);
   a0 = aodp * (1.0 - d0);
   if (a0 < 1e-20) a0 = 1e-20;
   d1 = t / (a1*a1);
   a1 = 1.0 - d1/3.0 - d1*d1 - 134.0 * d1*d1*d1 / 81.0;
   a1 = a0 / a1;
   if (a1 < 1e-20) a1 = 1e-20;
   if(fabs(a2 - a1) < 1.e-13) break;
  };

 a1 = a1*re/1000.0;
 a1 = a1*a1*a1;
 xno = 398600.8 / a1;
 xno = sqrt(xno)*86400.0/twopi;
 xincl *= de2ra;
 xnodeo *= de2ra;
 omegao *= de2ra;
 xmo *= de2ra;
 eo = ek;
}

